/* Copyright (C) 2012-13 MINHAP, Gobierno de España
   This program is licensed and may be used, modified and redistributed under the terms
   of the European Public License (EUPL), either version 1.1 or (at your
   option) any later version as soon as they are approved by the European Commission.
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
   or implied. See the License for the specific language governing permissions and
   more details.
   You should have received a copy of the EUPL1.1 license
   along with this program; if not, you may find it at
   http://joinup.ec.europa.eu/software/page/eupl/licence-eupl */

package es.mpt.dsic.inside.model.converter;

import java.io.ByteArrayInputStream;
import java.io.FileOutputStream;
import java.util.List;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import es.mpt.dsic.infofirma.model.FirmaElectronica;
import es.mpt.dsic.infofirma.model.InfoFirmaElectronica;
import es.mpt.dsic.infofirma.model.InfoFirmante;
import es.mpt.dsic.infofirma.model.TipoContenidoFirmado;
import es.mpt.dsic.infofirma.service.InfoFirmaService;
import es.mpt.dsic.infofirma.service.exception.InfoFirmaServiceException;
import es.mpt.dsic.infofirma.service.exception.InfoFirmaServiceNoEsFirmaException;
import es.mpt.dsic.inside.model.converter.exception.InsideConverterConversionToInsideException;
import es.mpt.dsic.inside.model.converter.exception.InsideConverterDocumentoEniToInsideException;
import es.mpt.dsic.inside.model.converter.exception.InsideConverterDocumentoInsideException;
import es.mpt.dsic.inside.model.converter.exception.InsideConverterDocumentoInsideInfoException;
import es.mpt.dsic.inside.model.converter.exception.InsideConverterDocumentoInsideToDocumentoContenidoException;
import es.mpt.dsic.inside.model.converter.exception.InsideConverterDocumentoInsideToEniException;
import es.mpt.dsic.inside.model.converter.exception.InsideConverterException;
import es.mpt.dsic.inside.model.converter.exception.InsideConverterFirmasException;
import es.mpt.dsic.inside.model.converter.exception.InsideConverterPrimitivosToInsideException;
import es.mpt.dsic.inside.model.converter.exception.InsideConverterUtilsException;
import es.mpt.dsic.inside.model.converter.exception.InsideConverterVersionInsideException;
import es.mpt.dsic.inside.model.converter.exception.InsideConverterVisualizacionToInsideException;
import es.mpt.dsic.inside.model.objetos.ObjetoInside;
import es.mpt.dsic.inside.model.objetos.ObjetoInsideMetadatos;
import es.mpt.dsic.inside.model.objetos.documento.ObjetoDocumentoInside;
import es.mpt.dsic.inside.model.objetos.documento.ObjetoDocumentoInsideContenido;
import es.mpt.dsic.inside.model.objetos.expediente.ObjetoExpedienteInside;
import es.mpt.dsic.inside.model.objetos.firmas.FirmaInside;
import es.mpt.dsic.inside.model.objetos.firmas.FirmaInsideTipoFirmaEnum;
import es.mpt.dsic.inside.model.objetos.firmas.contenido.ContenidoFirmaCSVInside;
import es.mpt.dsic.inside.model.objetos.firmas.contenido.ContenidoFirmaCertificadoContenidoBinarioInside;
import es.mpt.dsic.inside.model.objetos.firmas.contenido.ContenidoFirmaCertificadoReferenciaInside;
import es.mpt.dsic.inside.xml.eni.documento.TipoDocumento;
import es.mpt.dsic.inside.xml.eni.documento.contenido.TipoContenido;
import es.mpt.dsic.inside.xml.eni.firma.Firmas;
import es.mpt.dsic.inside.xml.inside.TipoDocumentoInside;
import es.mpt.dsic.inside.xml.inside.TipoDocumentoInsideConMAdicionales;
import es.mpt.dsic.inside.xml.inside.TipoMetadatosAdicionales;
import es.mpt.dsic.inside.xml.inside.ws.documento.DocumentoInsideInfo;
import es.mpt.dsic.inside.xml.inside.ws.documento.alta.TipoDocumentoAltaInside;
import es.mpt.dsic.inside.xml.inside.ws.documento.conversion.TipoDocumentoConversionInside;
import es.mpt.dsic.inside.xml.inside.ws.visualizacion.documento.TipoDocumentoEniBinarioOTipo;
import es.mpt.dsic.inside.xml.inside.ws.visualizacion.documento.TipoDocumentoVisualizacionInside;


public abstract class InsideConverterDocumento {

	protected static final Log logger = LogFactory.getLog(InsideConverterDocumento.class);
	
	public static ObjetoDocumentoInside documentoEniAndMetadatosToDocumentoInside(TipoDocumento documentoEni,TipoMetadatosAdicionales metadatosAdicionales)
	//,InfoFirmaService infoFirmaService) 
	throws InsideConverterDocumentoEniToInsideException 
	{
		ObjetoDocumentoInside documentoInside = new ObjetoDocumentoInside();

		if(documentoEni.getContenido() == null){
			throw new InsideConverterDocumentoEniToInsideException("El contenido del documento no puede estar vacío",true);
		}

		documentoInside.setContenido(Contenido.contenidoEniToInside(documentoEni.getContenido()));
		try {			
			documentoInside.setFirmas(InsideConverterFirmas.firmasEniToInside(documentoEni.getFirmas()));
		} catch (InsideConverterFirmasException e) {
			throw new InsideConverterDocumentoEniToInsideException ("Error convirtiendo las firmas", e);
		}
		
		if(documentoEni.getMetadatos().getIdentificador() != null){
			documentoInside.setIdentificador(documentoEni.getMetadatos().getIdentificador());
		}
		
		documentoInside.setMetadatos(InsideConverterMetadatos.Documento.metadatosEniToInside(documentoEni.getMetadatos(),metadatosAdicionales));
		
		return documentoInside;
	}
	
	public static ObjetoDocumentoInsideContenido documentoInsideToObjetoDocumentoInsideContenido (ObjetoDocumentoInside documentoInside, InfoFirmaService infoFirmaService) throws InsideConverterDocumentoInsideToDocumentoContenidoException {
		ObjetoDocumentoInsideContenido documentoInsideContenido = new ObjetoDocumentoInsideContenido ();
		
		FirmaElectronica fe = new FirmaElectronica ();
		InfoFirmaElectronica infoFirma = null;
		boolean contenidoEnContenido = false;
		
		// Si el contenido binario del documento no es nulo, 
		if (documentoInside.getContenido().getValorBinario() != null) {
			// comprobamos si es una firma implícita.			
			fe.setFirma(documentoInside.getContenido().getValorBinario());	
			contenidoEnContenido = true;			
		
		} else {
			String idReferencia = documentoInside.getContenido().getReferencia().substring(1);
			
			int i=0;
			boolean found = false;			
			ContenidoFirmaCertificadoContenidoBinarioInside contenidoFirma = null;
			while (i < documentoInside.getFirmas().size() && !found) {
				if (idReferencia.equalsIgnoreCase(documentoInside.getFirmas().get(i).getIdentificadorEnDocumento())) {
					contenidoFirma = (ContenidoFirmaCertificadoContenidoBinarioInside) documentoInside.getFirmas().get(i).getContenidoFirma();
					found = true;					
				}
				i++;
			}
			
			if (!found) {
				throw new InsideConverterDocumentoInsideToDocumentoContenidoException ("No se ha encontrado la firma a la que hace referencia el documento.");
			}
			
			// Obtenemos el documento que se firmó			
			fe.setFirma(contenidoFirma.getValorBinario());
		}
		
		try {
			infoFirma = infoFirmaService.getInfoFirma(fe, null);
		} catch (InfoFirmaServiceNoEsFirmaException e) {
			// En este caso puede que el contenido del documento-e no sea una firma, sino que sea simplemente el contenido del documento
			if (contenidoEnContenido) {
				documentoInsideContenido.setValorBinario(documentoInside.getContenido().getValorBinario());
				documentoInsideContenido.setMime(documentoInside.getContenido().getMime());
				documentoInsideContenido.setNombreFormato(documentoInside.getContenido().getNombreFormato());			
			} else {
				throw new InsideConverterDocumentoInsideToDocumentoContenidoException ("El documento no es una firma electrónica", e);
			}
		} catch (InfoFirmaServiceException e) {
			throw new InsideConverterDocumentoInsideToDocumentoContenidoException ("Error en la llamada al servicio de info-firma", e);
		}
		
		// Si la llamada ha tenido éxito, significa que el contenido de la petición era una firma, por lo que extraemos información del documento firmado de la respuesta.
		if (infoFirma != null) {		
			if (infoFirma.getTipoContenidoFirmado().equals(TipoContenidoFirmado.TC_H)) {
				throw new InsideConverterDocumentoInsideToDocumentoContenidoException ("La firma es explícita. No se pueden obtener los bytes del documento");
			}
			
			documentoInsideContenido.setValorBinario(infoFirma.getContenidoFirmado());
			documentoInsideContenido.setMime(infoFirma.getMimeContenidoFirmado());
			try {
				documentoInsideContenido.setNombreFormato(InsideConverterUtils.getNombreFormatoByMime(infoFirma.getMimeContenidoFirmado()));
			} catch (InsideConverterUtilsException e) {
				throw new InsideConverterDocumentoInsideToDocumentoContenidoException ("Error al obtener el nombre del formato del contenido firmado.", e);
			}
		}
		
		return documentoInsideContenido;
	}
	
	public static ObjetoDocumentoInside documentoConversionToInside (TipoDocumentoConversionInside documentoConversion, InfoFirmaService infoFirmaService) throws InsideConverterConversionToInsideException{
		
		ObjetoDocumentoInside documentoInside = new ObjetoDocumentoInside();
		try {
			documentoInside.setMetadatos(InsideConverterMetadatos.Documento.metadatosConversionToInside(documentoConversion.getMetadatosEni()));
		} catch (InsideConverterPrimitivosToInsideException e) {
			throw new InsideConverterConversionToInsideException ("Error al convertir los metadatos del tipo Conversión a Inside", e);
		}
		documentoInside.setIdentificador(documentoConversion.getMetadatosEni().getIdentificador());
		
		try  {
			documentoInside = fillContenidoAndFirmasObjetoDocumentoInside (documentoInside,
															   documentoConversion.getContenido(),
															   documentoConversion.isFirmadoConCertificado(),
															   documentoConversion.getCsv() != null ? documentoConversion.getCsv().getValorCSV() : null,
															   documentoConversion.getCsv() != null ? documentoConversion.getCsv().getRegulacionCSV() : null,															   
															   infoFirmaService);
		} catch (InsideConverterPrimitivosToInsideException e) {
			throw new InsideConverterConversionToInsideException ("Error al convertir el documento contenido del tipo Conversión a Inside", e);
		}
		
		return documentoInside;
		
		
	}
	
	public static ObjetoDocumentoInside documentoAltaXmlToInside (TipoDocumentoAltaInside documentoAlta, InfoFirmaService infoFirmaService) 	
	throws InsideConverterDocumentoEniToInsideException 
	{

		ObjetoDocumentoInside documentoInside = new ObjetoDocumentoInside();

		documentoInside.setMetadatos(InsideConverterMetadatos.Documento.metadatosEniToInside(documentoAlta.getMetadatosEni(),documentoAlta.getMetadatosAdicionales()));
		
		try {
			
			documentoInside = fillContenidoAndFirmasObjetoDocumentoInside (documentoInside,
																   		   documentoAlta.getContenido(),
																   		   documentoAlta.isFirmadoConCertificado(),
																   		   documentoAlta.getCsv() != null ? documentoAlta.getCsv().getValorCSV() : null,
																   		   documentoAlta.getCsv() != null ? documentoAlta.getCsv().getRegulacionCSV() : null,																   		   														   
																   		   infoFirmaService);
		} catch (InsideConverterPrimitivosToInsideException e) {
			throw new InsideConverterDocumentoEniToInsideException("Error al convertir el documento primitivo del tipo DocumentoAlta a Inside", e);
		}

		
											  								 /*ObjetoDocumentoInsideContenido contenido = new ObjetoDocumentoInsideContenido();
		contenido.setIdentificadorEnDocumento("CONTENIDO_DOCUMENTO");

		// Si el documento está firmado obtenemos algunos datos de la firma
		if (documentoAlta.isFirmadoConCertificado()) {
			FirmaElectronica fe = new FirmaElectronica ();
			fe.setFirma(documentoAlta.getContenido());
			InfoFirmaElectronica infoFirma = null;
			try {
				infoFirma = infoFirmaService.getInfoFirma(fe, null);
			} catch (InfoFirmaServiceNoEsFirmaException e) {
				throw new InsideConverterDocumentoEniToInsideException ("No se ha detectado que el documento proporcionado sea una firma", true);
			} catch (InfoFirmaServiceException e) {
				throw new InsideConverterDocumentoEniToInsideException ("No se puede obtener el documento firmado " + e.getMessage(), e);
			}

			// Si la firma es explícita elevamos una excepción.
			if (infoFirma.getTipoContenidoFirmado() == TipoContenidoFirmado.TC_H) {
				throw new InsideConverterDocumentoEniToInsideException ("La firma es explícita, sólo se admiten firmas implícitas", true);
			}

			try {
				contenido.setNombreFormato(InsideConverterUtils.getNombreFormatoByMime(infoFirma.getMimeContenidoFirmado()));
			} catch (InsideConverterUtilsException e) {
				throw new InsideConverterDocumentoEniToInsideException ("No puedo obtener el nombre de formato a partir del contenido firmado", false);
			}

			List<InfoFirmante> firmantes = infoFirma.getFirmantes();

			// Comprobamos el tipo de firma, y dependiendo de esto formaremos el documento de una manera u otra
			// XADES DETACHED, ponemos la firma en el contenido y las firmas referenciarán a los nodos ds:Signature
			if (infoFirma.getTipoFirma().getTipoFirma().equalsIgnoreCase("XADES DETACHED") ||
					infoFirma.getTipoFirma().getTipoFirma().equalsIgnoreCase("XADES ENVELOPED")) {

				contenido.setValorBinario(documentoAlta.getContenido());
				contenido.setMime("application/xml");				

				for (InfoFirmante firmante : firmantes) {
					FirmaInside firmaInside = new FirmaInside ();
					if (infoFirma.getTipoFirma().getTipoFirma().equalsIgnoreCase("XADES DETACHED")) {
						firmaInside.setTipoFirma(FirmaInsideTipoFirmaEnum.TF_02);
					} else {
						firmaInside.setTipoFirma(FirmaInsideTipoFirmaEnum.TF_03);
					}					
					ContenidoFirmaCertificadoReferenciaInside contenidoFirma = new ContenidoFirmaCertificadoReferenciaInside();
					// TODO poenr las referencias de verdad
					contenidoFirma.setReferenciaFirma("#" + contenido.getIdentificadorEnDocumento());					
					firmaInside.setContenidoFirma(contenidoFirma);
					firmaInside.setOrden(documentoInside.getFirmas().size());
					firmaInside.setIdentificadorEnDocumento("FIRMA_" + firmaInside.getOrden());
					documentoInside.getFirmas().add(firmaInside);
				}

				// CADES, ponemos la firma en el primer nodo de las firmas, el contenido referenciará a éste.	
			} else if (infoFirma.getTipoFirma().getTipoFirma().equalsIgnoreCase("CADES")) {

				for (int i=0; i < firmantes.size(); i++) {
					FirmaInside firmaInside = new FirmaInside ();
					firmaInside.setTipoFirma(FirmaInsideTipoFirmaEnum.TF_05);					
					if (i == 0) {
						ContenidoFirmaCertificadoContenidoBinarioInside contenidoFirma = new ContenidoFirmaCertificadoContenidoBinarioInside ();
						contenidoFirma.setValorBinario(documentoAlta.getContenido());
						contenidoFirma.setMime("application/octect-stream");
						firmaInside.setContenidoFirma(contenidoFirma);						
					} else {
						ContenidoFirmaCertificadoReferenciaInside contenidoFirma = new ContenidoFirmaCertificadoReferenciaInside ();						
						contenidoFirma.setReferenciaFirma("#FIRMA_0");
						firmaInside.setContenidoFirma(contenidoFirma);						
					}
					firmaInside.setOrden(documentoInside.getFirmas().size());
					firmaInside.setIdentificadorEnDocumento("FIRMA_" + firmaInside.getOrden());
					documentoInside.getFirmas().add(firmaInside);
				}

				contenido.setReferencia("#" + documentoInside.getFirmas().get(0).getIdentificadorEnDocumento());


				// PADES, ponemos la firma en el contenido, y desde las firmas referenciamos a este contenido.
			} else if (infoFirma.getTipoFirma().getTipoFirma().equalsIgnoreCase("PADES")) {
				contenido.setValorBinario(documentoAlta.getContenido());
				contenido.setMime("application/pdf");		

				for (InfoFirmante firmante : firmantes) {
					FirmaInside firmaInside = new FirmaInside ();					
					firmaInside.setTipoFirma(FirmaInsideTipoFirmaEnum.TF_06);

					ContenidoFirmaCertificadoReferenciaInside contenidoFirma = new ContenidoFirmaCertificadoReferenciaInside();					
					contenidoFirma.setReferenciaFirma("#" + contenido.getIdentificadorEnDocumento());					
					firmaInside.setContenidoFirma(contenidoFirma);
					firmaInside.setOrden(documentoInside.getFirmas().size());
					firmaInside.setIdentificadorEnDocumento("FIRMA_" + firmaInside.getOrden());
					documentoInside.getFirmas().add(firmaInside);					
				}
			}

			// El documento no está firmado	
		} else {
			contenido.setValorBinario(documentoAlta.getContenido());
			try {
				contenido.setMime(InsideConverterUtils.getMimeType(documentoAlta.getContenido()));
			} catch (InsideConverterUtilsException e) {
				throw new InsideConverterDocumentoEniToInsideException("Error extrayendo el mimeType del contenido",e);
			}
			try {
				contenido.setNombreFormato(InsideConverterUtils.getNombreFormatoByMime(contenido.getMime()));
			} catch (InsideConverterUtilsException e) {
				throw new InsideConverterDocumentoEniToInsideException("Error extrayendo el nombre del formato desde el mimeType " + contenido.getMime(),e);
			}
		}

		if(documentoAlta.getCsv() != null){
			FirmaInside firmaCsv = new FirmaInside();
			firmaCsv.setTipoFirma(FirmaInsideTipoFirmaEnum.TF_01);
			ContenidoFirmaCSVInside contenidoFirmaCsv = new ContenidoFirmaCSVInside();
			contenidoFirmaCsv.setRegulacionGeneracionCSV(documentoAlta.getCsv().getRegulacionCSV());
			contenidoFirmaCsv.setValorCSV(documentoAlta.getCsv().getValorCSV());
			firmaCsv.setContenidoFirma(contenidoFirmaCsv);
			firmaCsv.setOrden(documentoInside.getFirmas().size());
			firmaCsv.setIdentificadorEnDocumento("FIRMA_" + firmaCsv.getOrden());
			documentoInside.getFirmas().add(firmaCsv);
		}

		documentoInside.setContenido(contenido);

		return documentoInside;*/
											  								 
		return documentoInside;									  								 

	}

	public static ObjetoDocumentoInside fillContenidoAndFirmasObjetoDocumentoInside (ObjetoDocumentoInside documentoInside,
																		     byte[] bytesContenido, 
																			 boolean firmadoConCertificado,
											  								 String csv,
											  								 String regulacionCSV,											  								 
											  								 InfoFirmaService infoFirmaService) throws InsideConverterPrimitivosToInsideException{
		ObjetoDocumentoInsideContenido contenido = new ObjetoDocumentoInsideContenido();
		contenido.setIdentificadorEnDocumento("CONTENIDO_DOCUMENTO");

		// Si el documento está firmado obtenemos algunos datos de la firma
		if (firmadoConCertificado) {
			FirmaElectronica fe = new FirmaElectronica ();
			fe.setFirma(bytesContenido);
			InfoFirmaElectronica infoFirma = null;
			try {
				infoFirma = infoFirmaService.getInfoFirma(fe, null);
			} catch (InfoFirmaServiceNoEsFirmaException e) {
				throw new InsideConverterPrimitivosToInsideException ("No se ha detectado que el documento proporcionado sea una firma", true);
			} catch (InfoFirmaServiceException e) {
				throw new InsideConverterPrimitivosToInsideException ("No se puede obtener el documento firmado " + e.getMessage(), e);
			}

			// Si la firma es explícita elevamos una excepción.
			if (infoFirma.getTipoContenidoFirmado() == TipoContenidoFirmado.TC_H) {
				throw new InsideConverterPrimitivosToInsideException ("La firma es explícita, sólo se admiten firmas implícitas", true);
			}

			try {
				contenido.setNombreFormato(InsideConverterUtils.getNombreFormatoByMime(infoFirma.getMimeContenidoFirmado()));
			} catch (InsideConverterUtilsException e) {
				throw new InsideConverterPrimitivosToInsideException ("No puedo obtener el nombre de formato a partir del contenido firmado", false);
			}

			List<InfoFirmante> firmantes = infoFirma.getFirmantes();

			// Comprobamos el tipo de firma, y dependiendo de esto formaremos el documento de una manera u otra
			// XADES DETACHED, ponemos la firma en el contenido y las firmas referenciarán a los nodos ds:Signature
			if (infoFirma.getTipoFirma().getTipoFirma().equalsIgnoreCase("XADES DETACHED") ||
					infoFirma.getTipoFirma().getTipoFirma().equalsIgnoreCase("XADES ENVELOPED")) {

				contenido.setValorBinario(bytesContenido);
				contenido.setMime("application/xml");				

				for (InfoFirmante firmante : firmantes) {
					FirmaInside firmaInside = new FirmaInside ();
					if (infoFirma.getTipoFirma().getTipoFirma().equalsIgnoreCase("XADES DETACHED")) {
						firmaInside.setTipoFirma(FirmaInsideTipoFirmaEnum.TF_02);
					} else {
						firmaInside.setTipoFirma(FirmaInsideTipoFirmaEnum.TF_03);
					}					
					ContenidoFirmaCertificadoReferenciaInside contenidoFirma = new ContenidoFirmaCertificadoReferenciaInside();
					// TODO poenr las referencias de verdad
					contenidoFirma.setReferenciaFirma("#" + contenido.getIdentificadorEnDocumento());					
					firmaInside.setContenidoFirma(contenidoFirma);
					firmaInside.setOrden(documentoInside.getFirmas().size());
					firmaInside.setIdentificadorEnDocumento("FIRMA_" + firmaInside.getOrden());
					documentoInside.getFirmas().add(firmaInside);
				}

				// CADES, ponemos la firma en el primer nodo de las firmas, el contenido referenciará a éste.	
			} else if (infoFirma.getTipoFirma().getTipoFirma().equalsIgnoreCase("CADES")) {

				for (int i=0; i < firmantes.size(); i++) {
					FirmaInside firmaInside = new FirmaInside ();
					firmaInside.setTipoFirma(FirmaInsideTipoFirmaEnum.TF_05);					
					if (i == 0) {
						ContenidoFirmaCertificadoContenidoBinarioInside contenidoFirma = new ContenidoFirmaCertificadoContenidoBinarioInside ();
						contenidoFirma.setValorBinario(bytesContenido);
						contenidoFirma.setMime("application/octect-stream");
						firmaInside.setContenidoFirma(contenidoFirma);						
					} else {
						ContenidoFirmaCertificadoReferenciaInside contenidoFirma = new ContenidoFirmaCertificadoReferenciaInside ();						
						contenidoFirma.setReferenciaFirma("#FIRMA_0");
						firmaInside.setContenidoFirma(contenidoFirma);						
					}
					firmaInside.setOrden(documentoInside.getFirmas().size());
					firmaInside.setIdentificadorEnDocumento("FIRMA_" + firmaInside.getOrden());
					documentoInside.getFirmas().add(firmaInside);
				}

				contenido.setReferencia("#" + documentoInside.getFirmas().get(0).getIdentificadorEnDocumento());


				// PADES, ponemos la firma en el contenido, y desde las firmas referenciamos a este contenido.
			} else if (infoFirma.getTipoFirma().getTipoFirma().equalsIgnoreCase("PADES")) {
				contenido.setValorBinario(bytesContenido);
				contenido.setMime("application/pdf");		

				for (InfoFirmante firmante : firmantes) {
					FirmaInside firmaInside = new FirmaInside ();					
					firmaInside.setTipoFirma(FirmaInsideTipoFirmaEnum.TF_06);

					ContenidoFirmaCertificadoReferenciaInside contenidoFirma = new ContenidoFirmaCertificadoReferenciaInside();					
					contenidoFirma.setReferenciaFirma("#" + contenido.getIdentificadorEnDocumento());					
					firmaInside.setContenidoFirma(contenidoFirma);
					firmaInside.setOrden(documentoInside.getFirmas().size());
					firmaInside.setIdentificadorEnDocumento("FIRMA_" + firmaInside.getOrden());
					documentoInside.getFirmas().add(firmaInside);					
				}
			}

			// El documento no está firmado	
		} else {
			contenido.setValorBinario(bytesContenido);
			try {
				contenido.setMime(InsideConverterUtils.getMimeType(bytesContenido));
			} catch (InsideConverterUtilsException e) {
				throw new InsideConverterPrimitivosToInsideException("Error extrayendo el mimeType del contenido",e);
			}
			try {
				contenido.setNombreFormato(InsideConverterUtils.getNombreFormatoByMime(contenido.getMime()));
			} catch (InsideConverterUtilsException e) {
				throw new InsideConverterPrimitivosToInsideException("Error extrayendo el nombre del formato desde el mimeType " + contenido.getMime(),e);
			}
		}

		if(csv != null){
			FirmaInside firmaCsv = new FirmaInside();
			firmaCsv.setTipoFirma(FirmaInsideTipoFirmaEnum.TF_01);
			ContenidoFirmaCSVInside contenidoFirmaCsv = new ContenidoFirmaCSVInside();
			contenidoFirmaCsv.setRegulacionGeneracionCSV(regulacionCSV);
			contenidoFirmaCsv.setValorCSV(csv);
			firmaCsv.setContenidoFirma(contenidoFirmaCsv);
			firmaCsv.setOrden(documentoInside.getFirmas().size());
			firmaCsv.setIdentificadorEnDocumento("FIRMA_" + firmaCsv.getOrden());
			documentoInside.getFirmas().add(firmaCsv);
		}

		documentoInside.setContenido(contenido);

		return documentoInside;

	}
	
	
	public static DocumentoInsideInfo documentoInsideToDocumentoInfo(ObjetoDocumentoInside documento)
		throws InsideConverterDocumentoInsideInfoException 
	{
		logger.debug("Convirtiendo el ObjetoDocumentoInside a DocumentoInsideInfo");
		DocumentoInsideInfo documentoInfo = new DocumentoInsideInfo();
		documentoInfo.setIdentificador(documento.getIdentificador());
		try {
			documentoInfo.setVersionActual(InsideConverterVersion.versionInsideToXml(documento.getVersion()));
		} catch (InsideConverterVersionInsideException e) {
			throw new InsideConverterDocumentoInsideInfoException(e);
		}
		return documentoInfo;
	}
	
	public static TipoDocumentoInsideConMAdicionales documentoInsideToConMAdicionales (ObjetoDocumentoInside documentoInside) throws InsideConverterException {
		TipoDocumentoInsideConMAdicionales documentoConMAdicionales = new TipoDocumentoInsideConMAdicionales();
		documentoConMAdicionales.setDocumento(documentoInsideToEni (documentoInside));
		documentoConMAdicionales.setMetadatosAdicionales(InsideConverterMetadatos.metadatosAdicionalesInsideToXml(documentoInside.getMetadatos().getMetadatosAdicionales()));
		return documentoConMAdicionales;
	}

	public static TipoDocumentoInside documentoInsideToXml(ObjetoDocumentoInside documentoInside) throws InsideConverterDocumentoInsideException {
		TipoDocumentoInside documentoXml = new TipoDocumentoInside();
		try {
			documentoXml.setDocumentoENI(documentoInsideToEni(documentoInside));
			documentoXml.setInfo(documentoInsideToDocumentoInfo(documentoInside));
		} catch (InsideConverterException e) {
			throw new InsideConverterDocumentoInsideException(e);
		} 
		
		documentoXml.setMetadatosAdicionales(InsideConverterMetadatos.metadatosAdicionalesInsideToXml(documentoInside.getMetadatos().getMetadatosAdicionales()));
		return documentoXml;
	}

	public static TipoDocumento documentoInsideToEni(ObjetoDocumentoInside documentoInside) 
		throws InsideConverterDocumentoInsideToEniException 
	{
		TipoDocumento documentoEni = new TipoDocumento();
		TipoContenido contenidoEni = Contenido.contenidoInsideToEni(documentoInside.getContenido());
		documentoEni.setContenido(contenidoEni);
		
		Firmas firmasEni = new Firmas();
		
		// Documento Firmado
		if(documentoInside.getFirmas() != null) {			
			try {
				firmasEni = (InsideConverterFirmas.firmasInsideToEni(documentoInside.getFirmas()));
			} catch (InsideConverterFirmasException e) {
				throw new InsideConverterDocumentoInsideToEniException("Error convirtiendo firmas de Inside a Eni",e);
			}
		}
		
		documentoEni.setFirmas(firmasEni);
		documentoEni.setId("DOC_" + documentoInside.getIdentificador());
		documentoEni.setMetadatos(InsideConverterMetadatos.Documento.metadatosInsideToEni(documentoInside.getMetadatos()));
		return documentoEni;
	}
	
	public static String documentoInsideToString (ObjetoDocumentoInside documentoInside) {		
		StringBuffer sb = new StringBuffer ("ObjetoDocumentoInside:[");
		if (documentoInside == null) {
			sb.append("null");
		} else {
			sb.append(documentoInside.toString());
		}
		sb.append("]");
		return sb.toString();
	}

	public static class Contenido{

		public static ObjetoDocumentoInsideContenido contenidoEniToInside(TipoContenido contenidoEni) {
			
			ObjetoDocumentoInsideContenido contenidoInside = new ObjetoDocumentoInsideContenido();
			contenidoInside.setNombreFormato(contenidoEni.getNombreFormato());
			
			if (contenidoEni.getDatosXML() != null) {				
				//contenidoInside.setValorBinario((contenidoEni.getDatosXML() + "").getBytes());
				String s = InsideConverterUtils.objectXMLToString(contenidoEni.getDatosXML());
				contenidoInside.setValorBinario(s.getBytes());
				contenidoInside.setMime("application/xml");
				try {
					FileOutputStream fout = new FileOutputStream ("/home/rus/desarrollo/ficheros/guardado.xml");
					fout.write(contenidoInside.getValorBinario());
					fout.close();
				} catch (Exception e) {
					e.printStackTrace();
				}
			} else if (contenidoEni.getValorBinario() != null) {
				contenidoInside.setValorBinario((contenidoEni.getValorBinario()));
				contenidoInside.setMime(InsideConverterUtils.getMimeByNombreFormato(contenidoEni.getNombreFormato()));
			} else if (contenidoEni.getReferenciaFichero() != null) {
				contenidoInside.setReferencia(contenidoEni.getReferenciaFichero());
			}
			
			contenidoInside.setIdentificadorEnDocumento(contenidoEni.getId());
			return contenidoInside;
		}
		
		public static TipoContenido contenidoInsideToEni(ObjetoDocumentoInsideContenido contenidoInside) {
			TipoContenido contenido = new TipoContenido();
			if (contenidoInside.getValorBinario() != null) {				
				if (contenidoInside.getMime().contentEquals("text/xml") ||
						contenidoInside.getMime().contains("application/xml")) {				
					contenido.setDatosXML (new String(contenidoInside.getValorBinario()));					
				} else {					
					contenido.setValorBinario(contenidoInside.getValorBinario());
				} 
			} else if (contenidoInside.getReferencia() != null) {
				contenido.setReferenciaFichero(contenidoInside.getReferencia());
			}
			contenido.setId(contenidoInside.getIdentificadorEnDocumento());
			contenido.setNombreFormato(contenidoInside.getNombreFormato());

			return contenido;
		}
		
	}
	
	public static ObjetoDocumentoInside documentoVisualizacionToInside (TipoDocumentoVisualizacionInside documentoVisualizacion) throws InsideConverterVisualizacionToInsideException{
		ObjetoDocumentoInside documentoInside = null;
		TipoDocumentoEniBinarioOTipo docEniBinarioOTipo = documentoVisualizacion.getDocumentoEni();
		TipoDocumento docEni = null;
		// Si se recibe el documento Eni en base64 hacemos unmarshall para obtenerlo
		if (docEniBinarioOTipo.getDocumentoEniBinario() != null) {
			try {
				JAXBContext jc = JAXBContext.newInstance(TipoDocumento.class.getPackage().getName());			
				
				Unmarshaller unmarshaller = jc.createUnmarshaller();			//
				JAXBElement<?> e = (JAXBElement<?>) unmarshaller.unmarshal(new ByteArrayInputStream(docEniBinarioOTipo.getDocumentoEniBinario()));
				docEni = (TipoDocumento) e.getValue();
				
			} catch (JAXBException e) {
				throw new InsideConverterVisualizacionToInsideException ("No se ha podido construir el objeto mediante los bytes del documento Eni: " + e.getMessage(), true);
			}
		} else {
			docEni = docEniBinarioOTipo.getDocumentoEniTipo();
		}
		
		/*byte[] docEniBytes = documentoVisualizacion.getDocumentoEni();
		TipoDocumento docEni = null;
		ObjetoDocumentoInside documentoInside = null;
		try {
			JAXBContext jc = JAXBContext.newInstance(TipoDocumento.class.getPackage().getName());			
			
			Unmarshaller unmarshaller = jc.createUnmarshaller();			//
			JAXBElement<?> e = (JAXBElement<?>) unmarshaller.unmarshal(new ByteArrayInputStream(docEniBytes));
			docEni = (TipoDocumento) e.getValue();
			
		} catch (JAXBException e) {
			throw new InsideConverterVisualizacionToInsideException ("No se ha podido construir el objeto mediante los bytes del documento Eni: " + e.getMessage(), true);
		}*/
		
		TipoMetadatosAdicionales metadatosAdicionales = documentoVisualizacion.getMetadatosAdicionales();
		try {
			documentoInside = documentoEniAndMetadatosToDocumentoInside (docEni, metadatosAdicionales);
		} catch (InsideConverterDocumentoEniToInsideException e) {
			// No le metemos mensaje porque la excepción ya lo lleva implícito
			throw new InsideConverterVisualizacionToInsideException ("", e);
		}
		
		return documentoInside;
	}
	
	
/*	public static ObjetoDocumentoInside documentoAltaXmlToInside(TipoDocumentoAltaInside documentoAlta,InfoFirmaService infoFirmaService)
		throws InsideConverterDocumentoEniToInsideException
	{
		ObjetoDocumentoInside documentoInside = new ObjetoDocumentoInside();
		
		documentoInside.setMetadatos(InsideConverterMetadatos.Documento.metadatosEniToInside(documentoAlta.getMetadatosEni(),documentoAlta.getMetadatosAdicionales()));
		
		ObjetoDocumentoInsideContenido contenido = new ObjetoDocumentoInsideContenido();

		if(documentoAlta.isFirmadoConCertificado()){
			FirmaElectronica petInfoFirma = new FirmaElectronica();
			petInfoFirma.setFirma(documentoAlta.getContenido());
			String mimeContenidoFirmado = "";
			try {
				InfoFirmaElectronica infoFirmaResp = infoFirmaService.getInfoFirma(petInfoFirma);
				contenido.setValorBinario(infoFirmaResp.getContenidoFirmado());
				mimeContenidoFirmado = infoFirmaResp.getMimeContenidoFirmado();
				contenido.setMime(mimeContenidoFirmado);
				contenido.setNombreFormato(InsideConverterUtils.getNombreFormatoByMime(mimeContenidoFirmado));
				
				
				
				
				
				//TODO crear las firmas ... averiguar el tipo de firma etc ..
			} catch (InfoFirmaServiceException e) {
				throw new InsideConverterDocumentoEniToInsideException("Error obteniendo información de la firma",e);
			} catch (InsideConverterUtilsException e) {
				throw new InsideConverterDocumentoEniToInsideException("Error obteniendo el nombre de formato desde el mime " + mimeContenidoFirmado,e);
			}
		}else{
			contenido.setValorBinario(documentoAlta.getContenido());
			try {
				contenido.setMime(InsideConverterUtils.getMimeType(documentoAlta.getContenido()));
			} catch (InsideConverterUtilsException e) {
				throw new InsideConverterDocumentoEniToInsideException("Error extrayendo el mimeType del contenido",e);
			}
			try {
				contenido.setNombreFormato(InsideConverterUtils.getNombreFormatoByMime(contenido.getMime()));
			} catch (InsideConverterUtilsException e) {
				throw new InsideConverterDocumentoEniToInsideException("Error extrayendo el nombre del formato desde el mimeType " + contenido.getMime(),e);
			}
		}
		if(documentoAlta.getCsv() != null){
			FirmaInside firmaCsv = new FirmaInside();
			firmaCsv.setTipoFirma(FirmaInsideTipoFirmaEnum.TF_01);
			ContenidoFirmaCSVInside contenidoFirmaCsv = new ContenidoFirmaCSVInside();
			contenidoFirmaCsv.setRegulacionGeneracionCSV(documentoAlta.getCsv().getRegulacionCSV());
			contenidoFirmaCsv.setValorCSV(documentoAlta.getCsv().getValorCSV());
			firmaCsv.setContenidoFirma(contenidoFirmaCsv);
			documentoInside.getFirmas().add(firmaCsv);
		}
		
		
		documentoInside.setContenido(contenido);

		return documentoInside;
	}*/
	
	public <T extends ObjetoInside<K>, K extends ObjetoInsideMetadatos> int prueba (Class<T> clase) {
		if (clase.equals(ObjetoDocumentoInside.class)) {
			int i=0;
		}
		return 1;
		
	}

}
