/* Copyright (C) 2012-13 MINHAP, Gobierno de España
   This program is licensed and may be used, modified and redistributed under the terms
   of the European Public License (EUPL), either version 1.1 or (at your
   option) any later version as soon as they are approved by the European Commission.
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
   or implied. See the License for the specific language governing permissions and
   more details.
   You should have received a copy of the EUPL1.1 license
   along with this program; if not, you may find it at
   http://joinup.ec.europa.eu/software/page/eupl/licence-eupl */

package es.mpt.dsic.inside.model.converter;

import java.util.ArrayList;
import java.util.List;

import es.mpt.dsic.inside.model.converter.exception.InsideConverterEniToInsideException;
import es.mpt.dsic.inside.model.converter.exception.InsideConverterException;
import es.mpt.dsic.inside.model.converter.exception.InsideConverterExpedienteInsideToEniException;
import es.mpt.dsic.inside.model.objetos.expediente.ObjetoExpedienteInside;
import es.mpt.dsic.inside.model.objetos.firmas.FirmaInside;
import es.mpt.dsic.inside.xml.eni.expediente.TipoExpediente;
import es.mpt.dsic.inside.xml.eni.expediente.metadatos.TipoMetadatos;
import es.mpt.dsic.inside.xml.inside.TipoExpedienteInside;
import es.mpt.dsic.inside.xml.inside.TipoExpedienteInsideConMAdicionales;
import es.mpt.dsic.inside.xml.inside.TipoMetadatosAdicionales;
import es.mpt.dsic.inside.xml.inside.ws.expediente.conversion.TipoExpedienteConversionInside;

/**
 *
 * @author miguel.ortega
 *
 */
public abstract class InsideConverterExpediente {

	/**
	 * Crea un ObjetoExpedienteInside a partir de los metadatos de entrada
	 * @param metadatosEni
	 * @param metadatosAdicionales
	 * @return
	 */
	public static ObjetoExpedienteInside fromMetadatos(
			TipoMetadatos metadatosEni,
			TipoMetadatosAdicionales metadatosAdicionales) {

		ObjetoExpedienteInside expediente = new ObjetoExpedienteInside();
		
		if(metadatosEni.getIdentificador() != null){
			expediente.setIdentificador(metadatosEni.getIdentificador());
		}

		expediente.setMetadatos(InsideConverterMetadatos.Expediente.metadatosXmlToMetadatosInside(metadatosEni,metadatosAdicionales));
		
		return expediente;
		
	}

	/**
	 * Crea un ObjetoExpedienteInside a partir de un TipoExpediente(Eni) de entrada
	 * @param expedienteEni
	 * @return
	 * @throws InsideConverterEniToInsideException 
	 * @throws InsideWsRequestErrorException 
	 */
	public static ObjetoExpedienteInside expedienteEniToInside(TipoExpediente expedienteEni) 
			throws InsideConverterEniToInsideException 
	{
		ObjetoExpedienteInside expediente = new ObjetoExpedienteInside();
		if(expedienteEni.getMetadatosExp().getIdentificador() == null){
			throw new InsideConverterEniToInsideException("El identificador de los metadatos del expediente no puede estar vacio",true);
		}
		expediente.setIdentificador(expedienteEni.getMetadatosExp().getIdentificador());
		expediente.setIndice(InsideConverterIndice.indiceEniToInside(expedienteEni.getIndice()));
		
		if (expedienteEni.getVisualizacionIndice() != null) {
			expediente.setVisualizacionIndice(InsideConverterDocumento.Contenido.contenidoEniToInside(expedienteEni.getVisualizacionIndice()));
		}
		expediente.setMetadatos(InsideConverterMetadatos.Expediente.metadatosEniToMetadatosInside(expedienteEni.getMetadatosExp()));
		return expediente;
	}

	/**
	 * Crea un ObjetoExpedienteInside a partir de un TipoExpediente(Eni) de entrada y 
	 * un listado de metadatos adicionales
	 * @param expedienteEni
	 * @param metadatosAdicionales
	 * @return
	 * @throws InsideConverterEniToInsideException
	 * @throws InsideWsRequestErrorException 
	 */
	public static ObjetoExpedienteInside expedienteEniToInside(TipoExpediente expedienteEni,TipoMetadatosAdicionales metadatosAdicionales) 
		throws InsideConverterEniToInsideException 
	{
		ObjetoExpedienteInside expediente = expedienteEniToInside(expedienteEni);
		expediente.getMetadatos().setMetadatosAdicionales(InsideConverterMetadatos.metadatosAdicionalesXmlToInside(metadatosAdicionales));
		return expediente;
	}

	public static TipoExpediente expedienteInsideToEni(ObjetoExpedienteInside expediente) throws InsideConverterExpedienteInsideToEniException
	{
		TipoExpediente expedienteEni = new TipoExpediente();
		
		expedienteEni.setId(expediente.getIdentificador());
		try {
			expedienteEni.setIndice(InsideConverterIndice.indiceInsideToEni(expediente.getIndice(),expediente.getIdentificador()));
			expedienteEni.setMetadatosExp(InsideConverterMetadatos.Expediente.metadatosInsideToEni(expediente.getMetadatos()));
		} catch (InsideConverterException e) {
			throw new InsideConverterExpedienteInsideToEniException(e);
		}
		if (expediente.getVisualizacionIndice() != null) {
			expedienteEni.setVisualizacionIndice(InsideConverterDocumento.Contenido.contenidoInsideToEni(expediente.getVisualizacionIndice()));
		}
		return expedienteEni;
	}

	public static TipoExpedienteInside expedienteInsideToXml(ObjetoExpedienteInside expedienteInside) throws InsideConverterException
	{
		TipoExpedienteInside expedienteXml = new TipoExpedienteInside();
		expedienteXml.setExpedienteENI(expedienteInsideToEni(expedienteInside));
		expedienteXml.setMetadatosAdicionales(InsideConverterMetadatos.metadatosAdicionalesInsideToXml(expedienteInside.getMetadatos().getMetadatosAdicionales()));
		expedienteXml.setInfo(InsideConverterExpedienteInfo.fromExpediente(expedienteInside));
		return expedienteXml;
	}
	
	public static ObjetoExpedienteInside expedienteConversionToInside (TipoExpedienteConversionInside expedienteConversion) throws InsideConverterException {
		
		ObjetoExpedienteInside expedienteInside = new ObjetoExpedienteInside ();
		expedienteInside.setIdentificador(expedienteConversion.getMetadatosEni().getIdentificador());
		
		expedienteInside.setMetadatos(InsideConverterMetadatos.Expediente.metadatosPrimitivoToInside(expedienteConversion.getMetadatosEni().getClasificacion(),
																									 expedienteConversion.getMetadatosEni().getEstado().getValue().value(),
																									 expedienteConversion.getMetadatosEni().getFechaAperturaExpediente(),
																									 expedienteConversion.getMetadatosEni().getInteresado(),
																									 expedienteConversion.getMetadatosEni().getOrgano(),
																									 expedienteConversion.getMetadatosEni().getVersionNTI(),
																									 expedienteConversion.getMetadatosEni().getIdentificador()));
		expedienteInside.setIndice(InsideConverterIndice.indiceConversionToInside(expedienteConversion.getIndice()));
		expedienteInside.getIndice().setFirmas(new ArrayList<FirmaInside> ());
				
		return expedienteInside;
																										 
	}
	
	public static TipoExpedienteInsideConMAdicionales expedienteInsideToConMAdicionales (ObjetoExpedienteInside expedienteInside) throws InsideConverterException {
		TipoExpedienteInsideConMAdicionales expedienteConMAdicionales = new TipoExpedienteInsideConMAdicionales ();
		expedienteConMAdicionales.setExpediente(expedienteInsideToEni(expedienteInside));
		expedienteConMAdicionales.setMetadatosAdicionales(InsideConverterMetadatos.metadatosAdicionalesInsideToXml (expedienteInside.getMetadatos().getMetadatosAdicionales()));
		return expedienteConMAdicionales;
	}

	public static String expedienteInsideToString (ObjetoExpedienteInside expedienteInside) {		
		StringBuffer sb = new StringBuffer ("ObjetoExpedienteInside:[");
		if (expedienteInside == null) {
			sb.append("null");
		} else {
			sb.append(expedienteInside.toString());
		}
		sb.append("]");
		return sb.toString();
	}
	
	

}
