/* Copyright (C) 2012-13 MINHAP, Gobierno de España
   This program is licensed and may be used, modified and redistributed under the terms
   of the European Public License (EUPL), either version 1.1 or (at your
   option) any later version as soon as they are approved by the European Commission.
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
   or implied. See the License for the specific language governing permissions and
   more details.
   You should have received a copy of the EUPL1.1 license
   along with this program; if not, you may find it at
   http://joinup.ec.europa.eu/software/page/eupl/licence-eupl */

package es.mpt.dsic.inside.model.converter;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.xml.datatype.XMLGregorianCalendar;

import org.springframework.util.Assert;
import org.w3c.dom.Element;

import es.mpt.dsic.inside.model.converter.exception.InsideConverterCalendarToXmlCalendarException;
import es.mpt.dsic.inside.model.converter.exception.InsideConverterDocumentoMetadatosPrimitivosToEniException;
import es.mpt.dsic.inside.model.converter.exception.InsideConverterDocumentoMetatosInsideToEniException;
import es.mpt.dsic.inside.model.converter.exception.InsideConverterExpedienteMetadatosPrimitivosToInsideException;
import es.mpt.dsic.inside.model.converter.exception.InsideConverterMetadatosInsideToEniException;
import es.mpt.dsic.inside.model.converter.exception.InsideConverterPrimitivosToEniException;
import es.mpt.dsic.inside.model.converter.exception.InsideConverterPrimitivosToInsideException;
import es.mpt.dsic.inside.model.converter.exception.InsideConverterXmlGregorianCalendar;
import es.mpt.dsic.inside.model.converter.exception.InsideDocumentoMetadatosEniToInsideConverterException;
import es.mpt.dsic.inside.model.objetos.InsideMetadatosAdicionales;
import es.mpt.dsic.inside.model.objetos.ObjetoInsideMetadatoAdicional;
import es.mpt.dsic.inside.model.objetos.documento.metadatos.ObjetoDocumentoInsideMetadatos;
import es.mpt.dsic.inside.model.objetos.documento.metadatos.ObjetoDocumentoInsideMetadatosEstadoElaboracion;
import es.mpt.dsic.inside.model.objetos.documento.metadatos.ObjetoDocumentoInsideMetadatosEstadoElaboracionEnumeracion;
import es.mpt.dsic.inside.model.objetos.documento.metadatos.ObjetoDocumentoInsideMetadatosTipoDocumental;
import es.mpt.dsic.inside.model.objetos.expediente.metadatos.ObjetoExpedienteInsideMetadatos;
import es.mpt.dsic.inside.model.objetos.expediente.metadatos.ObjetoExpedienteInsideMetadatosEnumeracionEstados;
import es.mpt.dsic.inside.xml.eni.documento.metadatos.EnumeracionEstadoElaboracion;
import es.mpt.dsic.inside.xml.eni.documento.metadatos.TipoEstadoElaboracion;
import es.mpt.dsic.inside.xml.eni.documento.metadatos.TipoDocumental;
import es.mpt.dsic.inside.xml.eni.expediente.metadatos.EnumeracionEstados;
import es.mpt.dsic.inside.xml.eni.expediente.metadatos.TipoMetadatos;
import es.mpt.dsic.inside.xml.inside.DocumentoInsideMetadatos;
import es.mpt.dsic.inside.xml.inside.ExpedienteInsideMetadatos;
import es.mpt.dsic.inside.xml.inside.MetadatoAdicional;
import es.mpt.dsic.inside.xml.inside.ws.documento.alta.TipoDocumentoAltaInside.MetadatosEni;
//import es.mpt.dsic.inside.xml.inside.MetadatoValor;
import es.mpt.dsic.inside.xml.inside.TipoMetadatosAdicionales;

public abstract class InsideConverterMetadatos {
	
	public static class Expediente{
		
		/**
		 * Convierte los metadatos adicionales desde la entrada del WS 
		 * a un tipo válido para el InsideService
		 * @param metadatosEni
		 * @param metadatosAdicionales
		 * @return
		 */
		public static ObjetoExpedienteInsideMetadatos metadatosXmlToMetadatosInside(
				TipoMetadatos metadatosEni,
				TipoMetadatosAdicionales metadatosAdicionales) {
			Assert.notNull(metadatosEni);
			ObjetoExpedienteInsideMetadatos metadatos = metadatosEniToMetadatosInside(metadatosEni);
			metadatos.setMetadatosAdicionales(metadatosAdicionalesXmlToInside(metadatosAdicionales));
			
			
			return metadatos;
		}
		
			
		/**
		 * Convierte los metadatos del ENI a metadatos de Inside
		 * @param metadatosExp
		 * @return
		 */
		public static ObjetoExpedienteInsideMetadatos metadatosEniToMetadatosInside(TipoMetadatos metadatosExp)
		{
			Assert.notNull(metadatosExp);
			ObjetoExpedienteInsideMetadatos metadatos = new ObjetoExpedienteInsideMetadatos();
			metadatos.setClasificacion(metadatosExp.getClasificacion());
			if(metadatosExp.getEstado() != null && metadatosExp.getEstado().getValue() != null){
				metadatos.setEstado(ObjetoExpedienteInsideMetadatosEnumeracionEstados.fromValue(metadatosExp.getEstado().getValue().value()));
			}
			if(metadatosExp.getFechaAperturaExpediente() != null){
				metadatos.setFechaAperturaExpediente(metadatosExp.getFechaAperturaExpediente().toGregorianCalendar());
			}
			metadatos.setIdentificadorObjetoInside(metadatosExp.getIdentificador());
			metadatos.setVersionNTI(metadatosExp.getVersionNTI());
			metadatos.setOrgano(metadatosExp.getOrgano());
			metadatos.setInteresado(metadatosExp.getInteresado());			
			return metadatos;
		}
		
		/**
		 * Convierte todos los metadatos de un expediente al mensaje 
		 * metadatos del WS de Inside
		 * @param metadatos
		 * @return
		 * @throws InsideConverterMetadatosInsideToEniException 
		 * @throws InsideWSInternalErrorException 
		 */
		public static ExpedienteInsideMetadatos metadatosInsideToXml(ObjetoExpedienteInsideMetadatos metadatos) throws InsideConverterMetadatosInsideToEniException
		{
			Assert.notNull(metadatos);
			ExpedienteInsideMetadatos metadatosXml = new ExpedienteInsideMetadatos(); 
			metadatosXml.setMetadatosAdicionales(metadatosAdicionalesInsideToXml(metadatos.getMetadatosAdicionales()));
			metadatosXml.setMetadatosExpedienteENI(metadatosInsideToEni(metadatos));
			return metadatosXml;
		}

		/**
		 * Convierte los metadatos del Expediente en MetadatosEni
		 * @param metadatosInside
		 * @return
		 * @throws InsideConverterMetadatosInsideToEniException 
		 * @throws InsideWSInternalErrorException 
		 */
		public static TipoMetadatos metadatosInsideToEni(ObjetoExpedienteInsideMetadatos metadatosInside) throws InsideConverterMetadatosInsideToEniException
		{
			Assert.notNull(metadatosInside);
			TipoMetadatos metadatosEni = new TipoMetadatos();
			metadatosEni.setClasificacion(metadatosInside.getClasificacion());
			TipoMetadatos.Estado estado = new TipoMetadatos.Estado();
			estado.setValue(EnumeracionEstados.fromValue(metadatosInside.getEstado().value()));
			metadatosEni.setEstado(estado);
			try {
				metadatosEni.setFechaAperturaExpediente(InsideConverterXmlGregorianCalendar.calendarToXmlCalendar(metadatosInside.getFechaAperturaExpediente()));
			} catch (InsideConverterCalendarToXmlCalendarException e) {
				throw new InsideConverterMetadatosInsideToEniException(e);
			}
			metadatosEni.setId("EXP_" + metadatosInside.getIdentificadorObjetoInside() + "_METADATOS");
			metadatosEni.setIdentificador(metadatosInside.getIdentificadorObjetoInside());
			metadatosEni.setVersionNTI(metadatosInside.getVersionNTI());
			metadatosEni.getInteresado().addAll(metadatosInside.getInteresado());
			metadatosEni.getOrgano().addAll(metadatosInside.getOrgano());
			return metadatosEni;
		}
		
		public static ObjetoExpedienteInsideMetadatos metadatosPrimitivoToInside (String clasificacion,
				String estado,
				Object fechaApertura,
				List<String> interesados,
				List<String> organos,
				String versionNTI,
				String identificadorObjetoInside) throws InsideConverterExpedienteMetadatosPrimitivosToInsideException{
			ObjetoExpedienteInsideMetadatos metadatosInside = new ObjetoExpedienteInsideMetadatos ();
			metadatosInside.setClasificacion(clasificacion);
			if (estado != null) {
				metadatosInside.setEstado(ObjetoExpedienteInsideMetadatosEnumeracionEstados.fromValue(estado));
			}

			if (fechaApertura instanceof Calendar) {				
				metadatosInside.setFechaAperturaExpediente((Calendar) fechaApertura);
			} else if (fechaApertura instanceof XMLGregorianCalendar) {
				metadatosInside.setFechaAperturaExpediente(((XMLGregorianCalendar) fechaApertura).toGregorianCalendar());
			} else if (fechaApertura instanceof GregorianCalendar) {
				metadatosInside.setFechaAperturaExpediente((GregorianCalendar) fechaApertura);
			} else {
				throw new InsideConverterExpedienteMetadatosPrimitivosToInsideException ("El metadato fechaApertura no se puede convertir", true);
			}

			metadatosInside.setInteresado(interesados);
			metadatosInside.setOrgano(organos);		
			metadatosInside.setVersionNTI(versionNTI);
			metadatosInside.setIdentificadorObjetoInside(identificadorObjetoInside);

			return metadatosInside;
		}

		/*public static String objetoExpedienteInsideMetadatosToString (ObjetoExpedienteInsideMetadatos metadatosExpedienteInside) {
			
			StringBuffer sb = new StringBuffer ("ObjetoExpedienteInsideMetadatos=");
			if (metadatosExpedienteInside == null) {
				sb.append("null");
			} else {
				sb.append(metadatosExpedienteInside.toString());
				sb.append("IdentificadorObjetoInside=" + metadatosExpedienteInside.getIdentificadorObjetoInside() + LN);
				sb.append("Clasificacion=" + metadatosExpedienteInside.getClasificacion() + LN);
				sb.append("VersionNTI=" + metadatosExpedienteInside.getVersionNTI() + LN);
				sb.append("Estado=" + metadatosExpedienteInside.getEstado() + LN);
				sb.append("Fecha Apertura Exp=" + metadatosExpedienteInside.getFechaAperturaExpediente() + LN);
				sb.append("Organo=" + InsideConverterUtils.listaToString(metadatosExpedienteInside.getOrgano()) + LN);
				sb.append("Interesado=" + InsideConverterUtils.listaToString(metadatosExpedienteInside.getInteresado()) + LN);
				sb.append("MetadatosAdicionales=" + insideMetadatosAdicionalesToString(metadatosExpedienteInside) + LN);
			}
			
			return sb.toString();
		}*/
		
	}
	
	public static class Documento{

		/**
		 * Convierte los metadatos de un objeto Eni y los Adicionales del WS de Inside
		 * en un objeto ObjetoDocumentoInsideMetadatos de Inside
		 * @param metadatosEni
		 * @param metadatosAdicionales
		 * @return
		 * @throws InsideWsRequestErrorException 
		 */
		public static ObjetoDocumentoInsideMetadatos metadatosEniToInside(
				es.mpt.dsic.inside.xml.eni.documento.metadatos.TipoMetadatos metadatosEni, TipoMetadatosAdicionales metadatosAdicionales)
			throws InsideDocumentoMetadatosEniToInsideConverterException
		{
			Assert.notNull(metadatosEni);
			ObjetoDocumentoInsideMetadatos metadatosInside = new ObjetoDocumentoInsideMetadatos();
			ObjetoDocumentoInsideMetadatosEstadoElaboracion estadoElaboracion = new ObjetoDocumentoInsideMetadatosEstadoElaboracion();
			if(metadatosEni.getEstadoElaboracion() != null){
				if(metadatosEni.getEstadoElaboracion().getValorEstadoElaboracion() != null){
					estadoElaboracion.setValorEstadoElaboracion(ObjetoDocumentoInsideMetadatosEstadoElaboracionEnumeracion.fromValue(metadatosEni.getEstadoElaboracion().getValorEstadoElaboracion().value()));
				}
				estadoElaboracion.setIdentificadorDocumentoOrigen(metadatosEni.getEstadoElaboracion().getIdentificadorDocumentoOrigen());
			}
			metadatosInside.setEstadoElaboracion(estadoElaboracion);
			if(metadatosEni.getFechaCaptura() != null){
				metadatosInside.setFechaCaptura(metadatosEni.getFechaCaptura().toGregorianCalendar());
			}
			metadatosInside.setIdentificadorObjetoInside(metadatosEni.getIdentificador());
			metadatosInside.setMetadatosAdicionales(InsideConverterMetadatos.metadatosAdicionalesXmlToInside(metadatosAdicionales));
			metadatosInside.setOrigenCiudadanoAdministracion(metadatosEni.isOrigenCiudadanoAdministracion());
			if(metadatosEni.getTipoDocumental() != null){
				metadatosInside.setTipoDocumental(ObjetoDocumentoInsideMetadatosTipoDocumental.fromValue(metadatosEni.getTipoDocumental().value()));
			}else{
				throw new InsideDocumentoMetadatosEniToInsideConverterException("Error en el tipo documental de los metadatos",true);
			}
			
			metadatosInside.setVersionNTI(metadatosEni.getVersionNTI());
			metadatosInside.getOrgano().addAll(metadatosEni.getOrgano());
			return metadatosInside;
		}

		public static es.mpt.dsic.inside.xml.eni.documento.metadatos.TipoMetadatos metadatosInsideToEni(ObjetoDocumentoInsideMetadatos metadatosInside)
			throws InsideConverterDocumentoMetatosInsideToEniException
		{
			// TODO Llamar a metadatosPrimitivosToEni
			Assert.notNull(metadatosInside);
			es.mpt.dsic.inside.xml.eni.documento.metadatos.TipoMetadatos metadatosEni = new es.mpt.dsic.inside.xml.eni.documento.metadatos.TipoMetadatos();
			TipoEstadoElaboracion estadoElaboracion = new TipoEstadoElaboracion();
			if(metadatosInside.getEstadoElaboracion() != null){
				estadoElaboracion.setValorEstadoElaboracion(EnumeracionEstadoElaboracion.fromValue(metadatosInside.getEstadoElaboracion().getValorEstadoElaboracion().value()));
				estadoElaboracion.setIdentificadorDocumentoOrigen(metadatosInside.getEstadoElaboracion().getIdentificadorDocumentoOrigen());
			}
			metadatosEni.setEstadoElaboracion(estadoElaboracion);
			try {
				metadatosEni.setFechaCaptura(InsideConverterXmlGregorianCalendar.calendarToXmlCalendar(metadatosInside.getFechaCaptura()));
			} catch (InsideConverterCalendarToXmlCalendarException e) {
				throw new InsideConverterDocumentoMetatosInsideToEniException(e);
			}
			metadatosEni.setIdentificador(metadatosInside.getIdentificadorObjetoInside());
			metadatosEni.setId("DOC_" + metadatosInside.getIdentificadorObjetoInside() + "_METADATOS");
			metadatosEni.setOrigenCiudadanoAdministracion(metadatosInside.isOrigenCiudadanoAdministracion());
			metadatosEni.setTipoDocumental(TipoDocumental.fromValue(metadatosInside.getTipoDocumental().value()));
			metadatosEni.setVersionNTI(metadatosInside.getVersionNTI());
			metadatosEni.getOrgano().addAll(metadatosInside.getOrgano());			
			return metadatosEni;
		}
		
		public static es.mpt.dsic.inside.xml.eni.documento.metadatos.TipoMetadatos metadatosPrimitivoToEni (String identificador,
																											String id,
																											String estadoElaboracion,
																											String identificadorDocumentoOrigen,
																											Calendar fechaCaptura,
																											boolean origenCiudadanoAdministracion,
																											String tipoDocumental,
																											String versionNTI,
																											List<String> organos) throws InsideConverterPrimitivosToEniException {
			es.mpt.dsic.inside.xml.eni.documento.metadatos.TipoMetadatos metadatosEni = new es.mpt.dsic.inside.xml.eni.documento.metadatos.TipoMetadatos();
			metadatosEni.setIdentificador(identificador);
			metadatosEni.setEstadoElaboracion(estadoElaboracionPrimitivoToEni (estadoElaboracion, identificadorDocumentoOrigen));
			try {
				metadatosEni.setFechaCaptura(InsideConverterXmlGregorianCalendar.calendarToXmlCalendar(fechaCaptura));
			} catch (InsideConverterCalendarToXmlCalendarException e) {
				throw new InsideConverterDocumentoMetadatosPrimitivosToEniException("Error al convertir la fecha de captura ", e);
				
			}
			metadatosEni.setId(id);
			metadatosEni.setOrigenCiudadanoAdministracion(origenCiudadanoAdministracion);
			metadatosEni.setTipoDocumental(TipoDocumental.fromValue(tipoDocumental));
			if (organos != null) {
				metadatosEni.getOrgano().addAll(organos);
			}
			
			return metadatosEni;
			
		}
		
		public static es.mpt.dsic.inside.xml.eni.documento.metadatos.TipoEstadoElaboracion estadoElaboracionPrimitivoToEni (String estadoElaboracion,
																																			 String identificadorDocumentoOrigen) {
			TipoEstadoElaboracion estadoElaboracionEni = new TipoEstadoElaboracion();
			if (estadoElaboracion != null) {
				estadoElaboracionEni.setValorEstadoElaboracion(EnumeracionEstadoElaboracion.fromValue(estadoElaboracion));
			}
			estadoElaboracionEni.setIdentificadorDocumentoOrigen(identificadorDocumentoOrigen);
			return estadoElaboracionEni;
		}
		
		public static ObjetoDocumentoInsideMetadatos metadatosPrimitivoToInside (String estadoElaboracion,
																				 String identificadorDocumentoOrigen,
																				 Object fechaCaptura,
																				 boolean origenCiudadanoAdministracion,
																				 String tipoDocumental,
																				 String versionNTI,
																				 List<String> organos,
																				 String identificadorObjetoInside) throws InsideConverterPrimitivosToInsideException {
			
			ObjetoDocumentoInsideMetadatos metadatosInside = new ObjetoDocumentoInsideMetadatos();
			
			if (estadoElaboracion != null) {				
				metadatosInside.setEstadoElaboracion(estadoElaboracionPrimitivoToInside (estadoElaboracion, identificadorDocumentoOrigen));
			}
						
			if (fechaCaptura instanceof Calendar) {				
				metadatosInside.setFechaCaptura(InsideConverterXmlGregorianCalendar.calendarToGregorianCalendar((Calendar) fechaCaptura));
			} else if (fechaCaptura instanceof XMLGregorianCalendar) {
				metadatosInside.setFechaCaptura(((XMLGregorianCalendar) fechaCaptura).toGregorianCalendar());
			} else if (fechaCaptura instanceof GregorianCalendar) {
				metadatosInside.setFechaCaptura((GregorianCalendar) fechaCaptura);
			}
			
			metadatosInside.setOrgano(organos);
			metadatosInside.setOrigenCiudadanoAdministracion(origenCiudadanoAdministracion);
			if (tipoDocumental != null) {
				ObjetoDocumentoInsideMetadatosTipoDocumental tDocumental =  ObjetoDocumentoInsideMetadatosTipoDocumental.fromValue(tipoDocumental);
				metadatosInside.setTipoDocumental(tDocumental);
			}			
			metadatosInside.setVersionNTI(versionNTI);
			metadatosInside.setIdentificadorObjetoInside(identificadorObjetoInside);
			return metadatosInside;
			
		}

		public static ObjetoDocumentoInsideMetadatosEstadoElaboracion estadoElaboracionPrimitivoToInside (String estadoElaboracion, String identificadorDocumentoOrigen) {
			ObjetoDocumentoInsideMetadatosEstadoElaboracion eElaboracion =  new ObjetoDocumentoInsideMetadatosEstadoElaboracion ();
			eElaboracion.setValorEstadoElaboracion(ObjetoDocumentoInsideMetadatosEstadoElaboracionEnumeracion.fromValue(estadoElaboracion));
			eElaboracion.setIdentificadorDocumentoOrigen(identificadorDocumentoOrigen);
			return eElaboracion;
		}
		
		public static DocumentoInsideMetadatos metadatosInsideToXml(ObjetoDocumentoInsideMetadatos metadatos) throws InsideConverterDocumentoMetatosInsideToEniException
		{
			Assert.notNull(metadatos);
			DocumentoInsideMetadatos metadatosXml = new DocumentoInsideMetadatos();
			metadatosXml.setMetadatosAdicionales(metadatosAdicionalesInsideToXml(metadatos.getMetadatosAdicionales()));
			metadatosXml.setMetadatosDocumentoENI(metadatosInsideToEni(metadatos));
			return metadatosXml;
		}

		public static ObjetoDocumentoInsideMetadatos metadatosEniToInside(MetadatosEni metadatosAltaEni,TipoMetadatosAdicionales metadatosAdicionales)
			throws InsideDocumentoMetadatosEniToInsideConverterException
		{
			es.mpt.dsic.inside.xml.eni.documento.metadatos.TipoMetadatos metadatosEni = new es.mpt.dsic.inside.xml.eni.documento.metadatos.TipoMetadatos();
			metadatosEni.setFechaCaptura(metadatosAltaEni.getFechaCaptura());
			metadatosEni.setOrigenCiudadanoAdministracion(metadatosAltaEni.isOrigenCiudadanoAdministracion());
			metadatosEni.setTipoDocumental(metadatosAltaEni.getTipoDocumental());
			metadatosEni.getOrgano().addAll(metadatosAltaEni.getOrgano());
			return metadatosEniToInside(metadatosEni,metadatosAdicionales);
		}
		
		public static ObjetoDocumentoInsideMetadatos metadatosConversionToInside (es.mpt.dsic.inside.xml.inside.ws.documento.conversion.TipoDocumentoConversionInside.MetadatosEni metadatosConversion) throws InsideConverterPrimitivosToInsideException {
			
			ObjetoDocumentoInsideMetadatos metadatosInside = metadatosPrimitivoToInside(metadatosConversion.getEstadoElaboracion().getValorEstadoElaboracion().value(),
																						metadatosConversion.getEstadoElaboracion().getIdentificadorDocumentoOrigen(),
																						metadatosConversion.getFechaCaptura(),
																						metadatosConversion.isOrigenCiudadanoAdministracion(),
																						metadatosConversion.getTipoDocumental().value(),
																						metadatosConversion.getVersionNTI(),
																						metadatosConversion.getOrgano(),
																						metadatosConversion.getIdentificador());
																						
			return metadatosInside;
		}
		
		
	}

	

	/**
	 * Convierte los metadatos adicionales desde la entrada del WS 
	 * a un tipo válido para el InsideService
	 * @param metadatosAdicionales
	 * @return
	 */
	public static Map<String, ObjetoInsideMetadatoAdicional> metadatosAdicionalesXmlToInside(
			TipoMetadatosAdicionales metadatosAdicionales) {

		Map<String, ObjetoInsideMetadatoAdicional> adicionales = new HashMap<String, ObjetoInsideMetadatoAdicional>();
		
		if(metadatosAdicionales != null){
			for (MetadatoAdicional metadatoAdicional : metadatosAdicionales.getMetadatoAdicional()) {
				ObjetoInsideMetadatoAdicional adicional = metadatoAdicionalXmlToInside(metadatoAdicional);
				adicionales.put(adicional.getNombre(), adicional);
			}
		}
		
		return adicionales;
	}

	/*public static String insideMetadatosAdicionalesToString(
			InsideMetadatosAdicionales metadatosAdicionales) {

		StringBuffer sb = new StringBuffer ("InsideMetadatosAdicionales=");
		if (metadatosAdicionales == null) {
			sb.append("null" + LN);		
		} else {
			Map<String, ObjetoInsideMetadatoAdicional> mapaAdicionales = metadatosAdicionales.getMetadatosAdicionales();
			if (mapaAdicionales == null) {
				sb.append("Mapa de metadatos adicionales es null");
			} else {
				Set<String> keys = mapaAdicionales.keySet();
				
				for (String key : keys) {
					sb.append("Nombre Met.Adicional=" + key + LN);
					sb.append("Valor Met.Adicional=" + objetoInsideMetadatoAdicionalToString(mapaAdicionales.get(key)) + LN);
				}
				
			}			
		}
		return sb.toString();
	}*/

	/**
	 * Convierte un MetadatoAdicional de la entrada del WS a un tipo
	 * válido para el InsideService
	 * @param metadatoAdicional
	 * @return
	 */
	private static ObjetoInsideMetadatoAdicional metadatoAdicionalXmlToInside(
			MetadatoAdicional metadatoAdicional) {
		ObjetoInsideMetadatoAdicional adicional = new ObjetoInsideMetadatoAdicional();
		adicional.setNombre(metadatoAdicional.getNombre());
		
//		MetadatoValor valor = metadatoAdicional.getValor();
//
//		Object objValor = null;
//		
//		if(valor.getMetadatoId() != null){
//			objValor = valor.getMetadatoId();
//		}else if(valor.getMetadatoInt() != null){
//			objValor = valor.getMetadatoInt();
//		}else if(valor.getMetadatoString() != null){
//			objValor = valor.getMetadatoString();
//		}else if(valor.getMetadatoXML() != null){
//			objValor = valor.getMetadatoXML();
//		}
//		
//		adicional.setValor(objValor);
		

		//if (metadatoAdicional.getTipo().getNamespaceURI().equalsIgnoreCase("http://www.w3.org/2001/XMLSchema") &&
			//	metadatoAdicional.getTipo().getLocalPart().equalsIgnoreCase("string")) {
		
		// TODO Esto no funcionará para cuando no sea un String, mirarlo más adelante
		if (metadatoAdicional.getTipo().getLocalPart().equalsIgnoreCase("string")) {
				Element e = (Element) metadatoAdicional.getValor();				
				adicional.setValor(e.getTextContent());
				
		} else {
			
			adicional.setValor(metadatoAdicional.getValor());
			
		}
				
		adicional.setTipo(metadatoAdicional.getTipo());
		
		return adicional;
	}


	/**
	 * Convierte los metadatos Adicionales a un mensaje
	 * MetadatosAdicionales del WS de Inside
	 * @param metadatosAdicionales
	 * @return
	 */
	public static TipoMetadatosAdicionales metadatosAdicionalesInsideToXml(
			Map<String, ObjetoInsideMetadatoAdicional> metadatosAdicionales)
	{
		TipoMetadatosAdicionales metadatosAdicionalesXml = new TipoMetadatosAdicionales();
		for (ObjetoInsideMetadatoAdicional metadatoAdicional : metadatosAdicionales.values()) {
			metadatosAdicionalesXml.getMetadatoAdicional().add(metadatoInsideToXml(metadatoAdicional));
		}
		
		return metadatosAdicionalesXml;
	}
	
	
	
	public static String tipoMetadatosToString (TipoMetadatos metadatosEni) {
		String coma = ", ";
		StringBuffer sb = new StringBuffer ("TipoMetadatos=[");
		if (metadatosEni == null) {
			sb.append("null");
		} else {
			sb.append("Id=" + metadatosEni.getId() + coma);
			sb.append("Identificador=" + metadatosEni.getIdentificador() + coma);
			sb.append("VersionNTI=" + metadatosEni.getVersionNTI() + coma);
			sb.append("Clasificacion=" + metadatosEni.getClasificacion() + coma);
			sb.append("Estado=" + metadatosEni.getEstado() + coma);
			sb.append("Fecha Apertura Exp=" + metadatosEni.getFechaAperturaExpediente() +  coma);
			sb.append("Organo=" + InsideConverterUtils.listaToString(metadatosEni.getOrgano()) + coma);
			sb.append("Interesado=" + InsideConverterUtils.listaToString(metadatosEni.getInteresado()));
		}
		sb.append("]");
		return sb.toString();
	}
	
	public static String tipoMetadatosAdicionalesToString (TipoMetadatosAdicionales metadatosAdicionales) {
		String coma = ", ";
		StringBuffer sb = new StringBuffer ("TipoMetadatosAdicionales=[");
		if (metadatosAdicionales == null) {
			sb.append("null");
		} else {
			
			List<MetadatoAdicional> lista = metadatosAdicionales.getMetadatoAdicional();
			if (lista == null) {
				sb.append("lista de metadatos adicionales es null");
			} else {
				for (MetadatoAdicional adic : lista) {
					sb.append(metadatoAdicionalToString(adic) + coma);
				}
			}
		}
		sb.append("]");
		
		return sb.toString();
	}
	
	public static String metadatoAdicionalToString (MetadatoAdicional metadatoAdicional) {
		
		StringBuffer sb = new StringBuffer ("MetadatoAdicional=[");
		
		if (metadatoAdicional == null) {
			sb.append("null");
		} else {
			sb.append("Nombre=" + metadatoAdicional.getNombre() + " , ");
			sb.append("Tipo=" + metadatoAdicional.getTipo() + " , ");
			sb.append("Valor=" + metadatoAdicional.getValor() + " , ");
		}
		sb.append("]");
		return sb.toString();
	}
	
	public static String objetoInsideMetadatoAdicionalToString (ObjetoInsideMetadatoAdicional metadatoAdicional) {
		StringBuffer sb = new StringBuffer ("ObjetoInsideMetadatoAdicional=[");
		if (metadatoAdicional == null) {
			sb.append("null");
		} else {
			sb.append("Nombre=" + metadatoAdicional.getNombre() + " , ");
			sb.append("Tipo=" + metadatoAdicional.getTipo() + " , ");
			sb.append("Valor=" + metadatoAdicional.getValor() + " , ");
		}
		sb.append("]");
		return sb.toString();		
	}

	/**
	 * Convierte un metadatoAdicional del modelo de Inside a un
	 * mensaje MetadatoAdicional del WS de Inside
	 * @param metadatoAdicional
	 * @return
	 */
	private static MetadatoAdicional metadatoInsideToXml(
			ObjetoInsideMetadatoAdicional metadatoAdicional) {
		MetadatoAdicional metadatoAdicionalXml = new MetadatoAdicional();
		metadatoAdicionalXml.setNombre(metadatoAdicional.getNombre());
		metadatoAdicionalXml.setTipo(metadatoAdicional.getTipo());
		metadatoAdicionalXml.setValor(metadatoAdicional.getValor());
		return metadatoAdicionalXml;
	}
	

	
	



}
