/* Copyright (C) 2012-13 MINHAP, Gobierno de España
   This program is licensed and may be used, modified and redistributed under the terms
   of the European Public License (EUPL), either version 1.1 or (at your
   option) any later version as soon as they are approved by the European Commission.
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
   or implied. See the License for the specific language governing permissions and
   more details.
   You should have received a copy of the EUPL1.1 license
   along with this program; if not, you may find it at
   http://joinup.ec.europa.eu/software/page/eupl/licence-eupl */

package es.mpt.dsic.inside.model.converter.mime;

import java.util.Collection;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import net.sf.jmimemagic.Magic;
import net.sf.jmimemagic.MagicException;
import net.sf.jmimemagic.MagicMatch;
import net.sf.jmimemagic.MagicMatchNotFoundException;
import net.sf.jmimemagic.MagicParseException;

import es.mpt.dsic.inside.model.converter.InsideConverterDocumento;
import es.mpt.dsic.inside.model.converter.exception.InsideConverterUtilsException;
import eu.medsea.mimeutil.detector.ExtensionMimeDetector;
import eu.medsea.mimeutil.detector.MagicMimeMimeDetector;

public class InsideMimeUtils {
	
	protected static final Log logger = LogFactory.getLog(InsideMimeUtils.class);

	/**
	 * Obtiene el tipo mime en base al contenido de un fichero
	 * @param data
	 * @return
	 * @throws InsideConverterUtilsException 
	 * @throws  
	 */
	public static String getMimeType(byte[] data) throws InsideConverterUtilsException{
		String mime = getFirstMimeTypeMimeUtil(new MagicMimeMimeDetector().getMimeTypes(data));
		logger.debug ("Mime obtenido con mimeutil: " + mime);
		// Si con mimeutil no hemos podido obtener el mime, lo intentamos con jmimemagic
		if (mime == null) {
			mime = getMimeMimeMagic (data);
		}
		// Si el mime es de un ZIP, hacemos otra pasada por si fuera un documento de Office moderno.
		if ("application/zip".equalsIgnoreCase(mime) || "application/x-zip".equalsIgnoreCase(mime)) {
			mime = OfficeAnalizer.getMimeType(data);
		}
		
		return mime;
	}
	
	/**
	 * Obtiene el tipo mime en base al contenido de un fichero
	 * y el nombre de su fichero
	 * @param filename
	 * @param data
	 * @return
	 * @throws  
	 */
	private static String getMimeType(String filename,byte[] data) throws InsideConverterUtilsException{
		String mimeType = getFirstMimeTypeMimeUtil(new ExtensionMimeDetector().getMimeTypes(filename));
		return mimeType != null ? mimeType : getMimeType(data);
	}
	
	/**
	 * Dada una colección de tipos mime, da el primero
	 * @param mimeTypes
	 * @return
	 * @throws InsideConverterUtilsException
	 */
	private static String getFirstMimeTypeMimeUtil(Collection<?> mimeTypes) throws InsideConverterUtilsException{
		if(mimeTypes.size() >= 1){
			return mimeTypes.iterator().next() + "";
		}else{
			return null;
			//throw new InsideConverterUtilsException("No se ha detectado ningún tipo mime");
		}
	}
	
	private static String getMimeMimeMagic (byte[] data) {
		String mime = null;
		Magic parser = new Magic() ;
		try {
			@SuppressWarnings("static-access")
			MagicMatch match = parser.getMagicMatch(data);
			mime = match.getMimeType();
		} catch (MagicException e) {
			logger.debug ("MagicException", e);
		} catch (MagicParseException e) {
			logger.debug ("MagicParseException", e);			
		} catch (MagicMatchNotFoundException e) {
			logger.debug ("MagicMatchNotFoundException", e);		}
		return mime;
	}
}
