/* Copyright (C) 2012-13 MINHAP, Gobierno de España
   This program is licensed and may be used, modified and redistributed under the terms
   of the European Public License (EUPL), either version 1.1 or (at your
   option) any later version as soon as they are approved by the European Commission.
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
   or implied. See the License for the specific language governing permissions and
   more details.
   You should have received a copy of the EUPL1.1 license
   along with this program; if not, you may find it at
   http://joinup.ec.europa.eu/software/page/eupl/licence-eupl */

package es.mpt.dsic.inside.service;

import java.util.Collection;
import java.util.List;

import es.mpt.dsic.inside.model.busqueda.consulta.ConsultaInside;
import es.mpt.dsic.inside.model.busqueda.resultado.ResultadoBusquedaInside;
import es.mpt.dsic.inside.model.objetos.ObjetoInsideVersion;
import es.mpt.dsic.inside.model.objetos.documento.ObjetoDocumentoInside;
import es.mpt.dsic.inside.model.objetos.documento.ObjetoDocumentoInsideContenido;
import es.mpt.dsic.inside.model.objetos.documento.metadatos.ObjetoDocumentoInsideMetadatos;
import es.mpt.dsic.inside.model.objetos.enivalidation.OpcionValidacionDocumento;
import es.mpt.dsic.inside.model.objetos.enivalidation.OpcionValidacionExpediente;
import es.mpt.dsic.inside.model.objetos.enivalidation.ResultadoValidacionDocumento;
import es.mpt.dsic.inside.model.objetos.enivalidation.ResultadoValidacionExpediente;
import es.mpt.dsic.inside.model.objetos.expediente.ObjetoExpedienteInside;
import es.mpt.dsic.inside.model.objetos.expediente.metadatos.ObjetoExpedienteInsideMetadatos;
import es.mpt.dsic.inside.model.objetos.expediente.metadatos.ObjetoExpedienteInsideMetadatosEnumeracionEstados;
import es.mpt.dsic.inside.model.objetos.foliado.FoliadoInside;
import es.mpt.dsic.inside.service.exception.InSideServiceException;
import es.mpt.dsic.inside.service.visualizacion.ResultadoVisualizacionDocumento;


public interface InSideService {

	public ObjetoExpedienteInside altaExpediente(ObjetoExpedienteInside expediente) throws InSideServiceException;
	
	public ObjetoExpedienteInside getExpediente(String identificador) throws InSideServiceException;
	
	public ObjetoExpedienteInside getExpediente(String identificador,Integer version) throws InSideServiceException;
	
	public List<ObjetoInsideVersion> getVersionesExpediente(String identificador) throws InSideServiceException;
	
	public ObjetoExpedienteInsideMetadatos getMetadatosExpediente(String identificador) throws InSideServiceException;
	
	public ObjetoExpedienteInsideMetadatos getMetadatosExpediente(String identificador, Integer version) throws InSideServiceException;
	
	public ObjetoExpedienteInside modificaExpedienteInside(ObjetoExpedienteInside expediente) throws InSideServiceException;
	
	public ObjetoExpedienteInside cambiaMetadatosExpediente(String identificador,ObjetoExpedienteInsideMetadatos metadatos) throws InSideServiceException;
	
	public ObjetoExpedienteInside importarExpediente(String identificador,String identificadorImportado,String ruta) throws InSideServiceException;
	
	public ObjetoExpedienteInside vincularExpediente(String identificador,String identificadorVinculado,String ruta) throws InSideServiceException;
	
	public ObjetoExpedienteInside crearVistaAbiertaExpediente (String identificador) throws InSideServiceException;
	
	public ObjetoExpedienteInside crearVistaCerradaExpediente (String identificador) throws InSideServiceException;
		
	public ObjetoExpedienteInside vincularDocumentosEnExpediente(String identificadorExpediente,List<String> identificadoresDocumentos, String ruta)throws InSideServiceException;
	
	public ObjetoExpedienteInside vincularDocumentosEnExpediente(ObjetoExpedienteInside expediente,Collection<ObjetoDocumentoInside> documentos, String ruta)throws InSideServiceException;
	
	public ObjetoExpedienteInside altaCarpeta(String identificadorExpediente,String identificadorCarpeta,String ruta)throws InSideServiceException;
	
	public ObjetoExpedienteInside borrarCarpeta(String identificadorExpediente,String identificadorCarpeta, String ruta)throws InSideServiceException;
		
	public ObjetoExpedienteInside setEstadoExpediente(String identificador,ObjetoExpedienteInsideMetadatosEnumeracionEstados estado) throws InSideServiceException;
	
	public ObjetoDocumentoInside altaDocumento (ObjetoDocumentoInside documento) throws InSideServiceException;
	
	public ObjetoDocumentoInside altaDocumento (ObjetoDocumentoInside documento,String identificadorExpediente,String ruta) throws InSideServiceException;
	
	public ObjetoDocumentoInside validarDocumento (ObjetoDocumentoInside documento) throws InSideServiceException;
	
	public ObjetoExpedienteInside validarExpediente (ObjetoExpedienteInside expediente) throws InSideServiceException;
	
	public ObjetoExpedienteInside obtenerVisualizacionIndice (ObjetoExpedienteInside expediente, boolean estamparImagen, boolean estamparNombreOrganismo, List<String> lineasNombreOrganismo, boolean expedienteExterno) throws InSideServiceException;
	
	public ObjetoDocumentoInside getDocumento (String identificador) throws InSideServiceException;
	
	public ObjetoDocumentoInside getDocumento (String identificador,Integer version) throws InSideServiceException;
	
	public ObjetoDocumentoInsideContenido getDocumentoContenido (String identificador,Integer version) throws InSideServiceException;
	
	public ObjetoDocumentoInsideMetadatos getMetadatosDocumento (String identificador) throws InSideServiceException;
	
	public ObjetoDocumentoInsideMetadatos getMetadatosDocumento(String identificador, Integer version) throws InSideServiceException;
	
	public ObjetoDocumentoInside modificaDocumento(ObjetoDocumentoInside documento) throws InSideServiceException;
	
	public List<ObjetoInsideVersion> getVersionesDocumento(String identificador) throws InSideServiceException;
	
	public FoliadoInside getFoliadoExpediente(String identificador) throws InSideServiceException;

	public ObjetoDocumentoInside firmaServidorDocumento(ObjetoDocumentoInside documento) throws InSideServiceException;
	
	public ResultadoBusquedaInside<ObjetoExpedienteInsideMetadatos> buscarExpedientes(ConsultaInside consulta,int limite, int pagina) throws InSideServiceException;
	
	public ResultadoBusquedaInside<ObjetoDocumentoInsideMetadatos> buscarDocumentos(ConsultaInside consulta,int limite, int pagina) throws InSideServiceException;
	
	public List<ResultadoValidacionExpediente> validaExpedienteEniFile (byte[] expediente, List<OpcionValidacionExpediente> opciones) throws InSideServiceException;
	
	public List<ResultadoValidacionDocumento> validaDocumentoEniFile (byte[] documento, List<OpcionValidacionDocumento> opciones) throws InSideServiceException;
	
	public ResultadoVisualizacionDocumento visualizaDocumentoInside (ObjetoDocumentoInside documento, boolean estamparImagen, boolean estamparNombreOrganismo, List<String> lineasNombreOrganismo) throws InSideServiceException;

		 
	//public DocumentoInterno getFicheroIndice(String idObjetoExpedienteInside) throws InSideServiceException;
	// public Documento altaDocumento (Documento documentoEntrada, ContenidoDoc contenido) throws InSideServiceException;
}
