/* Copyright (C) 2012-13 MINHAP, Gobierno de España
   This program is licensed and may be used, modified and redistributed under the terms
   of the European Public License (EUPL), either version 1.1 or (at your
   option) any later version as soon as they are approved by the European Commission.
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
   or implied. See the License for the specific language governing permissions and
   more details.
   You should have received a copy of the EUPL1.1 license
   along with this program; if not, you may find it at
   http://joinup.ec.europa.eu/software/page/eupl/licence-eupl */

package es.mpt.dsic.inside.service.impl;

import java.io.StringWriter;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.GregorianCalendar;
import java.util.List;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.Marshaller;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Required;
import org.springframework.util.Assert;

import es.mpt.dsic.infofirma.service.InfoFirmaService;
import es.mpt.dsic.inside.model.busqueda.consulta.ConsultaInside;
import es.mpt.dsic.inside.model.busqueda.resultado.ResultadoBusquedaInside;
import es.mpt.dsic.inside.model.converter.InsideConverterDocumento;
import es.mpt.dsic.inside.model.converter.InsideConverterExpediente;
import es.mpt.dsic.inside.model.converter.InsideConverterIndice;
import es.mpt.dsic.inside.model.converter.exception.InsideConverterDocumentoInsideToDocumentoContenidoException;
import es.mpt.dsic.inside.model.objetos.ObjetoInside;
import es.mpt.dsic.inside.model.objetos.ObjetoInsideMetadatos;
import es.mpt.dsic.inside.model.objetos.ObjetoInsideVersion;
import es.mpt.dsic.inside.model.objetos.ObjetoInsideVersionable;
import es.mpt.dsic.inside.model.objetos.documento.ObjetoDocumentoInside;
import es.mpt.dsic.inside.model.objetos.documento.ObjetoDocumentoInsideContenido;
import es.mpt.dsic.inside.model.objetos.documento.metadatos.ObjetoDocumentoInsideMetadatos;
import es.mpt.dsic.inside.model.objetos.enivalidation.OpcionValidacionDocumento;
import es.mpt.dsic.inside.model.objetos.enivalidation.OpcionValidacionExpediente;
import es.mpt.dsic.inside.model.objetos.enivalidation.ResultadoValidacionDocumento;
import es.mpt.dsic.inside.model.objetos.enivalidation.ResultadoValidacionExpediente;
import es.mpt.dsic.inside.model.objetos.expediente.ObjetoExpedienteInside;
import es.mpt.dsic.inside.model.objetos.expediente.indice.ObjetoExpedienteInsideIndice;
import es.mpt.dsic.inside.model.objetos.expediente.indice.ObjetoExpedienteInsideIndiceContenido;
import es.mpt.dsic.inside.model.objetos.expediente.indice.ObjetoExpedienteInsideIndiceContenidoCarpetaIndizada;
import es.mpt.dsic.inside.model.objetos.expediente.indice.ObjetoExpedienteInsideIndiceContenidoDocumentoIndizado;
import es.mpt.dsic.inside.model.objetos.expediente.indice.ObjetoExpedienteInsideIndiceContenidoElementoContenedorElementos;
import es.mpt.dsic.inside.model.objetos.expediente.indice.ObjetoExpedienteInsideIndiceContenidoElementoIndizado;
import es.mpt.dsic.inside.model.objetos.expediente.metadatos.ObjetoExpedienteInsideMetadatos;
import es.mpt.dsic.inside.model.objetos.expediente.metadatos.ObjetoExpedienteInsideMetadatosEnumeracionEstados;
import es.mpt.dsic.inside.model.objetos.firmas.FirmaInside;
import es.mpt.dsic.inside.model.objetos.firmas.FirmaInsideTipoFirmaEnum;
import es.mpt.dsic.inside.model.objetos.firmas.contenido.ContenidoFirmaCertificadoContenidoBinarioInside;
import es.mpt.dsic.inside.model.objetos.foliado.FoliadoInside;
import es.mpt.dsic.inside.model.objetos.foliado.FoliadoNodo;
import es.mpt.dsic.inside.service.InSideService;
import es.mpt.dsic.inside.service.enivalidation.InsideEniValidationService;
import es.mpt.dsic.inside.service.enivalidation.exception.InsideEniValidationException;
import es.mpt.dsic.inside.service.exception.InSideServiceContentExtractException;
import es.mpt.dsic.inside.service.exception.InSideServiceException;
import es.mpt.dsic.inside.service.exception.InSideServiceRequestErrorException;
import es.mpt.dsic.inside.service.exception.InsideServiceInternalException;
import es.mpt.dsic.inside.service.foliado.InsideServiceFoliado;
import es.mpt.dsic.inside.service.foliado.impl.InsideServiceIndiceFoliadoConverter;
import es.mpt.dsic.inside.service.object.converter.InsideServiceObjectConverterException;
import es.mpt.dsic.inside.service.object.definitions.InSideServiceObjectDefinitionNotFoundException;
import es.mpt.dsic.inside.service.object.definitions.InsideObjectDefinitionsContainer;
import es.mpt.dsic.inside.service.object.metadatos.validator.Exception.InSideServiceValidationException;
import es.mpt.dsic.inside.service.object.signer.InsideServiceObjectSignerException;
import es.mpt.dsic.inside.service.object.signer.InsideServiceSigner;
import es.mpt.dsic.inside.service.store.InsideServiceStore;
import es.mpt.dsic.inside.service.store.exception.InsideServiceStoreException;
import es.mpt.dsic.inside.service.store.exception.InsideStoreObjectNotFoundException;
import es.mpt.dsic.inside.service.util.InsideUtils;
import es.mpt.dsic.inside.service.visualizacion.InsideServiceVisualizacionIndice;
import es.mpt.dsic.inside.service.visualizacion.OpcionesVisualizacionIndice;
import es.mpt.dsic.inside.service.visualizacion.ResultadoVisualizacionDocumento;
import es.mpt.dsic.inside.xml.eni.expediente.indice.TipoIndice;
import es.mpt.dsic.inside.xml.eni.expediente.indice.contenido.ObjectFactory;
import es.mpt.dsic.inside.xml.eni.expediente.indice.contenido.TipoIndiceContenido;

public class InSideServiceImpl implements InSideService {

	protected static final Log logger = LogFactory.getLog(InSideServiceImpl.class);
	
	//@Autowired
	//private MetadatosConverterService metadatosConverterService;

	//@Autowired(required=false)
	//private RepoDocIntegration repoDocIntegration;

	//@Autowired
	//private InSideDao insideDao;

	@Autowired
	@Qualifier("InsideObjectDefinitions")
	private InsideObjectDefinitionsContainer objectDefinitions;

	//@Autowired(required=false)
	//private AplicacionContext aplicacionContext;

	@Autowired
	private InsideServiceStore insideStore;

	@Autowired 
	private InsideServiceFoliado insideFoliado;
	
	@Autowired
	private InsideServiceSigner insideSigner;
	
	@Autowired
	private InfoFirmaService infoFirmaService;
	
	@Autowired
	private InsideEniValidationService insideEniValidationService;
	
	@Autowired
	private InsideServiceVisualizacionIndice insideVisualizacion;
	
	
	private String funcionResumenDocumentos;



	private MessageDigest messageDigest;

	

	@Override
	public ObjetoExpedienteInside altaExpediente(ObjetoExpedienteInside expediente) 
			throws InSideServiceException 
	{
		logger.debug("Inicio operacion altaExpediente");
		
		logger.debug("Parametro <expediente>=" + InsideConverterExpediente.expedienteInsideToString(expediente));
		
		validateObjetoInsideIdentificador(expediente);
		validateObjetoInsideVersion(expediente); //TODO no se rellena bien la version

		logger.debug("Expediente tras validar identificador y version : " + InsideConverterExpediente.expedienteInsideToString(expediente));
		
		expediente.getMetadatos().setEstado(ObjetoExpedienteInsideMetadatosEnumeracionEstados.E_01);
		
		logger.debug("Expediente tras setEstado  : " + InsideConverterExpediente.expedienteInsideToString(expediente));
		
		//TODO ¿Esto está bien aquí o va en un validador?
		if(expediente.getMetadatos().getFechaAperturaExpediente() == null){
			expediente.getMetadatos().setFechaAperturaExpediente(GregorianCalendar.getInstance());
		}
		
		logger.debug("Expediente tras set FechaAperturaExpediente : " + InsideConverterExpediente.expedienteInsideToString(expediente));
		
		expediente = validate(expediente);
		
		logger.debug("Expediente tras validate : " + InsideConverterExpediente.expedienteInsideToString(expediente));
		ObjetoExpedienteInside expedienteRepo = converToRepository(expediente);
		
		logger.debug("Expediente tras converToRepository : " + InsideConverterExpediente.expedienteInsideToString(expediente));
		
		ObjetoExpedienteInside expedienteStored = insideStore.storeObject(expedienteRepo);
		
		logger.debug("Expediente stored : " + InsideConverterExpediente.expedienteInsideToString(expedienteStored));
		
		ObjetoExpedienteInside expedienteConverted = convertFromRepository (expedienteStored);
		
		logger.debug("Expediente convertedFromRepository : " + InsideConverterExpediente.expedienteInsideToString(expedienteConverted));
		
		return expedienteConverted;
	}


	@Override
	public ObjetoExpedienteInside getExpediente(String identificador) 
			throws InSideServiceException 
	{
		logger.debug("Inicio operacion getExpediente");
		logger.debug("Parametro <identificador> : " + identificador);
		
		ObjetoExpedienteInside expediente = getObjetoInside(ObjetoExpedienteInside.class,identificador,null);
		
		logger.debug("Expediente recuperado de getObjetoInside" + InsideConverterExpediente.expedienteInsideToString(expediente));
		
		expediente = convertFromRepository (expediente);
		
		logger.debug("Expediente recuperado de convertFromRepository" + InsideConverterExpediente.expedienteInsideToString(expediente));
		
		if (insideVisualizacion.isActivo()) {			
			expediente = insideVisualizacion.expedienteConVisualizacionIndice(expediente);
		}
		
		logger.debug("Expediente recuperado tras comprobar activacion de visualizacion de indice: " + InsideConverterExpediente.expedienteInsideToString(expediente));
		logger.debug("Fin operacion getExpediente");
		return expediente;
	}

	@Override
	public ObjetoExpedienteInside getExpediente(String identificador,Integer version) 
			throws InSideServiceException 
	{
		logger.debug("Inicio operacion getExpediente");
		logger.debug("Parametro <identificador> : " + identificador + " , Parametro <version> : " + version);
		
		ObjetoExpedienteInside expediente = getObjetoInside(ObjetoExpedienteInside.class,identificador,version);
		
		logger.debug("Expediente recuperado de getObjetoInside" + InsideConverterExpediente.expedienteInsideToString(expediente));
		
		expediente = convertFromRepository (expediente);
		
		logger.debug("Expediente recuperado de convertFromRepository" + InsideConverterExpediente.expedienteInsideToString(expediente));
		
		if (insideVisualizacion.isActivo()) {
			expediente = insideVisualizacion.expedienteConVisualizacionIndice(expediente);
		}
		
		logger.debug("Expediente recuperado tras comprobar activacion de visualizacion de indice: " + InsideConverterExpediente.expedienteInsideToString(expediente));
		logger.debug("Fin operacion getExpediente");
		
		return expediente;		
	}

	@Override
	public List<ObjetoInsideVersion> getVersionesExpediente(String identificador)
		throws InSideServiceException {
		logger.debug ("Inicio operacion getVersionesExpediente");
		logger.debug ("Parametro <identificador> = " + identificador);
		List<ObjetoInsideVersion> versiones = insideStore.getObjectVersions(ObjetoExpedienteInside.class, identificador);
		logger.debug ("Recuperadas " + versiones.size() + " versiones");
		logger.debug ("Fin operacion getVersionesExpediente");
		return versiones;
	}

	@Override
	public ObjetoExpedienteInsideMetadatos getMetadatosExpediente(String identificador)
		throws InSideServiceException {
		logger.debug ("Inicio operacion getMetadatosExpediente");
		logger.debug("Parametro <identificador> = " + identificador);
		ObjetoExpedienteInsideMetadatos metadatos = insideStore.getObjectMetadatos(ObjetoExpedienteInside.class,identificador);
		logger.debug("metadatos recuperados: " + metadatos.toString());
		logger.debug ("Fin operacion getMetadatosExpediente");
		//TODO convertFromRepository(metadatos);
		return metadatos;
	}

	@Override
	public ObjetoExpedienteInsideMetadatos getMetadatosExpediente(String identificador, Integer version)
		throws InSideServiceException {
		logger.debug ("Inicio operacion getMetadatosExpediente");
		logger.debug("Parametro <identificador>, <version> = " + identificador + ", " + version);
		ObjetoExpedienteInsideMetadatos metadatos = insideStore.getObjectMetadatos(ObjetoExpedienteInside.class,identificador,version);
		logger.debug("metadatos recuperados: " + metadatos.toString());
		logger.debug ("Fin operacion getMetadatosExpediente");
		//TODO convertFromRepository(metadatos);
		return metadatos;
	}

	@Override
	public ObjetoExpedienteInside modificaExpedienteInside(ObjetoExpedienteInside expediente)
			throws InSideServiceException 
	{
		logger.debug("Inicio operacion modificaExpedienteInside");
		logger.debug ("Parametro <expediente>" + InsideConverterExpediente.expedienteInsideToString(expediente));
		expediente = validate(expediente);
		logger.debug ("expediente tras validate: " + InsideConverterExpediente.expedienteInsideToString(expediente));
		expediente = converToRepository(expediente);
		logger.debug ("expediente tras converToRepository: " + InsideConverterExpediente.expedienteInsideToString(expediente));
		expediente = insideStore.updateObject(expediente);
		logger.debug ("expediente tras store.updateObject: " + InsideConverterExpediente.expedienteInsideToString(expediente));
		logger.debug("Fin operacion modificaExpedienteInside");
		return convertFromRepository(expediente);
	}

	@Override
	public ObjetoExpedienteInside cambiaMetadatosExpediente(String identificador, ObjetoExpedienteInsideMetadatos metadatos)
			throws InSideServiceException
	{
		logger.debug("Inicio operacion cambiaMetadatosExpediente");
		logger.debug("Parametro <identificador> : " + identificador);
		logger.debug("Parametro <metadatos>: " + metadatos.toString());
		ObjetoExpedienteInside expediente = getExpediente(identificador);
		logger.debug("expediente tras getExpediente(identificador): " + InsideConverterExpediente.expedienteInsideToString(expediente));
		expediente.setMetadatos(metadatos);
		logger.debug("expediente tras expediente.setMetadatos(metadatos): " + InsideConverterExpediente.expedienteInsideToString(expediente));
		expediente = modificaExpedienteInside(expediente);
		logger.debug("expediente tras modificaExpedienteInside(expediente): " + InsideConverterExpediente.expedienteInsideToString(expediente));
		logger.debug("Fin operacion cambiaMetadatosExpediente");
		return expediente;
	}
	
	@Override
	public ObjetoExpedienteInside importarExpediente(String identificador,String identificadorImportado, String ruta)
		throws InSideServiceException
	{
		logger.debug ("Inicio operacion importarExpediente");
		logger.debug ("Parametro <identificador> " + identificador);
		logger.debug ("Parametro <identificadorImportado> " + identificadorImportado);
		logger.debug ("Parametro <ruta> " + ruta);
		
		ObjetoExpedienteInside expediente = getExpediente(identificador);
		
		logger.debug ("expediente tras getExpediente(identificador): " + InsideConverterExpediente.expedienteInsideToString(expediente));
				
		ObjetoExpedienteInside expedienteImportado = getExpediente(identificadorImportado);
		
		logger.debug ("expedienteImportado tras getExpediente(identificadorImportado): " + InsideConverterExpediente.expedienteInsideToString(expedienteImportado));
		
		ObjetoExpedienteInsideIndiceContenidoElementoContenedorElementos destino = getCarpetaOExpedienteDeRuta(expediente,ruta);
		
		logger.debug ("destino tras getCarpetaOExpedienteDeRuta(expediente,ruta): " + destino.toString());
		
		ObjetoExpedienteInsideIndiceContenido indiceImportado = new ObjetoExpedienteInsideIndiceContenido();

		indiceImportado.setIdentificadorExpedienteAsociado(identificadorImportado);
		indiceImportado.setTipoAsociacion(ObjetoExpedienteInsideIndiceContenido.TipoAsociacion.IMPORTACION);
		indiceImportado.setOrden(destino.getElementosIndizados().size() + 1);
		destino.getElementosIndizados().add(indiceImportado);
		
		addElementosIndizadosImportados(indiceImportado,expedienteImportado.getIndice().getIndiceContenido().getElementosIndizados());
		
		logger.debug ("destino tras importacion " + destino.toString());
		expediente = modificaExpedienteInside(expediente);
		
		logger.debug ("expediente tras modificaExpedienteInside(expediente) " + InsideConverterExpediente.expedienteInsideToString(expediente));
		logger.debug ("Fin operacion importarExpediente");
		return expediente;
	}

	@Override
	public ObjetoExpedienteInside vincularExpediente(String identificador,String identificadorVinculado, String ruta)
		throws InSideServiceException 
	{
		logger.debug ("Inicio operacion vincularExpediente");
		logger.debug ("Parametro <identificador> " + identificador);
		logger.debug ("Parametro <identificadorVinculado> " + identificadorVinculado);
		logger.debug ("Parametro <ruta> " + ruta);
		
		ObjetoExpedienteInside expediente = getExpediente(identificador);		
		logger.debug ("expediente tras getExpediente(identificador): " + InsideConverterExpediente.expedienteInsideToString(expediente));
		
		ObjetoExpedienteInside expedienteVinculado = getExpediente(identificadorVinculado);
		logger.debug ("expedienteVinculado tras getExpediente(identificadorVinculado): " + InsideConverterExpediente.expedienteInsideToString(expedienteVinculado));
		
		ObjetoExpedienteInsideIndiceContenidoElementoContenedorElementos destino = getCarpetaOExpedienteDeRuta(expediente,ruta);
		logger.debug ("destino tras getCarpetaOExpedienteDeRuta(expediente,ruta): " + destino.toString());
		
		ObjetoExpedienteInsideIndiceContenido indiceVinculado = new ObjetoExpedienteInsideIndiceContenido();
		indiceVinculado.setIdentificadorExpedienteAsociado(identificadorVinculado);
		indiceVinculado.setTipoAsociacion(ObjetoExpedienteInsideIndiceContenido.TipoAsociacion.VINCULACION);
		indiceVinculado.setOrden(destino.getElementosIndizados().size() + 1);
		destino.getElementosIndizados().add(indiceVinculado);
		
		addElementosIndizadosImportados(indiceVinculado,expedienteVinculado.getIndice().getIndiceContenido().getElementosIndizados());
		
		logger.debug ("destino tras vinculacion " + destino.toString());
		
		expediente = modificaExpedienteInside(expediente);
		logger.debug ("expediente tras modificaExpedienteInside(expediente) " + InsideConverterExpediente.expedienteInsideToString(expediente));
		logger.debug ("Inicio operacion vincularExpediente");
		return expediente;
	}
	
	@Override
	public ObjetoExpedienteInside crearVistaAbiertaExpediente (String identificador) throws InSideServiceException {
		logger.debug ("Inicio operacion crearVistaAbiertaExpediente");
		logger.debug ("Parametro <identificador>: " + identificador);
		ObjetoExpedienteInsideMetadatos metadatos = getMetadatosExpediente(identificador);
		logger.debug ("metadatos tras getMetadatosExpediente(identificador): " + metadatos.toString());
		ObjetoExpedienteInside nuevoExpediente = new ObjetoExpedienteInside ();
		nuevoExpediente.setMetadatos(metadatos);
		nuevoExpediente = altaExpediente (nuevoExpediente);
		logger.debug ("nuevoExpediente tras altaExpediente(nuevoExpediente): " + InsideConverterExpediente.expedienteInsideToString(nuevoExpediente));
		nuevoExpediente = vincularExpediente (nuevoExpediente.getIdentificador(), identificador, "/");
		logger.debug ("nuevoExpediente tras vincularExpediente (nuevoExpediente.getIdentificador(), identificador, '/') " + InsideConverterExpediente.expedienteInsideToString(nuevoExpediente));
		logger.debug ("Fin operacion crearVistaAbiertaExpediente");
		return nuevoExpediente;
	}
	
	@Override 
	public ObjetoExpedienteInside crearVistaCerradaExpediente (String identificador) throws InSideServiceException  {
		logger.debug ("Inicio operacion crearVistaCerradaExpediente");
		logger.debug ("Parametro <identificador>: " + identificador);
		
		ObjetoExpedienteInsideMetadatos metadatos = getMetadatosExpediente(identificador);
		logger.debug ("metadatos tras getMetadatosExpediente(identificador): " + metadatos.toString());
		
		ObjetoExpedienteInside nuevoExpediente = new ObjetoExpedienteInside ();
		nuevoExpediente.setMetadatos(metadatos);		
		nuevoExpediente = altaExpediente (nuevoExpediente);
		logger.debug ("nuevoExpediente tras altaExpediente(nuevoExpediente): " + InsideConverterExpediente.expedienteInsideToString(nuevoExpediente));
		
		nuevoExpediente = importarExpediente (nuevoExpediente.getIdentificador(), identificador, "/");		
		nuevoExpediente = setEstadoExpediente(nuevoExpediente.getIdentificador(), ObjetoExpedienteInsideMetadatosEnumeracionEstados.E_02);
		logger.debug ("nuevoExpediente tras importarExpediente (nuevoExpediente.getIdentificador(), identificador, '/'); setEstadoExpediente(); " + InsideConverterExpediente.expedienteInsideToString(nuevoExpediente));
		logger.debug("Fin operacion crearVistaCerradaExpediente");
		return nuevoExpediente;
	}
	
	protected void addElementosIndizadosImportados(ObjetoExpedienteInsideIndiceContenidoElementoContenedorElementos contenedor, List<ObjetoExpedienteInsideIndiceContenidoElementoIndizado> elementos) throws InsideServiceInternalException
	{
		for(ObjetoExpedienteInsideIndiceContenidoElementoIndizado elementoIndizado : elementos){
			if(elementoIndizado instanceof ObjetoExpedienteInsideIndiceContenidoElementoContenedorElementos){
				ObjetoExpedienteInsideIndiceContenidoElementoContenedorElementos contenedorIndizado = (ObjetoExpedienteInsideIndiceContenidoElementoContenedorElementos) elementoIndizado;
				ObjetoExpedienteInsideIndiceContenidoElementoContenedorElementos nuevoContenedor = null;
				if(contenedorIndizado instanceof ObjetoExpedienteInsideIndiceContenidoCarpetaIndizada){
					ObjetoExpedienteInsideIndiceContenidoCarpetaIndizada carpetaIndizada = (ObjetoExpedienteInsideIndiceContenidoCarpetaIndizada) contenedorIndizado;
					ObjetoExpedienteInsideIndiceContenidoCarpetaIndizada carpIndizadaImportada = new ObjetoExpedienteInsideIndiceContenidoCarpetaIndizada();
					carpIndizadaImportada.setIdentificadorCarpeta(carpetaIndizada.getIdentificadorCarpeta());
					nuevoContenedor = carpIndizadaImportada;
					
				}else if(contenedorIndizado instanceof ObjetoExpedienteInsideIndiceContenido){
					ObjetoExpedienteInsideIndiceContenido expIndizado = (ObjetoExpedienteInsideIndiceContenido) contenedorIndizado;
					ObjetoExpedienteInsideIndiceContenido expIndizadoImportado = new ObjetoExpedienteInsideIndiceContenido();
					expIndizadoImportado.setFechaIndiceElectronico(expIndizado.getFechaIndiceElectronico());
					expIndizadoImportado.setIdentificadorExpedienteAsociado(expIndizado.getIdentificadorExpedienteAsociado());
					expIndizadoImportado.setTipoAsociacion(expIndizado.getTipoAsociacion());
					nuevoContenedor = expIndizadoImportado;
				}else{
					throw new InsideServiceInternalException("El elementoIndizado es de un tipo no válido " + elementoIndizado.getClass().getName());
				}
				addElementosIndizadosImportados(nuevoContenedor,contenedorIndizado.getElementosIndizados());
				nuevoContenedor.setOrden(contenedorIndizado.getOrden());
				contenedor.getElementosIndizados().add(nuevoContenedor);
			}else if(elementoIndizado instanceof ObjetoExpedienteInsideIndiceContenidoDocumentoIndizado){
				ObjetoExpedienteInsideIndiceContenidoDocumentoIndizado documentoIndizado = (ObjetoExpedienteInsideIndiceContenidoDocumentoIndizado) elementoIndizado;
				ObjetoExpedienteInsideIndiceContenidoDocumentoIndizado docIndizadoImportado = new ObjetoExpedienteInsideIndiceContenidoDocumentoIndizado();
				docIndizadoImportado.setOrden(documentoIndizado.getOrden());
				docIndizadoImportado.setFuncionResumen(documentoIndizado.getFuncionResumen());
				docIndizadoImportado.setFechaIncorporacionExpediente(documentoIndizado.getFechaIncorporacionExpediente());
				//TODO duplicar el documento importado
				docIndizadoImportado.setIdentificadorDocumento(documentoIndizado.getIdentificadorDocumento());
				docIndizadoImportado.setValorHuella(documentoIndizado.getValorHuella());
				contenedor.getElementosIndizados().add(docIndizadoImportado);
			}else{
				throw new InsideServiceInternalException("El elementoIndizado es de un tipo no válido " + elementoIndizado.getClass().getName());
			}
		}
	}


	@Override
	public ObjetoExpedienteInside setEstadoExpediente(String identificador,ObjetoExpedienteInsideMetadatosEnumeracionEstados estado) 
			throws InSideServiceException
	{
		logger.debug ("Inicio operacion setEstadoExpediente");
		logger.debug ("Parametro <identificador> :" + identificador);
		logger.debug ("Parametro <estado> :" + estado.value());
		
		ObjetoExpedienteInside expediente = getExpediente(identificador);
		logger.debug ("expediente tras getExpediente(identificador) :" + InsideConverterExpediente.expedienteInsideToString(expediente));
		expediente.getMetadatos().setEstado(estado);
		
		if (estado == ObjetoExpedienteInsideMetadatosEnumeracionEstados.E_02) {			
			this.firmarIndiceExpediente(expediente);
			logger.debug ("expediente tras firmarIndiceExpediente(expediente): " + InsideConverterExpediente.expedienteInsideToString(expediente));
		}
		
		expediente = modificaExpedienteInside(expediente);
		logger.debug ("expediente tras modificaExpedienteInside(expediente) :" + InsideConverterExpediente.expedienteInsideToString(expediente));
		logger.debug ("Fin operacion setEstadoExpediente");
		return expediente;
	}
	
	@Override
	public ObjetoExpedienteInside altaCarpeta(String identificadorExpediente,String identificadorCarpeta, String ruta)
		throws InSideServiceException
	{
		logger.debug ("Inicio operacion altaCarpeta");
		logger.debug ("Parametro <identificadorExpediente> : " + identificadorExpediente);
		logger.debug ("Parametro <identificadorCarpeta> : "+ identificadorCarpeta);
		logger.debug ("Parametro <ruta> : "+ ruta);
		
		ObjetoExpedienteInside expediente = getExpediente(identificadorExpediente);
		logger.debug ("expediente tras getExpediente(identificadorExpediente) : " + InsideConverterExpediente.expedienteInsideToString(expediente));
		
		ObjetoExpedienteInsideIndiceContenidoElementoContenedorElementos destino = getCarpetaOExpedienteDeRuta(expediente,ruta);
		logger.debug ("destino tras getCarpetaOExpedienteDeRuta(expediente,ruta); : " + destino.toString());
		
		ObjetoExpedienteInsideIndiceContenidoCarpetaIndizada carpetaIndizada = new ObjetoExpedienteInsideIndiceContenidoCarpetaIndizada();
		carpetaIndizada.setIdentificadorCarpeta(identificadorCarpeta);
		carpetaIndizada.setOrden(destino.getElementosIndizados().size() + 1);
		destino.getElementosIndizados().add(carpetaIndizada);
		
		logger.debug ("expediente tras meter carpeta : " + InsideConverterExpediente.expedienteInsideToString(expediente));
		expediente = modificaExpedienteInside(expediente);
		logger.debug ("expediente tras modificaExpedienteInside(expediente) :" + InsideConverterExpediente.expedienteInsideToString(expediente));
		logger.debug ("Fin operacion altaCarpeta");
		
		return expediente;	
	}
	
	@Override
	public ObjetoExpedienteInside borrarCarpeta(String identificadorExpediente,String identificadorCarpeta, String ruta)
		throws InSideServiceException
	{
		logger.debug ("Inicio operacion borrarCarpeta");
		logger.debug ("Parametro <identificadorExpediente> : " + identificadorExpediente);
		logger.debug ("Parametro <identificadorCarpeta> : " + identificadorCarpeta);
		logger.debug ("Parametro <ruta> : " + ruta);
		
		ObjetoExpedienteInside expediente = getExpediente(identificadorExpediente);
		logger.debug ("expediente tras getExpediente(identificadorExpediente) : " + InsideConverterExpediente.expedienteInsideToString(expediente));
		ObjetoExpedienteInsideIndiceContenidoElementoContenedorElementos destino = getCarpetaOExpedienteDeRuta(expediente,ruta);
		logger.debug ("destino tras getCarpetaOExpedienteDeRuta(expediente,ruta); : " + destino.toString());
		
		ObjetoExpedienteInsideIndiceContenidoCarpetaIndizada carpetaBorrar = null;
		
		for(ObjetoExpedienteInsideIndiceContenidoElementoIndizado elementoIndizado : destino.getElementosIndizados()){
			if(elementoIndizado instanceof ObjetoExpedienteInsideIndiceContenidoCarpetaIndizada){
				ObjetoExpedienteInsideIndiceContenidoCarpetaIndizada carpetaIndizada = (ObjetoExpedienteInsideIndiceContenidoCarpetaIndizada) elementoIndizado;
				if(identificadorCarpeta.equals(carpetaIndizada.getIdentificadorCarpeta())){
					carpetaBorrar = carpetaIndizada;
				}
			}
		}
		
		if(carpetaBorrar == null){
			throw new InSideServiceRequestErrorException("No existe la carpeta con identificador " + identificadorCarpeta + " en la ruta " + ruta);
		}
		
		destino.getElementosIndizados().remove(carpetaBorrar);
		
		int orden = 1;
		for(ObjetoExpedienteInsideIndiceContenidoElementoIndizado elementoIndizado : destino.getElementosIndizados()){
			elementoIndizado.setOrden(orden++);
		}
		logger.debug ("destino tras eliminar la carpeta : " + destino.toString());
		expediente = modificaExpedienteInside(expediente);
		logger.debug ("expediente tras modificaExpedienteInside(expediente) : " + InsideConverterExpediente.expedienteInsideToString(expediente));
		logger.debug ("Fin operacion borrarCarpeta");
		return expediente;
	}

	@Override
	public ObjetoDocumentoInside altaDocumento(ObjetoDocumentoInside documento)
		throws InSideServiceException 
	{
		logger.debug ("Inicio operacion altaDocumento");
		logger.debug ("Parametro <documento> " + InsideConverterDocumento.documentoInsideToString(documento));
		validateObjetoInsideIdentificador(documento);
		logger.debug ("documento tras validateObjetoInsideIdentificador(documento) : " + InsideConverterDocumento.documentoInsideToString(documento));
		validateObjetoInsideVersion(documento);
		logger.debug ("documento tras validateObjetoInsideVersion(documento); : " + InsideConverterDocumento.documentoInsideToString(documento));
		documento = validate(documento);
		logger.debug ("documento tras  validate(documento); : " + InsideConverterDocumento.documentoInsideToString(documento));
		documento = converToRepository(documento);
		logger.debug ("documento tras  converToRepository(documento); : " + InsideConverterDocumento.documentoInsideToString(documento));
		documento = insideStore.storeObject(documento);		
		logger.debug ("documento tras  insideStore.storeObject(documento); : " + InsideConverterDocumento.documentoInsideToString(documento));
		documento = convertFromRepository(documento);
		logger.debug ("documento tras  convertFromRepository(documento) : " + InsideConverterDocumento.documentoInsideToString(documento));
		logger.debug ("Fin operacion altaDocumento");
		return documento;
	}

	//TODO esto en una transaccion
	@Override
	public ObjetoDocumentoInside altaDocumento(ObjetoDocumentoInside documento,String identificadorExpediente, String ruta)
		throws InSideServiceException 
	{
		logger.debug ("Inicio operacion altaDocumento");
		logger.debug ("Parametro <documento> : " + InsideConverterDocumento.documentoInsideToString(documento));
		logger.debug ("Parametro <identificadorExpediente> : " + identificadorExpediente);
		logger.debug ("Parametro <ruta> : " + ruta);
		
		ObjetoExpedienteInside expediente = getExpediente(identificadorExpediente);
		logger.debug ("expediente tras getExpediente(identificadorExpediente) : " + InsideConverterExpediente.expedienteInsideToString(expediente));
		
		documento = altaDocumento(documento);
		logger.debug ("documento tras altaDocumento(documento) : " + InsideConverterDocumento.documentoInsideToString(documento));
		
		ArrayList<ObjetoDocumentoInside> documentos = new ArrayList<ObjetoDocumentoInside>();
		documentos.add(documento);
		vincularDocumentosEnExpediente(expediente,documentos,ruta);
		logger.debug ("documento tras vincularDocumentosEnExpediente(expediente,documentos,ruta); : " + InsideConverterDocumento.documentoInsideToString(documento));
		logger.debug ("Fin operacion altaDocumento");
		return documento;
	}
	
	@Override 
	public ObjetoDocumentoInside validarDocumento (ObjetoDocumentoInside documento) throws InSideServiceException {
		logger.debug ("Inicio operacion validarDocumento");
		logger.debug ("Parametro <documento> : " + InsideConverterDocumento.documentoInsideToString(documento));
		documento = validate(documento);
		logger.debug ("documento tras validate(documento); : " + InsideConverterDocumento.documentoInsideToString(documento));
		logger.debug ("Fin operacion validarDocumento");
		return documento;		
	}
	
	@Override
	public ObjetoExpedienteInside validarExpediente (ObjetoExpedienteInside expediente) throws InSideServiceException {
		logger.debug ("Inicio operacion validarExpediente");
		logger.debug ("Parametro <expediente> : " + InsideConverterExpediente.expedienteInsideToString(expediente));
		expediente = validate(expediente);
		logger.debug ("expediente tras validate(expediente) : " + expediente);
		logger.debug ("Fin operacion validarExpediente");
		return expediente;
	}
	
	@Override
	public ObjetoExpedienteInside obtenerVisualizacionIndice (ObjetoExpedienteInside expediente, boolean estamparImagen, boolean estamparNombreOrganismo, List<String> lineasNombreOrganismo, boolean expedienteExterno) throws InSideServiceException {
		logger.debug ("Inicio operacion obtenerVisualizacionIndice");
		logger.debug ("Parametro <expediente> : " + InsideConverterExpediente.expedienteInsideToString(expediente));
		logger.debug ("Parametro <estamparImagen> : " + estamparImagen);
		logger.debug ("Parametro <estamparNombreOrganismo> : " + estamparNombreOrganismo);
		logger.debug ("Parametro <lineasNombreOrganismo> : " + lineasNombreOrganismo);
		logger.debug ("Parametro <expedienteExterno> : " + expedienteExterno);
		
		OpcionesVisualizacionIndice opciones = new OpcionesVisualizacionIndice ();
		opciones.setEstamparImagen(estamparImagen);
		opciones.setEstamparNombreOrganismo(estamparNombreOrganismo);
		opciones.setLineasNombreOrganismo(lineasNombreOrganismo);
		opciones.setExterno(expedienteExterno);
		expediente = insideVisualizacion.expedienteConVisualizacionIndice(expediente, opciones);
		logger.debug ("expediente tras insideVisualizacion.expedienteConVisualizacionIndice(expediente, opciones); : " + InsideConverterExpediente.expedienteInsideToString(expediente));
		logger.debug ("Fin operacion obtenerVisualizacionIndice");
		return expediente;
		
	}
	@Override
	public ObjetoExpedienteInside vincularDocumentosEnExpediente(String identificadorExpediente,List<String> identificadoresDocumentos,String ruta)
		throws InSideServiceException
	{
		logger.debug ("Inicio operacion vincularDocumentosEnExpediente");
		logger.debug ("Parametro <identificadorExpediente> : " + identificadorExpediente);
		logger.debug ("Parametro <identificadoresDocumentos> : " + InsideUtils.listaToString(identificadoresDocumentos.toArray(), ",", false, true));
		logger.debug ("Parametro <ruta> : " + ruta);
		
		ObjetoExpedienteInside expediente = getExpediente(identificadorExpediente);
		logger.debug ("expediente tras  getExpediente(identificadorExpediente) : " + InsideConverterExpediente.expedienteInsideToString(expediente));
	
		Collection<ObjetoDocumentoInside> documentos = new ArrayList<ObjetoDocumentoInside>();
		for(String indentificadorDocumento : identificadoresDocumentos){
			documentos.add(getDocumento(indentificadorDocumento));
		}
		
		expediente = vincularDocumentosEnExpediente(expediente,documentos,ruta);
		logger.debug ("expediente tras  vincularDocumentosEnExpediente(expediente,documentos,ruta); : " + InsideConverterExpediente.expedienteInsideToString(expediente));
		logger.debug ("Fin operacion vincularDocumentosEnExpediente");
		return expediente;
	}
	@Override
	public ObjetoExpedienteInside vincularDocumentosEnExpediente(ObjetoExpedienteInside expediente, Collection<ObjetoDocumentoInside> documentos,String ruta)
		throws InSideServiceException
	{
		logger.debug ("Inicio operacion vincularDocumentosEnExpediente");
		logger.debug ("Parametro <expediente> : " + InsideConverterExpediente.expedienteInsideToString(expediente));
		logger.debug ("Parametro <documentos> : " + InsideUtils.listaToString(documentos.toArray(), ",", false, true));
		logger.debug ("Parametro <ruta> : " + ruta);
		
		ObjetoExpedienteInsideIndiceContenidoElementoContenedorElementos destino = getCarpetaOExpedienteDeRuta(expediente,ruta);
		logger.debug ("destino tras getCarpetaOExpedienteDeRuta(expediente,ruta); : " + destino.toString());
		
		int orden = destino.getElementosIndizados().size();
			
		for (ObjetoDocumentoInside documento : documentos) {
			ObjetoExpedienteInsideIndiceContenidoDocumentoIndizado documentoIndizado = new ObjetoExpedienteInsideIndiceContenidoDocumentoIndizado();
			documentoIndizado.setFechaIncorporacionExpediente(GregorianCalendar.getInstance());
			documentoIndizado.setIdentificadorDocumento(documento.getIdentificador());
			// TODO Cuando el contenido del documento tenga una referencia, quizá habría que obtener el valor binario de esa referencia. De momento lo dejamos así.
			byte[] bytesToDigest = (documento.getContenido().getValorBinario() != null) ? documento.getContenido().getValorBinario() : documento.getContenido().getReferencia().getBytes();
			
			documentoIndizado.setValorHuella(InsideUtils.getStringDigest(this.messageDigest.digest(bytesToDigest)));
			documentoIndizado.setFuncionResumen(this.getFuncionResumenDocumento());
			documentoIndizado.setOrden(++orden);
			destino.getElementosIndizados().add(documentoIndizado);
		}
		
		logger.debug ("destino tras agregar documentos : " + destino.toString());
		
		firmarIndiceExpediente (expediente); 		
		
		logger.debug ("expediente tras firmarIndiceExpediente(expediente) : " + InsideConverterExpediente.expedienteInsideToString(expediente));
		
		expediente = modificaExpedienteInside(expediente);
		logger.debug ("expediente tras modificaExpedienteInside(expediente) : " + InsideConverterExpediente.expedienteInsideToString(expediente));
		logger.debug ("Fin operacion vincularDocumentosEnExpediente");
		return expediente;
	}
	
	public void firmarIndiceExpediente (ObjetoExpedienteInside expediente) throws InSideServiceException {

		try {
			// Necesitamos firmar el índice del Eni			
			TipoIndice indiceEni = InsideConverterIndice.indiceInsideToEni(expediente.getIndice(), expediente.getIdentificador());
			
			ObjectFactory of = new ObjectFactory();
			JAXBElement<TipoIndiceContenido> jaxbElement = of.createIndiceContenido(indiceEni.getIndiceContenido());
			
			JAXBContext jc = JAXBContext.newInstance(TipoIndiceContenido.class);			
			Marshaller marshaller = jc.createMarshaller();
			marshaller.setProperty(Marshaller.JAXB_FRAGMENT, Boolean.TRUE);
			StringWriter sw = new StringWriter ();
			marshaller.marshal(jaxbElement, sw);
			
			/*String firmaIndice = insideSigner.firmarCadena(sw.toString(),
															null,
															"CAdES",
															"IMPLICIT"
														   );*/

			
			byte[] firmaIndice = insideSigner.firmarFichero(sw.toString().getBytes(),
					null,
					"CAdES",
					"IMPLICIT"
				   );

			// Metemos la firma en el expediente			
			FirmaInside firmaInside = new FirmaInside ();
			firmaInside.setTipoFirma(FirmaInsideTipoFirmaEnum.TF_05);

			ContenidoFirmaCertificadoContenidoBinarioInside firmaConCertificado = new ContenidoFirmaCertificadoContenidoBinarioInside();
			//firmaConCertificado.setValorBinario(firmaIndice.getBytes());
			firmaConCertificado.setValorBinario(firmaIndice);
			firmaConCertificado.setMime("application/octect-stream");
			
			firmaInside.setContenidoFirma(firmaConCertificado);
			
			// Sólo guardamos la última firma, que es la que acabamos de hacer.
			expediente.getIndice().getFirmas().clear();
			expediente.getIndice().getFirmas().add(firmaInside);
			

		} catch (Exception e) {
			throw new InsideServiceInternalException ("Se ha producido un error al firmar el índice", e);
		}
	}
	
	
	@Override
	public ObjetoDocumentoInside firmaServidorDocumento(ObjetoDocumentoInside documento)
		throws InSideServiceException 
	{
		//¿Hace falta? documento = validate(documento);
		logger.debug ("Inicio operacion firmaServidorDocumento");
		logger.debug ("Parametro <documento> : " + InsideConverterDocumento.documentoInsideToString(documento));
		documento = sign(documento);
		logger.debug ("documento tras sign(documento) : " + InsideConverterDocumento.documentoInsideToString(documento));
		logger.debug ("Fin operacion firmaServidorDocumento");
		return documento;
	}
	

	@Override
	public ObjetoDocumentoInside getDocumento(String identificador)
		throws InSideServiceException
	{
		logger.debug ("Inicio operacion getDocumento");
		logger.debug ("Parametro <identificador> : " + identificador);		
		ObjetoDocumentoInside documento = getObjetoInside(ObjetoDocumentoInside.class,identificador,null);
		logger.debug ("documento tras getObjetoInside(ObjetoDocumentoInside.class,identificador,null) : " + InsideConverterDocumento.documentoInsideToString(documento));
		documento = convertFromRepository(documento);		
		logger.debug ("documento tras convertFromRepository(documento); : " + InsideConverterDocumento.documentoInsideToString(documento));
		logger.debug ("Fin operacion getDocumento");
		return documento;
	}

	@Override
	public ObjetoDocumentoInside getDocumento(String identificador, Integer version)
		throws InSideServiceException
	{
		logger.debug ("Inicio operacion getDocumento");
		logger.debug ("Parametro <identificador> : " + identificador);
		logger.debug ("Parametro <version> : " + version);
		
		ObjetoDocumentoInside documento = getObjetoInside(ObjetoDocumentoInside.class,identificador,version);
		logger.debug ("documento tras getObjetoInside(ObjetoDocumentoInside.class,identificador,version) : " + InsideConverterDocumento.documentoInsideToString(documento));
		documento = convertFromRepository(documento);
		logger.debug ("documento tras convertFromRepository(documento); : " + InsideConverterDocumento.documentoInsideToString(documento));
		logger.debug ("Fin operacion getDocumento");
		return documento;
	}
	
	@Override
	public ObjetoDocumentoInsideContenido getDocumentoContenido (String identificador, Integer version) throws InSideServiceException{
		logger.debug ("Inicio operacion getDocumentoContenido");
		logger.debug ("Parametro <identificador> : " + identificador);
		logger.debug ("Parametro <version> : " + version);
		
		ObjetoDocumentoInside documento = getObjetoInside(ObjetoDocumentoInside.class,identificador,version);
		logger.debug ("documento tras getObjetoInside(ObjetoDocumentoInside.class,identificador,version) : " + InsideConverterDocumento.documentoInsideToString(documento));
		
		documento = convertFromRepository(documento);
		logger.debug ("documento tras convertFromRepository(documento); : " + InsideConverterDocumento.documentoInsideToString(documento));
		
		ObjetoDocumentoInsideContenido contenido = null;
		try {
			contenido = InsideConverterDocumento.documentoInsideToObjetoDocumentoInsideContenido(documento, infoFirmaService);
			logger.debug ("contenido tras documentoInsideToObjetoDocumentoInsideContenido(documento, infoFirmaService);; : " + contenido);
		} catch (InsideConverterDocumentoInsideToDocumentoContenidoException e) {
			throw new InSideServiceContentExtractException ("Error al extraer el contenido del documento firmado" , e);
		}
		logger.debug ("Fin operacion getDocumentoContenido");
		return contenido;
	}

	@Override
	public ObjetoDocumentoInsideMetadatos getMetadatosDocumento(String identificador) 
		throws InSideServiceException
	{
		logger.debug ("Inicio operacion getMetadatosDocumento");
		logger.debug ("Parametro <identificador> : " + identificador);
		ObjetoDocumentoInsideMetadatos metadatos = insideStore.getObjectMetadatos(ObjetoDocumentoInside.class,identificador);
		logger.debug ("metadatos tras insideStore.getObjectMetadatos(ObjetoDocumentoInside.class,identificador); : " + metadatos.toString());
		logger.debug ("Fin operacion getMetadatosDocumento");
		return metadatos;
	}

	@Override
	public ObjetoDocumentoInsideMetadatos getMetadatosDocumento(String identificador, Integer version)
		throws InSideServiceException {
		logger.debug ("Inicio operacion getMetadatosDocumento");
		logger.debug ("Parametro <identificador> : " + identificador);
		logger.debug ("Parametro <version> : " + version);
		ObjetoDocumentoInsideMetadatos metadatos = insideStore.getObjectMetadatos(ObjetoDocumentoInside.class,identificador,version);
		logger.debug ("metadatos tras insideStore.getObjectMetadatos(ObjetoDocumentoInside.class,identificador, version); : " + metadatos.toString());
		logger.debug ("Fin operacion getMetadatosDocumento");
		return metadatos;
	}

	@Override
	public ObjetoDocumentoInside modificaDocumento(ObjetoDocumentoInside documento)
		throws InSideServiceException {
		logger.debug ("Inicio operacion modificaDocumento");
		logger.debug ("Parametro <documento> : " + InsideConverterDocumento.documentoInsideToString(documento));
		
		documento = validate(documento);		
		logger.debug ("documento tras validate(documento) : " + InsideConverterDocumento.documentoInsideToString(documento));
		documento = converToRepository(documento);
		logger.debug ("documento tras converToRepository(documento) : " + InsideConverterDocumento.documentoInsideToString(documento));
		documento = insideStore.updateObject(documento);
		logger.debug ("documento tras insideStore.updateObject(documento) : " + InsideConverterDocumento.documentoInsideToString(documento));
		documento = convertFromRepository(documento);
		logger.debug ("documento tras converFromRepository(documento) : " + InsideConverterDocumento.documentoInsideToString(documento));
		logger.debug ("Fin operacion modificaDocumento");
		return documento;
		
	}

	@Override
	public FoliadoInside getFoliadoExpediente(String identificador)
		throws InSideServiceException 
	{	
		logger.debug ("Inicio operacion getFoliadoExpediente");
		logger.debug ("Parametro <identificador> : " + identificador);
		ObjetoExpedienteInside expediente = getExpediente(identificador);
		logger.debug ("expediente tras getExpediente(identificador) : " + InsideConverterExpediente.expedienteInsideToString(expediente));
		FoliadoNodo foliadoNodo = InsideServiceIndiceFoliadoConverter.getEstructuraFoliadoNodo (identificador, expediente.getIndice(),this);		
		FoliadoInside foliadoInside =  insideFoliado.foliar(expediente.getIdentificador(), expediente.getMetadatos(), foliadoNodo);
		logger.debug ("foliadoInside : " + foliadoInside.toString());
		logger.debug ("Fin operacion getFoliadoExpediente");
		return foliadoInside;
	}
	
	@Override
	public List<ObjetoInsideVersion> getVersionesDocumento(String identificador)
			throws InSideServiceException {
		logger.debug ("Inicio operacion getVersionesDocumento");
		logger.debug ("Parametro <identificador> : " + identificador);
		List<ObjetoInsideVersion> versiones = insideStore.getObjectVersions(ObjetoDocumentoInside.class, identificador);
		logger.debug ("versiones tras  insideStore.getObjectVersions(ObjetoDocumentoInside.class, identificador) : " + InsideUtils.listaToString(versiones.toArray(), ", ", false, true));
		logger.debug ("Fin operacion getVersionesDocumento");
		return versiones;
	}
	
	@Override
	public ResultadoBusquedaInside<ObjetoExpedienteInsideMetadatos> buscarExpedientes(ConsultaInside consulta,int limite, int pagina)
		throws InSideServiceException
	{
	
		ResultadoBusquedaInside<ObjetoExpedienteInsideMetadatos> res = insideStore.queryObjects(consulta,ObjetoExpedienteInside.class);
		return res;
	}
	
	@Override
	public ResultadoBusquedaInside<ObjetoDocumentoInsideMetadatos> buscarDocumentos(ConsultaInside consulta,int limite, int pagina)
		throws InSideServiceException
	{
		ResultadoBusquedaInside<ObjetoDocumentoInsideMetadatos> res = insideStore.queryObjects(consulta,ObjetoDocumentoInside.class);
		return res;
	}
	
	@Override
	public List<ResultadoValidacionExpediente> validaExpedienteEniFile (byte[] expediente, List<OpcionValidacionExpediente> opciones) throws InSideServiceException {
		List<ResultadoValidacionExpediente> resultados = new ArrayList<ResultadoValidacionExpediente> ();
				
		// Validación contra schemas
		if (opciones.contains(OpcionValidacionExpediente.TOVE_01)) {
			boolean valido = true;
			String mensaje = "Validación Correcta";
			try {
				valido = insideEniValidationService.validarExpedienteDocumentoEniFile(expediente);				
			} catch (InsideEniValidationException e) {
				valido = false;
				mensaje = e.getMessage();				
			}
			ResultadoValidacionExpediente res = new ResultadoValidacionExpediente ();
			res.setTipoValidacion(OpcionValidacionExpediente.TOVE_01);
			res.setValido(valido);
			res.setMensaje(mensaje);
			resultados.add(res);
		}
		
		return resultados;

	}
	
	@Override
	public List<ResultadoValidacionDocumento> validaDocumentoEniFile (byte[] documento, List<OpcionValidacionDocumento> opciones) throws InSideServiceException {
		
		List<ResultadoValidacionDocumento> resultados = new ArrayList<ResultadoValidacionDocumento> ();
		
		// Validacion contra schemas
		if (opciones.contains(OpcionValidacionDocumento.TOVD_01)) {
			boolean valido = true;
			String mensaje = "Validación Correcta";
			try {
				valido = insideEniValidationService.validarExpedienteDocumentoEniFile(documento);				
			} catch (InsideEniValidationException e) {
				valido = false;
				mensaje = e.getMessage();				
			}
			ResultadoValidacionDocumento res = new ResultadoValidacionDocumento ();
			res.setTipoValidacion(OpcionValidacionDocumento.TOVD_01);
			res.setValido(valido);
			res.setMensaje(mensaje);
			resultados.add(res);
		}
		
		return resultados;
	}
	
	/**
	 * Devuelve la visualización de un documento Inside.
	 * 
	 * @param documento Documento Inside que se quiere visualizar
	 * @param estamparImagen Indica si se desea estampar el logo del Ministerio.
	 * @param estamparNombreOrganismo Indica si se quiere estampar el nombre del organismo
	 * @param lineasNombreOrganismo Nombre del organismo, separado en las líneas en que se desee que se estampen.
	 * 
	 */
	@Override
	public ResultadoVisualizacionDocumento visualizaDocumentoInside (ObjetoDocumentoInside documento, 
																     boolean estamparImagen, 
																     boolean estamparNombreOrganismo,
																     List<String> lineasNombreOrganismo) throws InSideServiceException {
		
		logger.debug ("Inicio operacion visualizaDocumentoInside");
		logger.debug ("Parametro <documento> : " + InsideConverterDocumento.documentoInsideToString(documento));
		logger.debug ("Parametro <estamparImagen> : " + estamparImagen);
		logger.debug ("Parametro <estamparNombreOrganismo> : " + estamparNombreOrganismo);
		logger.debug ("Parametro <lineasNombreOrganismo> : " + InsideUtils.listaToString(lineasNombreOrganismo.toArray(), ",", false, true));
		
		OpcionesVisualizacionIndice opciones = new OpcionesVisualizacionIndice ();		
		opciones.setEstamparImagen(estamparImagen);
		opciones.setEstamparNombreOrganismo(estamparNombreOrganismo);
		opciones.setLineasNombreOrganismo(lineasNombreOrganismo);
		
		ResultadoVisualizacionDocumento resultadoVisualizacionDocumento = insideVisualizacion.visualizacionDocumento(documento, opciones);
		logger.debug ("resultadoVisualizacionDocumento : " +  resultadoVisualizacionDocumento);
		logger.debug ("Fin operacion visualizaDocumentoInside");
		return resultadoVisualizacionDocumento;
		
	}
	

	/**
	 * Devuelve la carpeta o el expediente indizado de un indice en la 
	 * ruta pasada como parámetro
	 * 
	 * @param objetoExpedienteInsideIndice
	 * @param ruta
	 * @return
	 * @throws InSideServiceRequestErrorException
	 */
	private ObjetoExpedienteInsideIndiceContenidoElementoContenedorElementos getCarpetaOExpedienteDeRuta(ObjetoExpedienteInside expediente, String ruta) 
		throws InSideServiceRequestErrorException 
	{
		ObjetoExpedienteInsideIndiceContenidoElementoIndizado ret = getElementoIndizado(expediente.getIndice(),ruta);
		
		if(ret instanceof ObjetoExpedienteInsideIndiceContenidoElementoContenedorElementos){
			return (ObjetoExpedienteInsideIndiceContenidoElementoContenedorElementos) ret;
		}
		throw new InSideServiceRequestErrorException("No se encuentra carpeta o expediente en la ruta " + ruta);
		
	}
	
	/**
	 * Devueleve el elemento del indice que se corresponde con una ruta
	 * @param objetoExpedienteInsideIndice
	 * @param ruta
	 * @return
	 */
	private ObjetoExpedienteInsideIndiceContenidoElementoIndizado getElementoIndizado(ObjetoExpedienteInsideIndice objetoExpedienteInsideIndice, String ruta) 
	{
		if(ruta.startsWith("/")){
			ruta = ruta.substring(1);
		}
		String[] path = ruta.split("/");
		if(StringUtils.isEmpty(ruta) || path.length == 0){
			return objetoExpedienteInsideIndice.getIndiceContenido();
		}
		return getElementoIndizado(objetoExpedienteInsideIndice.getIndiceContenido().getElementosIndizados(),path,0);
	}
	
	private ObjetoExpedienteInsideIndiceContenidoElementoIndizado getElementoIndizado(List<ObjetoExpedienteInsideIndiceContenidoElementoIndizado> elementosIndizados,String[] path, int nivel) 
	{
		Assert.notNull(elementosIndizados);
		Assert.notNull(path);
		Assert.notNull(nivel);
		if(nivel > path.length){
			throw new IndexOutOfBoundsException("El nivel (" + nivel + ") no puede ser superior a la longitud de path (" + path.length + ")");
		}
		String elemNombre = path[nivel];
		String compare = "";
		
		for(ObjetoExpedienteInsideIndiceContenidoElementoIndizado elemento : elementosIndizados){
			if(elemento instanceof ObjetoExpedienteInsideIndiceContenido){
				ObjetoExpedienteInsideIndiceContenido indiceIndizado = (ObjetoExpedienteInsideIndiceContenido) elemento;
				compare = indiceIndizado.getIdentificadorExpedienteAsociado();
			}else if(elemento instanceof ObjetoExpedienteInsideIndiceContenidoCarpetaIndizada){
				ObjetoExpedienteInsideIndiceContenidoCarpetaIndizada carpetaIndizada = (ObjetoExpedienteInsideIndiceContenidoCarpetaIndizada) elemento;
				compare = carpetaIndizada.getIdentificadorCarpeta();
			}else if(elemento instanceof ObjetoExpedienteInsideIndiceContenidoDocumentoIndizado){
				ObjetoExpedienteInsideIndiceContenidoDocumentoIndizado documentoIndizado = (ObjetoExpedienteInsideIndiceContenidoDocumentoIndizado) elemento;
				compare = documentoIndizado.getIdentificadorDocumento();
			}
			if(StringUtils.equals(compare,elemNombre)){
				if((nivel + 1) == path.length){
					return elemento;
				}else if(elemento instanceof ObjetoExpedienteInsideIndiceContenidoElementoContenedorElementos){
					return getElementoIndizado(((ObjetoExpedienteInsideIndiceContenidoElementoContenedorElementos)elemento).getElementosIndizados(), path, nivel+1);
				}
			}
		}
		
		return null;
	}


	/**
	 * Nos aseguramos de que los objetos siempre se den de alta con un identificador
	 * @param objeto
	 */
	private void validateObjetoInsideIdentificador(ObjetoInside<?> objeto)
		throws InSideServiceException 
	{
		if (objeto.getIdentificador() == null) {
			objeto.setIdentificador(InsideServiceIdentifierGenerator.generateObjetoInsideIdentificador(objeto));
			logger.debug("El objeto " + objeto.getClass().getSimpleName() + " no tenía identificador, le asigno el: " + objeto.getIdentificador());
		}
	}
	
	/**
	 * Valida que un objeto inside tenga versón
	 * @param objeto
	 */
	private void validateObjetoInsideVersion(ObjetoInside<?> objeto)
	{
		if((objeto instanceof ObjetoInsideVersionable) && ((ObjetoInsideVersionable)objeto).getVersion() == null){
			((ObjetoInsideVersionable)objeto).setVersion(ObjetoInsideVersion.createFirstVersion());
			logger.debug("Creada primera versión para objeto de tipo " + objeto.getClass().getSimpleName() + " con identificador " + objeto.getIdentificador());
		}

	}
	
	/**
	 * Recupera un objeto del repositorio, si la version es inferior
	 * a 1 se llama al getObject (sin version)
	 * @param tipo
	 * @param identificador
	 * @param version
	 * @return
	 * @throws InsideServiceStoreException
	 * @throws InsideStoreObjectNotFoundException
	 */
	private <T extends ObjetoInside<?>> T getObjetoInside(Class<T> tipo, String identificador,Integer version) throws InsideServiceStoreException, InsideStoreObjectNotFoundException
	{
		if(version != null && version >= 1){
			return insideStore.getObjectVersion(tipo, identificador, version);
		}else{
			return insideStore.getObject(tipo, identificador);
		}
	}
	
	private <T extends ObjetoInside<K>, K extends ObjetoInsideMetadatos> T validate(T objeto) throws InSideServiceObjectDefinitionNotFoundException, InSideServiceValidationException, InSideServiceException{
		@SuppressWarnings("unchecked")
		T objetoValidado = (T) objectDefinitions.getValidator(objeto.getClass()).validate(objeto);
		return objetoValidado;
	}
	
	private <T extends ObjetoInside<?>> T converToRepository(T objeto) throws InSideServiceObjectDefinitionNotFoundException, InsideServiceObjectConverterException{
		@SuppressWarnings("unchecked")
		T objetoConvertido = (T) objectDefinitions.getConverter(objeto.getClass()).toRepository(objeto);
		return objetoConvertido;
	}
	
	private <T extends ObjetoInside<?>> T convertFromRepository(T objeto) throws InSideServiceObjectDefinitionNotFoundException, InsideServiceObjectConverterException{
		@SuppressWarnings("unchecked")
		T objetoConvertido = (T) objectDefinitions.getConverter(objeto.getClass()).fromRepository(objeto);
		return objetoConvertido;
	}
	
	private <T extends ObjetoInside<?>> T sign (T objeto) throws InSideServiceObjectDefinitionNotFoundException, InsideServiceObjectSignerException {
		@SuppressWarnings("unchecked")
		T objetoFirmado = (T) objectDefinitions.getSigner(objeto.getClass()).sign(objeto);
		return objetoFirmado;
	}
	
	public String getFuncionResumenDocumento() {
		return this.funcionResumenDocumentos;
	}
	
	/**
	 * Establece el Algoritmo que se utiliza para calcular
	 * la huella digital de los documentos para el indice
	 * del Expediente electrónico
	 * 
	 * @param funcionResumen
	 * @throws NoSuchAlgorithmException 
	 */
	@Required
	public void setFuncionResumenDocumentos(String funcionResumen) throws NoSuchAlgorithmException {
		logger.info("Estableciendo función de Resumen para documentos del índice a '" + funcionResumen+ "'");
		this.messageDigest = MessageDigest.getInstance(funcionResumen);
		this.funcionResumenDocumentos = funcionResumen;
	}



}
