/* Copyright (C) 2012-13 MINHAP, Gobierno de España
   This program is licensed and may be used, modified and redistributed under the terms
   of the European Public License (EUPL), either version 1.1 or (at your
   option) any later version as soon as they are approved by the European Commission.
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
   or implied. See the License for the specific language governing permissions and
   more details.
   You should have received a copy of the EUPL1.1 license
   along with this program; if not, you may find it at
   http://joinup.ec.europa.eu/software/page/eupl/licence-eupl */

package es.mpt.dsic.inside.service.impl;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Properties;

import javax.annotation.PostConstruct;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Required;


import es.mpt.dsic.firma.cliente.model.ApplicationLogin;
import es.mpt.dsic.firma.cliente.model.ContenidoSalida;
import es.mpt.dsic.firma.cliente.model.DatosEntradaFichero;
import es.mpt.dsic.firma.cliente.model.DatosSalida;
import es.mpt.dsic.firma.cliente.model.ResultadoFirmaFichero;
import es.mpt.dsic.firma.cliente.service.EeUtilFirmarService;
import es.mpt.dsic.firma.cliente.service.impl.EeUtilFirmarServiceImplService;
import es.mpt.dsic.infofirma.service.exception.InfoFirmaServiceException;
import es.mpt.dsic.inside.service.exception.InSideServiceSignerException;
import es.mpt.dsic.inside.service.object.signer.InsideServiceSigner;


public class InsideServiceSignerImpl implements InsideServiceSigner{

	protected static final Log logger = LogFactory.getLog(InsideServiceSignerImpl.class);
	
	private boolean activo;
	private static String ACTIVO = "S";
	
	private Properties properties;
	private EeUtilFirmarService port;
	private ApplicationLogin applicationLogin;
	
	@PostConstruct
	public void configureSigner() {
		
		String firmaActivo = properties.getProperty("firma.activo");
		
		if (!ACTIVO.contentEquals(firmaActivo)) {
			
			logger.info("El WS de FIRMA REMOTA no está activo");
			activo = false;
			
		} else {		
		
			URL url = null;
			String urlFirma = null;
			try {
				urlFirma = properties.getProperty("firma.url");
				logger.debug(String.format("El WS de firma remota se encuentra en %s", urlFirma));
				url = new URL (urlFirma);
			} catch (MalformedURLException me) {
				logger.error("No se puede crear la URL del servicio de firma " + urlFirma, me);
			}
			
			EeUtilFirmarServiceImplService ss = new EeUtilFirmarServiceImplService(url);	
			
		    port = ss.getEeUtilFirmarServiceImplPort();  
		    
		    applicationLogin = new ApplicationLogin();
		    applicationLogin.setIdaplicacion(properties.getProperty("firma.idaplicacion"));
		    applicationLogin.setPassword(properties.getProperty("firma.password"));
		    
		    logger.debug(String.format("Utilizando idaplicacion/password : %s/%s", properties.getProperty("firma.idaplicacion"), properties.getProperty("firma.password")));
		    activo = true;
		}
	    
	}
		
	/*@Override
	public String firmarCadena (String cadena, 
								 String algoritmoFirma, 
								 String formatoFirma,		
								 String modoFirma) throws InSideServiceSignerException {
		
		if (!activo) {
			throw new InSideServiceSignerException ("El servicio de firma remota no está activo");
		}
		
		String firma = null;
		
		DatosEntrada de = new DatosEntrada ();
	    de.setContenido(cadena);
	    de.setConvertirB64(true);
	    de.setAlgoritmoFirma(algoritmoFirma);
	    de.setFormatoFirma(formatoFirma);
	    de.setModoFirma(modoFirma);
	    
	    DatosSalida salida = null;
	    try {	    
	    	salida = port.firmaContenido(applicationLogin, de);
	    } catch (Exception e) {
	    	throw new InSideServiceSignerException ("Se ha producido un error en la llamada al servicio de firma remota " , e);	    	
	    }
	    
	    ContenidoSalida cont = salida.getDatosResultado();
	    	
	    if (salida.getEstado().equalsIgnoreCase(properties.getProperty("firma.cadena.ok"))) {
	    	logger.debug ("Cadena firmada correctamente");
	    	ResultadoFirma resultado = (ResultadoFirma) cont;
	    	firma = resultado.getContenidoFirma();
	    } else {
	    	es.mpt.dsic.firma.cliente.model.Error error = (es.mpt.dsic.firma.cliente.model.Error) cont;
	    	throw new InSideServiceSignerException ("No se ha firmado correctamente la cadena " + error.getMensaje() + " " + error.getCausa());
	    }
	    
	    return firma;
	    
		
	}*/
	
	@Override
	public byte [] firmarFichero (byte[] bytesFichero, 
								  String algoritmoFirma, 
								  String formatoFirma,
								  String modoFirma) throws InSideServiceSignerException {
		
		if (!activo) {
			throw new InSideServiceSignerException ("El servicio de firma remota no está activo");
		}
		
		byte[] firma = null;
	    
	    DatosEntradaFichero de = new DatosEntradaFichero ();
	    de.setContenido(bytesFichero);	    
	    de.setAlgoritmoFirma(algoritmoFirma);
	    de.setFormatoFirma(formatoFirma);
	    de.setModoFirma(modoFirma);
	    
	    DatosSalida salida = null;
	    try {	    
	    	salida = port.firmaFichero(applicationLogin, de);
	    } catch (Exception e) {
	    	throw new InSideServiceSignerException ("Se ha producido un error en la llamada al servicio de firma remota " , e);	    	
	    }
	    
	    ContenidoSalida cont = salida.getDatosResultado();
	    	
	    if (salida.getEstado().equalsIgnoreCase(properties.getProperty("firma.cadena.ok"))) {
	    	logger.debug("Fichero firmado correctamente");
	    	ResultadoFirmaFichero resultado = (ResultadoFirmaFichero) cont;
	    	firma = resultado.getFirma();
	    } else {
	    	es.mpt.dsic.firma.cliente.model.Error error = (es.mpt.dsic.firma.cliente.model.Error) cont;
	    	throw new InSideServiceSignerException ("No se ha firmado correctamente el fichero: " + error.getMensaje() + " " + error.getCausa());
	    }
	    
	    return firma;
		
	}

	public Properties getProperties() {
		return properties;
	}

	@Required
	public void setProperties(Properties properties) {
		this.properties = properties;
	}
	
	
}
