/* Copyright (C) 2012-13 MINHAP, Gobierno de España
   This program is licensed and may be used, modified and redistributed under the terms
   of the European Public License (EUPL), either version 1.1 or (at your
   option) any later version as soon as they are approved by the European Commission.
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
   or implied. See the License for the specific language governing permissions and
   more details.
   You should have received a copy of the EUPL1.1 license
   along with this program; if not, you may find it at
   http://joinup.ec.europa.eu/software/page/eupl/licence-eupl */

package es.mpt.dsic.inside.service.object.converter.impl.cmis;

import java.io.StringWriter;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.Marshaller;

import org.apache.chemistry.opencmis.client.api.Document;
import org.apache.chemistry.opencmis.client.api.Folder;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Required;

import es.mpt.dsic.inside.model.converter.InsideConverterExpediente;
import es.mpt.dsic.inside.model.objetos.documento.ObjetoDocumentoInside;
import es.mpt.dsic.inside.model.objetos.documento.ObjetoDocumentoInsideContenido;
import es.mpt.dsic.inside.model.objetos.documento.metadatos.ObjetoDocumentoInsideMetadatos;
import es.mpt.dsic.inside.model.objetos.expediente.ObjetoExpedienteInside;
import es.mpt.dsic.inside.model.objetos.expediente.metadatos.ObjetoExpedienteInsideMetadatos;
import es.mpt.dsic.inside.model.objetos.expediente.metadatos.ObjetoExpedienteInsideMetadatosEnumeracionEstados;
import es.mpt.dsic.inside.model.objetos.firmas.FirmaInside;
import es.mpt.dsic.inside.model.objetos.firmas.contenido.ContenidoFirmaCertificadoAlmacenableInside;
import es.mpt.dsic.inside.model.objetos.firmas.contenido.ContenidoFirmaInside;
import es.mpt.dsic.inside.xml.eni.expediente.ObjectFactory;
import es.mpt.dsic.inside.xml.eni.expediente.TipoExpediente;
import es.mpt.dsic.integration.cmis.adapter.CmisAdapterInterface;
import es.mpt.dsic.integration.cmis.exception.RepositoryCmisException;
import es.mpt.dsic.integration.cmis.model.ContenidoDocumento;

public class InsideServiceCmisAdapter {
	
	protected static final Log logger = LogFactory.getLog(InsideServiceCmisAdapter.class);

	protected String repoObjectId;
	protected String repoVersionLabel;
	protected String repoPathSeparator;
	protected String repoRootFolder;
	
	protected String documentosFolder;
	protected String documentoObjectTypeId;
	protected String documentoMetadatoVersionNti;
	protected String documentoMetadatoIdentificador;
	protected String documentoMetadatoOrgano;
	protected String documentoMetadatoFechaCaptura;
	protected String documentoMetadatoDocOrigen;
	protected String documentoMetadatoDocEstadoElaboracion;
	protected String documentoMetadatoNombreFormato;
	protected String documentoMetadatoTipoDocumental;
	protected String documentoMetadatoDocTipoFirma;
	protected String documentoMetadatoIdentificadorDocOrigen;

	protected String expedientesFolder;
	protected String expedienteObjectTypeId;
	protected String expedienteMetadatoVersionNti;
	protected String expedienteMetadatoIdentificador;
	protected String expedienteMetadatoOrgano;
	protected String expedienteMetadatoFechaApertura;
	protected String expedienteMetadatoClasificacion;
	protected String expedienteMetadatoEstado;
	protected String expedienteMetadatoInteresado;
	
	protected CmisAdapterInterface cmisAdapter;

	
	/**
	 * Almacena una firma en el repositorio CMIS un
	 * conjunto de firmas 
	 * @param firmas		Las firmas a almacenar en el repositorio
	 * @param parentfolder  La carpeta donde iran almacenadas las firmas
	 * @throws RepositoryCmisException 
	 * @throws InsideServiceCmisAdapterException 
	 */
	/*private void almacenaFirmasInside(List<FirmaInside> firmas,Folder parentFolder)
		throws InsideServiceCmisAdapterException
	{
		for (FirmaInside firma : firmas) {
			if(firma.getContenidoFirma() != null && firma.getContenidoFirma() != null){
				ContenidoFirmaInside contenidoFirma = firma.getContenidoFirma();
				if(contenidoFirma instanceof ContenidoFirmaCertificadoAlmacenableInside){
					ContenidoFirmaCertificadoAlmacenableInside contenidoFirmaAlmacenado = (ContenidoFirmaCertificadoAlmacenableInside) contenidoFirma;
					try {
						String firmaName = "" + MessageDigest.getInstance("MD5").digest(contenidoFirmaAlmacenado.getValorBinario());
						if(contenidoFirmaAlmacenado.getIdentificadorRepositorio() != null){
							//TODO ¿hay que modificar los metadatos de la firma?
						}else{
							Document documentoFirmaCmis = cmisAdapter.altaDocumento(firmaName,
																					parentFolder,
																					null,
																					contenidoFirmaAlmacenado.getValorBinario(),
																					contenidoFirmaAlmacenado.getMime(),
																					null,
																					false);
							contenidoFirmaAlmacenado.setValorBinario(null);
							//firma.getContenidoFirma().getFirmaConCertificado().setIdentificadorRepositorio((String) metadatosFirma.get(repoObjectId));
							contenidoFirmaAlmacenado.setIdentificadorRepositorio((String) documentoFirmaCmis.getId());
						}	
						
					} catch (NoSuchAlgorithmException e) {
						throw new InsideServiceCmisAdapterException("No existe el algoritmo para realizar el hash del contenido de la firma", e);
					} catch (RepositoryCmisException e) {
						throw new InsideServiceCmisAdapterException("Error almacenando la firma");
					}
				}
				
			}
		}
	}*/
	
	/**
	 * Almacena una firma en el repositorio CMIS un
	 * conjunto de firmas 
	 * @param firmas		Las firmas a almacenar en el repositorio
	 * @param folder        Objeto folder donde serán almacenadas. 
	 * @throws RepositoryCmisException 
	 * @throws InsideServiceCmisAdapterException 
	 */
	private void almacenaFirmasInside(List<FirmaInside> firmas, Folder folder)
		throws InsideServiceCmisAdapterException
	{
		for (FirmaInside firma : firmas) {
			if(firma.getContenidoFirma() != null && firma.getContenidoFirma() != null){
				ContenidoFirmaInside contenidoFirma = firma.getContenidoFirma();
				if(contenidoFirma instanceof ContenidoFirmaCertificadoAlmacenableInside){
					ContenidoFirmaCertificadoAlmacenableInside contenidoFirmaAlmacenado = (ContenidoFirmaCertificadoAlmacenableInside) contenidoFirma;
					try {
						String firmaName = "" + MessageDigest.getInstance("MD5").digest(contenidoFirmaAlmacenado.getValorBinario()) + "_" + System.currentTimeMillis();
						if(contenidoFirmaAlmacenado.getIdentificadorRepositorio() != null){
							//TODO ¿hay que modificar los metadatos de la firma?
						}else{
							Document documentoFirmaCmis = cmisAdapter.altaDocumento(firmaName,
																                    folder,
																					null,
																					contenidoFirmaAlmacenado.getValorBinario(),
																					contenidoFirmaAlmacenado.getMime(),
																					null,
																					false);
							contenidoFirmaAlmacenado.setValorBinario(null);
							//firma.getContenidoFirma().getFirmaConCertificado().setIdentificadorRepositorio((String) metadatosFirma.get(repoObjectId));
							contenidoFirmaAlmacenado.setIdentificadorRepositorio((String) documentoFirmaCmis.getId());
						}	
						
					} catch (NoSuchAlgorithmException e) {
						throw new InsideServiceCmisAdapterException("No existe el algoritmo para realizar el hash del contenido de la firma", e);
					} catch (RepositoryCmisException e) {
						throw new InsideServiceCmisAdapterException("Error almacenando la firma");
					}
				}
				
			}
		}
	}
	
	private void almacenaExpedienteEni (ObjetoExpedienteInside expediente, Folder folder) throws InsideServiceCmisAdapterException{
		String xmlExpediente = null;
		
		try {
			ObjectFactory of = new ObjectFactory();
			JAXBElement<TipoExpediente> jaxbElement = of.createExpediente(InsideConverterExpediente.expedienteInsideToEni(expediente));
			JAXBContext jc = JAXBContext.newInstance(TipoExpediente.class);
			Marshaller marshaller = jc.createMarshaller();
			marshaller.setProperty(Marshaller.JAXB_FRAGMENT, Boolean.TRUE);
			StringWriter sw = new StringWriter ();
			marshaller.marshal(jaxbElement, sw);
			xmlExpediente = sw.toString();
			
		} catch (Exception e) {
			throw new InsideServiceCmisAdapterException ("Error convirtiendo el ExpedinteInside a xml", e);
		}			
		
		try {
			cmisAdapter.altaDocumento(expediente.getIdentificador() + ".xml",
					folder, 
					null, 
					xmlExpediente.getBytes(), 
					"text/xml", 
					null, 
					false);
		} catch (RepositoryCmisException e) {
			throw new InsideServiceCmisAdapterException ("Error almacenando el cml del ExpedinteInside", e);
		}
	}

	/**
	 * Almacena un documento electrónico en el repositorio CMIS
	 * Si no estaba almacenado crea su carpeta
	 * En caso contrario modifica sus metadatos y almacena firmas nuevas
	 * @param documento
	 * @return
	 * @throws InsideServiceCmisAdapterException
	 */
	/*public ObjetoDocumentoInside almacenaObjetoDocumentoInside(ObjetoDocumentoInside documento) throws InsideServiceCmisAdapterException{
		ObjetoDocumentoInsideMetadatos metadatosEntrada = documento.getMetadatos();

		if (metadatosEntrada == null) {
			throw new InsideServiceCmisAdapterException ("Los metadatos del documento de entrada no pueden ser nulos");
		}

		Map<String, Object> metadatosRepository = new HashMap<String, Object> ();

		metadatosRepository.put(documentoMetadatoVersionNti, metadatosEntrada.getVersionNTI());
		metadatosRepository.put(documentoMetadatoIdentificador, documento.getIdentificador());
		metadatosRepository.put(documentoMetadatoOrgano, metadatosEntrada.getOrgano());
		metadatosRepository.put(documentoMetadatoFechaCaptura, metadatosEntrada.getFechaCaptura());
		metadatosRepository.put(documentoMetadatoDocOrigen, metadatosEntrada.isOrigenCiudadanoAdministracion());
		metadatosRepository.put(documentoMetadatoDocEstadoElaboracion, metadatosEntrada.getEstadoElaboracion().getValorEstadoElaboracion().value());
		metadatosRepository.put(documentoMetadatoNombreFormato, documento.getContenido().getNombreFormato());
		metadatosRepository.put(documentoMetadatoTipoDocumental, metadatosEntrada.getTipoDocumental().value());
		metadatosRepository.put(documentoMetadatoIdentificadorDocOrigen, metadatosEntrada.getEstadoElaboracion().getIdentificadorDocumentoOrigen());

		Document documentoCmis = null;
		if(documento.getIdentificadorRepositorio() != null){
			try {
				documentoCmis = (Document) cmisAdapter.getCmisObjectById(documento.getIdentificadorRepositorio());
				documentoCmis.updateProperties(metadatosRepository);
			} catch (RepositoryCmisException rce) {
				String mensajeError = String.format("No ha podido modificar el documento con identificador %s en el repositorio " + rce.getMessage(), documento.getIdentificador());
				logger.error(mensajeError);
				throw new InsideServiceCmisAdapterException (mensajeError, rce);
			}
		}else{
			try {
				Calendar today = GregorianCalendar.getInstance();
				String[] rutaDocumentoAlta = {repoRootFolder,documentosFolder,String.valueOf(today.get(Calendar.YEAR)),String.valueOf(today.get(Calendar.MONTH)),documento.getIdentificador()};
				documentoCmis = cmisAdapter.altaDocumento(documento.getIdentificador(),
						rutaDocumentoAlta, 
						documentoObjectTypeId, 
						documento.getContenido().getValorBinario(), 
						documento.getContenido().getMime(), 
						metadatosRepository, 
						true);
	
				//documento.setIdentificadorRepositorio((String) metadatosRepository.get(repoObjectId));
				documento.setIdentificadorRepositorio((String) documentoCmis.getId());
	
			} catch (RepositoryCmisException rce) {
				String mensajeError = String.format("No ha podido insertarse el documento con identificador %s en el repositorio: " + rce.getMessage(), documento.getIdentificador());
				logger.error(mensajeError);
				throw new InsideServiceCmisAdapterException (mensajeError, rce);
			}

		}
		this.almacenaFirmasInside(documento.getFirmas(), documentoCmis.getParents().get(0));
		
		return documento;

	}*/
	
	/**
	 * Almacena un documento electrónico en el repositorio CMIS, siempre que éste tenga contenido.
	 * Si no estaba almacenado crea su carpeta	 
	 * En caso contrario modifica sus metadatos y almacena firmas nuevas
	 * @param documento
	 * @return
	 * @throws InsideServiceCmisAdapterException
	 */
	public ObjetoDocumentoInside almacenaObjetoDocumentoInside(ObjetoDocumentoInside documento) throws InsideServiceCmisAdapterException{
		ObjetoDocumentoInsideMetadatos metadatosEntrada = documento.getMetadatos();

		if (metadatosEntrada == null) {
			throw new InsideServiceCmisAdapterException ("Los metadatos del documento de entrada no pueden ser nulos");
		}
		
		Folder folderFirmas = null;
		
		if (documento.getContenido().getValorBinario() != null) {
			Map<String, Object> metadatosRepository = new HashMap<String, Object> ();

			metadatosRepository.put(documentoMetadatoVersionNti, metadatosEntrada.getVersionNTI());
			metadatosRepository.put(documentoMetadatoIdentificador, documento.getIdentificador());
			metadatosRepository.put(documentoMetadatoOrgano, metadatosEntrada.getOrgano());
			metadatosRepository.put(documentoMetadatoFechaCaptura, metadatosEntrada.getFechaCaptura());
			metadatosRepository.put(documentoMetadatoDocOrigen, metadatosEntrada.isOrigenCiudadanoAdministracion());
			metadatosRepository.put(documentoMetadatoDocEstadoElaboracion, metadatosEntrada.getEstadoElaboracion().getValorEstadoElaboracion().value());
			metadatosRepository.put(documentoMetadatoNombreFormato, documento.getContenido().getNombreFormato());
			metadatosRepository.put(documentoMetadatoTipoDocumental, metadatosEntrada.getTipoDocumental().value());
			metadatosRepository.put(documentoMetadatoIdentificadorDocOrigen, metadatosEntrada.getEstadoElaboracion().getIdentificadorDocumentoOrigen());
			
			Document documentoCmis = null;			
			if(documento.getIdentificadorRepositorio() != null){
				try {
					documentoCmis = (Document) cmisAdapter.getCmisObjectById(documento.getIdentificadorRepositorio());
					documentoCmis.updateProperties(metadatosRepository);					
				} catch (RepositoryCmisException rce) {
					String mensajeError = String.format("No ha podido modificar el documento con identificador %s en el repositorio " + rce.getMessage(), documento.getIdentificador());
					logger.error(mensajeError);
					throw new InsideServiceCmisAdapterException (mensajeError, rce);
				}
			}else{
				try {
					Calendar today = GregorianCalendar.getInstance();
					String[] rutaDocumentoAlta = {repoRootFolder,documentosFolder,String.valueOf(today.get(Calendar.YEAR)),String.valueOf(today.get(Calendar.MONTH)),documento.getIdentificador()};
					documentoCmis = cmisAdapter.altaDocumento(documento.getIdentificador() + "_" + System.currentTimeMillis(),
							rutaDocumentoAlta, 
							documentoObjectTypeId, 
							documento.getContenido().getValorBinario(), 
							documento.getContenido().getMime(), 
							metadatosRepository, 
							true);
		
					//documento.setIdentificadorRepositorio((String) metadatosRepository.get(repoObjectId));
					documento.setIdentificadorRepositorio((String) documentoCmis.getId());
		
				} catch (RepositoryCmisException rce) {
					String mensajeError = String.format("No ha podido insertarse el documento con identificador %s en el repositorio: " + rce.getMessage(), documento.getIdentificador());
					logger.error(mensajeError);
					throw new InsideServiceCmisAdapterException (mensajeError, rce);
				}

			}
			folderFirmas = documentoCmis.getParents().get(0);
			
		} else {
			// Como no hemos definido la carpeta donde estarán las firmas, la definimos.
			Calendar today = GregorianCalendar.getInstance();
			String[] rutaFirmas = {repoRootFolder,documentosFolder,String.valueOf(today.get(Calendar.YEAR)),String.valueOf(today.get(Calendar.MONTH)),documento.getIdentificador()};
			try {
				folderFirmas = cmisAdapter.getFolder(rutaFirmas, true);
			} catch (RepositoryCmisException rce) {
				String mensajeError = String.format("No ha podido obtenerse el objeto Folder para almacenar las firmas del documento con identificador %s en el repositorio: " + rce.getMessage(), documento.getIdentificador());
				logger.error(mensajeError);
				throw new InsideServiceCmisAdapterException (mensajeError, rce);
			}
			
		}
		this.almacenaFirmasInside(documento.getFirmas(), folderFirmas);

		
		
		
		return documento;

	}
	
	public void almacenaObjetoExpedienteInside(ObjetoExpedienteInside expediente) throws InsideServiceCmisAdapterException{
			
		ObjetoExpedienteInsideMetadatos metadatosEntrada = expediente.getMetadatos();
		
		if (metadatosEntrada == null) {
			throw new InsideServiceCmisAdapterException ("Los metadatos del Expediente de entrada no pueden ser nulos");
		}
		
		Map<String, Object> metadatosRepository = new HashMap<String, Object> ();
		
		metadatosRepository.put(expedienteMetadatoVersionNti, metadatosEntrada.getVersionNTI());
		metadatosRepository.put(expedienteMetadatoIdentificador, expediente.getIdentificador());
		metadatosRepository.put(expedienteMetadatoOrgano, metadatosEntrada.getOrgano());
		metadatosRepository.put(expedienteMetadatoFechaApertura, metadatosEntrada.getFechaAperturaExpediente());
		metadatosRepository.put(expedienteMetadatoClasificacion, metadatosEntrada.getClasificacion());
		metadatosRepository.put(expedienteMetadatoEstado, metadatosEntrada.getEstado().value());
		metadatosRepository.put(expedienteMetadatoInteresado, metadatosEntrada.getInteresado());
		
		Folder expedienteFolder = null;
		if(expediente.getIdentificadorRepositorio() == null){
			try {
				Calendar today = GregorianCalendar.getInstance();
				String[] rutaExpedienteAlta = {repoRootFolder,expedientesFolder,String.valueOf(today.get(Calendar.YEAR)),String.valueOf(today.get(Calendar.MONTH))};
				expedienteFolder = cmisAdapter.altaCarpeta(	expediente.getIdentificador(),
																rutaExpedienteAlta,
																expedienteObjectTypeId,
																metadatosRepository,
																true);
				//expediente.setIdentificadorRepositorio((String) expedienteFolder.getProperty(repoObjectId).getValue());
				expediente.setIdentificadorRepositorio((String) expedienteFolder.getId());
				
			} catch (RepositoryCmisException rce) {
				String mensajeError = String.format("No se ha podido insertar el expediente con identificador %s en el repositorio " + rce.getMessage(), expediente.getIdentificador());
				logger.error(mensajeError);
				throw new InsideServiceCmisAdapterException (mensajeError, rce);
			}
		}else{
			try {
				expedienteFolder = cmisAdapter.modificaCarpeta(expediente.getIdentificadorRepositorio(), metadatosRepository);
			} catch (RepositoryCmisException rce) {
				String mensajeError = String.format("No ha podido modificar el expediente con identificador %s en el repositorio " + rce.getMessage(), expediente.getIdentificador());
				logger.error(mensajeError);
				throw new InsideServiceCmisAdapterException (mensajeError, rce);
			}
		}
		
		// Al cerrar el expediente generamos su xml para almacenarlo
		// Lo hacemos aquí porque después no vamos a tener el valor binario de las firmas, sino el id de repositorio, y no nos vale para convertir a Eni el expediente.
		// TODO Quizá esta lógica no debería estar aquí.
		if(expediente.getMetadatos().getEstado() == ObjetoExpedienteInsideMetadatosEnumeracionEstados.E_02){
			this.almacenaExpedienteEni(expediente, expedienteFolder);
		}
		
		if (expediente.getIndice().getFirmas() != null) {
			this.almacenaFirmasInside(expediente.getIndice().getFirmas(), expedienteFolder);
		}
		
		/*if(expediente.getMetadatos().getEstado() == ObjetoExpedienteInsideMetadatosEnumeracionEstados.E_02){
			//al cerrar el expediente generamos su xml para almacenarlo
			
			String xmlExpediente = null;
			
			try {
				ObjectFactory of = new ObjectFactory();
				JAXBElement<TipoExpediente> jaxbElement = of.createExpediente(InsideConverterExpediente.expedienteInsideToEni(expediente));
				JAXBContext jc = JAXBContext.newInstance(TipoExpediente.class);
				Marshaller marshaller = jc.createMarshaller();
				marshaller.setProperty(Marshaller.JAXB_FRAGMENT, Boolean.TRUE);
				StringWriter sw = new StringWriter ();
				marshaller.marshal(jaxbElement, sw);
				xmlExpediente = sw.toString();
				
			} catch (Exception e) {
				throw new InsideServiceCmisAdapterException ("Error convirtiendo el ExpedinteInside a xml", e);
			}			
			
			try {
				cmisAdapter.altaDocumento(expediente.getIdentificador() + ".xml",
						expedienteFolder, 
						null, 
						xmlExpediente.getBytes(), 
						"text/xml", 
						null, 
						false);
			} catch (RepositoryCmisException e) {
				throw new InsideServiceCmisAdapterException ("Error almacenando el cml del ExpedinteInside", e);
			}
			
		}*/
		
		
		
		
		/*
		if(expediente.getIndice() != null && expediente.getIndice().getIndiceContenido() != null && expediente.getIndice().getIndiceContenido().getElementoIndizado() != null){
			List<String> carpetasExpediente = new ArrayList<String>();
			
			try {
				for(Map<String,Object> propsCarpeta :cmisAdapter.getFoldersCarpeta(expediente.getIdentificadorRepositorio())){
					carpetasExpediente.add("TODO!!!");
				}
			} catch (RepositoryCmisException e) {
				throw new InsideServiceCmisAdapterException ("Error obteniendo las carpetas de los documentos en la carpeta del expediente",e);
			}
			
			for(ObjetoExpedienteInsideIndiceContenidoElementoIndizado elemento : expediente.getIndice().getIndiceContenido().getElementoIndizado()){
				if(elemento instanceof ObjetoExpedienteInsideIndiceContenidoDocumentoIndizado){
					String identificadorDomento = ((ObjetoExpedienteInsideIndiceContenidoDocumentoIndizado) elemento).getIdentificadorDocumento();
					if(!carpetasExpediente.contains(identificadorDomento)){
						logger.info("Copiando el documento " + identificadorDomento + " a la carpeta del expediente " + expediente.getIdentificador());
					}
				}
			}
		}*/
		
	}
	
	/*public void obtenerDocumentoInside(ObjetoDocumentoInside documento) throws InsideServiceCmisAdapterException {
		if (StringUtils.isEmpty(documento.getIdentificadorRepositorio())) {
			throw new InsideServiceCmisAdapterException ("El identificador del repositorio del documento no puede ser nulo");
		}
		
		Map<String, Object> metadatosRepository = null;
		ContenidoDocumento contenidoDocumento  = null;
		
		try {		
			// Metadatos del documento
			metadatosRepository = cmisAdapter.getInfoDocumento(documento.getIdentificadorRepositorio());
			// Bytes del documento
			contenidoDocumento = cmisAdapter.getContenidoDoc(documento.getIdentificadorRepositorio());
			// Firmas del documento
			for (FirmaInside firma : documento.getFirmas()) {
				ContenidoFirmaInside contenidoFirma = firma.getContenidoFirma();
				if(contenidoFirma != null && contenidoFirma instanceof ContenidoFirmaCertificadoAlmacenableInside)
				{
					ContenidoFirmaCertificadoAlmacenableInside contenidoFirmaAlmacenado = (ContenidoFirmaCertificadoAlmacenableInside) contenidoFirma;
					byte[] bytesFirma = cmisAdapter.getContenidoDoc(contenidoFirmaAlmacenado.getIdentificadorRepositorio()).getContenido();	
					contenidoFirmaAlmacenado.setValorBinario(bytesFirma);
				}
			}
			
		} catch (RepositoryCmisException rce) {
			String mensajeError = String.format("No ha podido obtenerse el documento con identificador %s del repositorio " + rce.getMessage(), documento.getIdentificador());
			logger.error(mensajeError);
			throw new InsideServiceCmisAdapterException (mensajeError, rce);
		} 
		
		if (documento.getContenido() == null) {
			ObjetoDocumentoInsideContenido contenidoInside = new ObjetoDocumentoInsideContenido ();
			documento.setContenido(contenidoInside);
		}
		
		documento.getContenido().setValorBinario(contenidoDocumento.getContenido());
		documento.getContenido().setNombreFormato((String)metadatosRepository.get(this.documentoMetadatoNombreFormato));
		
		logger.debug("convertFromRepository: documento con identificador de Repositorio " + documento.getIdentificadorRepositorio() + " convertido correctamente");

	}*/
	
	public void obtenerDocumentoInside(ObjetoDocumentoInside documento) throws InsideServiceCmisAdapterException {
		
		if (documento.getIdentificadorRepositorio() != null) {
		
			Map<String, Object> metadatosRepository = null;
			ContenidoDocumento contenidoDocumento  = null;

			try {		
				// Metadatos del documento
				metadatosRepository = cmisAdapter.getInfoDocumento(documento.getIdentificadorRepositorio());
				// Bytes del documento
				contenidoDocumento = cmisAdapter.getContenidoDoc(documento.getIdentificadorRepositorio());
				
				
			} catch (RepositoryCmisException rce) {
					String mensajeError = String.format("No ha podido obtenerse el documento con identificador %s " + rce.getMessage(), documento.getIdentificador());
					logger.error(mensajeError);
					throw new InsideServiceCmisAdapterException (mensajeError, rce);
			}
			
			if (documento.getContenido() == null) {
				ObjetoDocumentoInsideContenido contenidoInside = new ObjetoDocumentoInsideContenido ();
				documento.setContenido(contenidoInside);
			}

			documento.getContenido().setValorBinario(contenidoDocumento.getContenido());
			documento.getContenido().setNombreFormato((String)metadatosRepository.get(this.documentoMetadatoNombreFormato));
			
		}
		
		// Firmas del documento
		for (FirmaInside firma : documento.getFirmas()) {
			ContenidoFirmaInside contenidoFirma = firma.getContenidoFirma();
			if(contenidoFirma != null && contenidoFirma instanceof ContenidoFirmaCertificadoAlmacenableInside)
			{
				ContenidoFirmaCertificadoAlmacenableInside contenidoFirmaAlmacenado = (ContenidoFirmaCertificadoAlmacenableInside) contenidoFirma;
				byte[] bytesFirma = null;
				try {
					bytesFirma = cmisAdapter.getContenidoDoc(contenidoFirmaAlmacenado.getIdentificadorRepositorio()).getContenido();
				} catch (RepositoryCmisException rce) {
					String mensajeError = String.format("No ha podido obtenerse la firma del documento con identificador %s del repositorio " + rce.getMessage(), contenidoFirmaAlmacenado.getIdentificadorRepositorio());
					logger.error(mensajeError);
					throw new InsideServiceCmisAdapterException (mensajeError, rce);
				} 

				contenidoFirmaAlmacenado.setValorBinario(bytesFirma);
			}
		}

		logger.debug("convertFromRepository: documento con identificador de Repositorio " + documento.getIdentificadorRepositorio() + " convertido correctamente");

	}
	
	
	public void obtenerExpedienteInside (ObjetoExpedienteInside expediente) throws InsideServiceCmisAdapterException {
		if (StringUtils.isEmpty(expediente.getIdentificadorRepositorio())) {
			throw new InsideServiceCmisAdapterException ("El identificador del repositorio del expediente no puede ser nulo");
		}
		
		//Map<String, Object> metadatosRepository = null;
		
		try {		
			// Metadatos del expediente
			//metadatosRepository = cmisAdapter.getInfoMetadatosCarpeta(expediente.getIdentificadorRepositorio());
			
			// Firmas del expediente
			if (expediente.getIndice().getFirmas() != null) {
				
				for (FirmaInside firma : expediente.getIndice().getFirmas()) {
					ContenidoFirmaInside contenidoFirma = firma.getContenidoFirma();
					if( contenidoFirma != null && contenidoFirma instanceof ContenidoFirmaCertificadoAlmacenableInside)
					{
						ContenidoFirmaCertificadoAlmacenableInside contenidoFirmaAlmacenado = (ContenidoFirmaCertificadoAlmacenableInside) contenidoFirma;
						byte[] bytesFirma = cmisAdapter.getContenidoDoc(contenidoFirmaAlmacenado.getIdentificadorRepositorio()).getContenido();	
						contenidoFirmaAlmacenado.setValorBinario(bytesFirma);
					}
				}
				
			}
			
		} catch (RepositoryCmisException rce) {
			String mensajeError = String.format("No ha podido obtenerse el objeto del repositorio cuando intentaba consultarse el expediente con identificador %s " + rce.getMessage(), expediente.getIdentificador());
			logger.error(mensajeError);
			throw new InsideServiceCmisAdapterException (mensajeError, rce);
		}
	}

	public CmisAdapterInterface getCmisAdapter() {
		return cmisAdapter;
	}

	@Required
	public void setCmisAdapter(CmisAdapterInterface cmisAdapter) {
		this.cmisAdapter = cmisAdapter;
	}

	public String getDocumentoObjectTypeId() {
		return documentoObjectTypeId;
	}

	@Required
	public void setDocumentoObjectTypeId(String objectTypeId) {
		this.documentoObjectTypeId = objectTypeId;
	}

	public String getDocumentoMetadatoVersionNti() {
		return documentoMetadatoVersionNti;
	}

	@Required
	public void setDocumentoMetadatoVersionNti(String DocumentoMetadatoVersionNti) {
		this.documentoMetadatoVersionNti = DocumentoMetadatoVersionNti;
	}

	public String getDocumentoMetadatoIdentificador() {
		return documentoMetadatoIdentificador;
	}

	@Required
	public void setDocumentoMetadatoIdentificador(String DocumentoMetadatoIdentificador) {
		this.documentoMetadatoIdentificador = DocumentoMetadatoIdentificador;
	}

	public String getDocumentoMetadatoOrgano() {
		return documentoMetadatoOrgano;
	}

	@Required
	public void setDocumentoMetadatoOrgano(String DocumentoMetadatoOrgano) {
		this.documentoMetadatoOrgano = DocumentoMetadatoOrgano;
	}

	public String getDocumentoMetadatoFechaCaptura() {
		return documentoMetadatoFechaCaptura;
	}

	@Required
	public void setDocumentoMetadatoFechaCaptura(String DocumentoMetadatoFechaCaptura) {
		this.documentoMetadatoFechaCaptura = DocumentoMetadatoFechaCaptura;
	}

	public String getDocumentoMetadatoDocOrigen() {
		return documentoMetadatoDocOrigen;
	}

	@Required
	public void setDocumentoMetadatoDocOrigen(String DocumentoMetadatoDocOrigen) {
		this.documentoMetadatoDocOrigen = DocumentoMetadatoDocOrigen;
	}

	public String getDocumentoMetadatoDocEstadoElaboracion() {
		return documentoMetadatoDocEstadoElaboracion;
	}

	@Required
	public void setDocumentoMetadatoDocEstadoElaboracion(String DocumentoMetadatoDocEstadoElaboracion) {
		this.documentoMetadatoDocEstadoElaboracion = DocumentoMetadatoDocEstadoElaboracion;
	}

	public String getDocumentoMetadatoNombreFormato() {
		return documentoMetadatoNombreFormato;
	}

	@Required
	public void setDocumentoMetadatoNombreFormato(String DocumentoMetadatoNombreFormato) {
		this.documentoMetadatoNombreFormato = DocumentoMetadatoNombreFormato;
	}

	public String getDocumentoMetadatoTipoDocumental() {
		return documentoMetadatoTipoDocumental;
	}

	@Required
	public void setDocumentoMetadatoTipoDocumental(String DocumentoMetadatoTipoDocumental) {
		this.documentoMetadatoTipoDocumental = DocumentoMetadatoTipoDocumental;
	}

	public String getDocumentoMetadatoDocTipoFirma() {
		return documentoMetadatoDocTipoFirma;
	}

	@Required
	public void setDocumentoMetadatoDocTipoFirma(String DocumentoMetadatoDocTipoFirma) {
		this.documentoMetadatoDocTipoFirma = DocumentoMetadatoDocTipoFirma;
	}

	public String getDocumentoMetadatoIdentificadorDocOrigen() {
		return documentoMetadatoIdentificadorDocOrigen;
	}

	@Required
	public void setDocumentoMetadatoIdentificadorDocOrigen(
			String DocumentoMetadatoIdentificadorDocOrigen) {
		this.documentoMetadatoIdentificadorDocOrigen = DocumentoMetadatoIdentificadorDocOrigen;
	}

	public String getRepoObjectId() {
		return repoObjectId;
	}

	@Required
	public void setRepoObjectId(String repoObjectId) {
		this.repoObjectId = repoObjectId;
	}

	public String getRepoVersionLabel() {
		return repoVersionLabel;
	}

	@Required
	public void setRepoVersionLabel(String repoVersionLabel) {
		this.repoVersionLabel = repoVersionLabel;
	}

	public String getRepoRootFolder() {
		return repoRootFolder;
	}

	@Required
	public void setRepoRootFolder(String repoRootFolder) {
		this.repoRootFolder = repoRootFolder;
	}


	public String getRepoPathSeparator() {
		return repoPathSeparator;
	}

	@Required
	public void setRepoPathSeparator(String repoPathSeparator) {
		this.repoPathSeparator = repoPathSeparator;
	}
	

	public String getExpedientesFolder() {
		return expedientesFolder;
	}

	@Required
	public void setExpedientesFolder(String expedientesFolder) {
		this.expedientesFolder = expedientesFolder;
	}

	public String getDocumentosFolder() {
		return documentosFolder;
	}

	@Required
	public void setDocumentosFolder(String documentosFolder) {
		this.documentosFolder = documentosFolder;
	}

	public String getExpedienteObjectTypeId() {
		return expedienteObjectTypeId;
	}

	@Required
	public void setExpedienteObjectTypeId(String expedienteObjectTypeId) {
		this.expedienteObjectTypeId = expedienteObjectTypeId;
	}

	public String getExpedienteMetadatoVersionNti() {
		return expedienteMetadatoVersionNti;
	}

	@Required
	public void setExpedienteMetadatoVersionNti(String expedienteMetadatoVersionNti) {
		this.expedienteMetadatoVersionNti = expedienteMetadatoVersionNti;
	}

	public String getExpedienteMetadatoIdentificador() {
		return expedienteMetadatoIdentificador;
	}

	@Required
	public void setExpedienteMetadatoIdentificador(
			String expedienteMetadatoIdentificador) {
		this.expedienteMetadatoIdentificador = expedienteMetadatoIdentificador;
	}

	public String getExpedienteMetadatoOrgano() {
		return expedienteMetadatoOrgano;
	}

	@Required
	public void setExpedienteMetadatoOrgano(String expedienteMetadatoOrgano) {
		this.expedienteMetadatoOrgano = expedienteMetadatoOrgano;
	}

	public String getExpedienteMetadatoFechaApertura() {
		return expedienteMetadatoFechaApertura;
	}

	@Required
	public void setExpedienteMetadatoFechaApertura(
			String expedienteMetadatoFechaApertura) {
		this.expedienteMetadatoFechaApertura = expedienteMetadatoFechaApertura;
	}

	public String getExpedienteMetadatoClasificacion() {
		return expedienteMetadatoClasificacion;
	}

	@Required
	public void setExpedienteMetadatoClasificacion(
			String expedienteMetadatoClasificacion) {
		this.expedienteMetadatoClasificacion = expedienteMetadatoClasificacion;
	}

	public String getExpedienteMetadatoEstado() {
		return expedienteMetadatoEstado;
	}

	@Required
	public void setExpedienteMetadatoEstado(String expedienteMetadatoEstado) {
		this.expedienteMetadatoEstado = expedienteMetadatoEstado;
	}

	public String getExpedienteMetadatoInteresado() {
		return expedienteMetadatoInteresado;
	}

	@Required
	public void setExpedienteMetadatoInteresado(String expedienteMetadatoInteresado) {
		this.expedienteMetadatoInteresado = expedienteMetadatoInteresado;
	}

	

	
	
}
