/* Copyright (C) 2012-13 MINHAP, Gobierno de España
   This program is licensed and may be used, modified and redistributed under the terms
   of the European Public License (EUPL), either version 1.1 or (at your
   option) any later version as soon as they are approved by the European Commission.
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
   or implied. See the License for the specific language governing permissions and
   more details.
   You should have received a copy of the EUPL1.1 license
   along with this program; if not, you may find it at
   http://joinup.ec.europa.eu/software/page/eupl/licence-eupl */

package es.mpt.dsic.inside.service.object.converter.impl.cmis;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Required;

import es.mpt.dsic.inside.model.objetos.documento.ObjetoDocumentoInside;
import es.mpt.dsic.inside.model.objetos.documento.metadatos.ObjetoDocumentoInsideMetadatos;
import es.mpt.dsic.inside.service.object.converter.InsideServiceObjectConverter;
import es.mpt.dsic.inside.service.object.converter.InsideServiceObjectConverterException;

public class InsideServiceCmisDocumentoConverter  implements InsideServiceObjectConverter<ObjetoDocumentoInside,ObjetoDocumentoInsideMetadatos> {

	private InsideServiceCmisAdapter cmisAdapter;

	protected static final Log logger = LogFactory.getLog(InsideServiceCmisDocumentoConverter.class);
	
	
	@Override
	/**
	 * Convierte un ObjetoDocumentoInside. El objeto de entrada debe contener el identificador del repositorio.
	 * El objeto de salida tendrá relleno:
	 * 	- El identificador del documento, con el identificador del objeto de entrada.
	 * 	- El identificador del repositorio del documento, con el identificador del repositorio del objeto de entrada.
	 * 	- El contenido del documento, con el contenido del repositorio.
	 *  - Metadatos del documento, con los metadatos del repositorio.
	 *  - Firmas del documento, con las firmas del objeto de entrada, añadiendo los bytes de las firmas obtenidas del repositorio.
	 */
	public ObjetoDocumentoInside fromRepository(ObjetoDocumentoInside documento)
			throws InsideServiceObjectConverterException {
		
		try {
			cmisAdapter.obtenerDocumentoInside(documento);
		} catch (InsideServiceCmisAdapterException e) {
			throw new InsideServiceObjectConverterException("Excepción recuperando los datos del documento electrónico del repositorio CMIS",e);
		}
		
		return documento;
		
		
	}

	@Override
	/**
	 * Convierte un ObjetoDocumentoInside. El objeto de entrada debe contener metadatos.
	 * El objeto de salida tendrá relleno:
	 * 	- El identificador del documento, con el identificador del objeto de entrada.
	 * 	- El identificador del repositorio del documento, con el identificador de objeto CMIS.	  	
	 *  - Metadatos del documento, con los metadatos del del objeto de entrada.
	 *  - La versión del documento.
	 *  - Las firmas, con las firmas del objeto de entrada, EXCEPTO los bytes de la firma.
	 */
	public ObjetoDocumentoInside toRepository(ObjetoDocumentoInside documento)
			throws InsideServiceObjectConverterException {
		
		try {
			cmisAdapter.almacenaObjetoDocumentoInside(documento);
		} catch (InsideServiceCmisAdapterException e) {
			throw new InsideServiceObjectConverterException("Excepción trasladando el documento electrónico al repositorio CMIS",e);
		}
		
		
		return documento;
	}
	
	
	public InsideServiceCmisAdapter getCmisAdapter() {
		return cmisAdapter;
	}

	@Required
	public void setCmisAdapter(InsideServiceCmisAdapter cmisAdapter) {
		this.cmisAdapter = cmisAdapter;
	}

}
