/* Copyright (C) 2012-13 MINHAP, Gobierno de España
   This program is licensed and may be used, modified and redistributed under the terms
   of the European Public License (EUPL), either version 1.1 or (at your
   option) any later version as soon as they are approved by the European Commission.
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
   or implied. See the License for the specific language governing permissions and
   more details.
   You should have received a copy of the EUPL1.1 license
   along with this program; if not, you may find it at
   http://joinup.ec.europa.eu/software/page/eupl/licence-eupl */

package es.mpt.dsic.inside.service.store;

import java.util.List;
import java.util.SortedMap;

import es.mpt.dsic.inside.model.busqueda.consulta.ConsultaInside;
import es.mpt.dsic.inside.model.busqueda.resultado.ItemResultadoBusquedaInside;
import es.mpt.dsic.inside.model.busqueda.resultado.ResultadoBusquedaInside;
import es.mpt.dsic.inside.model.objetos.ObjetoInside;
import es.mpt.dsic.inside.model.objetos.ObjetoInsideAplicacion;
import es.mpt.dsic.inside.model.objetos.ObjetoInsideMetadatos;
import es.mpt.dsic.inside.model.objetos.ObjetoInsideVersion;
import es.mpt.dsic.inside.service.store.exception.InsideServiceStoreException;
import es.mpt.dsic.inside.service.store.exception.InsideStoreObjectAlreadyExistsException;
import es.mpt.dsic.inside.service.store.exception.InsideStoreObjectNotFoundException;


public interface InsideServiceStore {

	public <T extends ObjetoInside<?>> T storeObject(T object) throws InsideServiceStoreException, InsideStoreObjectAlreadyExistsException;
	
	public <T extends ObjetoInside<?>> T getObject(Class<T> tipo,final String Identifier) throws InsideServiceStoreException, InsideStoreObjectNotFoundException;
	
	public <T extends ObjetoInside<?>> T getObjectVersion(Class<T> tipo,final String Identifier,int version) throws InsideServiceStoreException, InsideStoreObjectNotFoundException;
	
	public <T extends ObjetoInside<K>, K extends ObjetoInsideMetadatos> K getObjectMetadatos(Class<T> tipo,final String Identifier) throws InsideServiceStoreException, InsideStoreObjectNotFoundException;
	
	public <T extends ObjetoInside<K>, K extends ObjetoInsideMetadatos> K getObjectMetadatos(Class<T> tipo,final String Identifier,int version) throws InsideServiceStoreException, InsideStoreObjectNotFoundException;

	public <T extends ObjetoInside<?>> T updateObject(T object) throws InsideServiceStoreException, InsideStoreObjectNotFoundException;
	
	//TODO ¿ updateObjectMetadatos ??
	
	public <T extends ObjetoInside<K>, K extends ObjetoInsideMetadatos> ResultadoBusquedaInside<K> queryObjects(ConsultaInside consulta, Class<T> tipo) throws InsideServiceStoreException;

	public <T extends ObjetoInside<?>> List<ObjetoInsideVersion> getObjectVersions(Class<T> tipo, String Identifier) throws InsideServiceStoreException, InsideStoreObjectNotFoundException;

	public <T extends ObjetoInside<?>> SortedMap<ObjetoInsideVersion, T> getVersionesObjeto(Class<T> tipo,String Identifier) throws InsideServiceStoreException,InsideStoreObjectNotFoundException;
	
	public ObjetoInsideAplicacion getAplicacion (String nombre) throws InsideServiceStoreException ;
	//public <T extends ObjetoInside<?>> String generarIdentificador (T object) throws InsideServiceStoreException;
}
