/* Copyright (C) 2012-13 MINHAP, Gobierno de España
   This program is licensed and may be used, modified and redistributed under the terms
   of the European Public License (EUPL), either version 1.1 or (at your
   option) any later version as soon as they are approved by the European Commission.
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
   or implied. See the License for the specific language governing permissions and
   more details.
   You should have received a copy of the EUPL1.1 license
   along with this program; if not, you may find it at
   http://joinup.ec.europa.eu/software/page/eupl/licence-eupl */

package es.mpt.dsic.inside.service.store.impl.hibernate;

import java.util.GregorianCalendar;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.classic.Session;
import org.springframework.beans.factory.annotation.Required;

import es.mpt.dsic.inside.model.objetos.ObjetoInside;
import es.mpt.dsic.inside.model.objetos.documento.ObjetoDocumentoInside;
import es.mpt.dsic.inside.model.objetos.expediente.ObjetoExpedienteInside;
import es.mpt.dsic.inside.service.store.exception.InsideServiceStoreException;
import es.mpt.dsic.inside.service.store.impl.hibernate.converter.InsideServiceStoreHibernateConverter;
import es.mpt.dsic.inside.service.store.impl.hibernate.entity.DocumentoInside;
import es.mpt.dsic.inside.service.store.impl.hibernate.entity.DocumentoInsideFirmas;
import es.mpt.dsic.inside.service.store.impl.hibernate.entity.DocumentoInsideMetadatosAdicionales;
import es.mpt.dsic.inside.service.store.impl.hibernate.entity.DocumentoInsideOrgano;
import es.mpt.dsic.inside.service.store.impl.hibernate.entity.ExpedienteInside;
import es.mpt.dsic.inside.service.store.impl.hibernate.entity.ExpedienteInsideIndice;
import es.mpt.dsic.inside.service.store.impl.hibernate.entity.ExpedienteInsideIndiceCarpetaIndizada;
import es.mpt.dsic.inside.service.store.impl.hibernate.entity.ExpedienteInsideIndiceDocumentoIndizado;
import es.mpt.dsic.inside.service.store.impl.hibernate.entity.ExpedienteInsideIndiceFirmas;
import es.mpt.dsic.inside.service.store.impl.hibernate.entity.ExpedienteInsideInteresado;
import es.mpt.dsic.inside.service.store.impl.hibernate.entity.ExpedienteInsideMetadatosAdicionales;
import es.mpt.dsic.inside.service.store.impl.hibernate.entity.ExpedienteInsideOrgano;

public class InsideServiceStoreHibernatePersister <I extends ObjetoInside<?>, E extends Object>{
	
	protected static final Log logger = LogFactory.getLog(InsideServiceStoreHibernatePersister.class);
	
	private SessionFactory sessionFactory;
	
	private InsideServiceStoreHibernateConverter<I,E> converter;

	@SuppressWarnings("unchecked")
	public <T extends ObjetoInside<?>> void persistExpedienteOrDocumento(T object) 
			throws InsideServiceStoreException
		{
			Session session = sessionFactory.openSession();
			
			try {
				if(object instanceof ObjetoExpedienteInside){
					saveExpediente((ExpedienteInside) converter.toEntity((I) object,session),session);
				}else if(object instanceof ObjetoDocumentoInside){
					saveDocumento((DocumentoInside) converter.toEntity((I) object,session),session);		
				}else{
					throw new InsideServiceStoreException("No se como almacenar objetos de tipo " + object.getClass());
				}
			} catch (InsideServiceStoreException e) {
				throw e;
			}catch (Exception e) {
				throw new InsideServiceStoreException("Error almacenando expediente o documento",e);
			}finally{
				session.close();
			}
			
			
		}
		
		private void saveDocumento(DocumentoInside documentoEntity, Session session)
			throws InsideServiceStoreException
		{
			logger.debug("Init saveDocumento");
			logger.debug("Parameter <documentoEntity>" + documentoEntity.toString());
			
			Transaction tx = session.beginTransaction();
			try {
				session.saveOrUpdate(documentoEntity);
				
				for(DocumentoInsideOrgano organo : documentoEntity.getDocumentoInsideOrganos()){
					session.saveOrUpdate(organo);
				}
				for(DocumentoInsideFirmas firma : documentoEntity.getDocumentoInsideFirmases()){
					session.saveOrUpdate(firma.getFirmaInside());
					session.saveOrUpdate(firma);
				}
				for(DocumentoInsideMetadatosAdicionales metadato : documentoEntity.getDocumentoInsideMetadatosAdicionaleses()){
					session.saveOrUpdate(metadato);
				}
				
				tx.commit();
			} catch (Exception e) {
				tx.rollback();
				throw new InsideServiceStoreException("Excepción almacenando Documento en BBDD" ,e);
			} 
			
			logger.debug("End saveDocumento");
		}

		private void saveExpediente(ExpedienteInside expedienteEntity, Session session) 
			throws InsideServiceStoreException
		{
			logger.debug("Init saveExpediente");
			logger.debug("Parameter <expedienteEntity>" + expedienteEntity.toString());
			Transaction tx = session.beginTransaction();
			try {
				
				saveIndiceExpediente(expedienteEntity.getExpedienteInsideIndice(), session);
				
				session.saveOrUpdate(expedienteEntity);
				
				for(ExpedienteInsideIndiceFirmas docFirma : expedienteEntity.getExpedienteInsideIndiceFirmases()){
					session.saveOrUpdate(docFirma.getFirmaInside());
					session.saveOrUpdate(docFirma);
				}
				for(ExpedienteInsideInteresado interesado : expedienteEntity.getExpedienteInsideInteresados()){
					session.saveOrUpdate(interesado);
				}
				for(ExpedienteInsideOrgano organo : expedienteEntity.getExpedienteInsideOrganos()){
					session.saveOrUpdate(organo);
				}
				for(ExpedienteInsideMetadatosAdicionales metadato : expedienteEntity.getExpedienteInsideMetadatosAdicionaleses()){
					session.saveOrUpdate(metadato);
				}
				
				tx.commit();
			} catch (Exception e) {
				tx.rollback();
				throw new InsideServiceStoreException("Excepción almacenando Expediente en BBDD" ,e);
			} 
			
			logger.debug("End saveExpediente");
		}
		
		private void saveIndiceExpediente(ExpedienteInsideIndice indiceExpedienteEntity,Session session) 
			throws InsideServiceStoreException
		{
			try {
				
				indiceExpedienteEntity.setCreatedAt(GregorianCalendar.getInstance().getTime());
				
				session.saveOrUpdate(indiceExpedienteEntity);
				
				for(ExpedienteInsideIndiceDocumentoIndizado docIndizado : indiceExpedienteEntity.getExpedienteInsideIndiceDocumentoIndizados()){
					docIndizado.setCreatedAt(GregorianCalendar.getInstance().getTime());
					session.saveOrUpdate(docIndizado);
				}
				for(ExpedienteInsideIndice expedienteIndizado : indiceExpedienteEntity.getExpedienteInsideIndices()){
					saveIndiceExpediente(expedienteIndizado, session);
				}
				for(ExpedienteInsideIndiceCarpetaIndizada carpeta : indiceExpedienteEntity.getExpedienteInsideIndiceCarpetaIndizadas()){
					saveCarpetaExpediente(carpeta,session);
				}
			
				
			} catch (Exception e) {
				throw new InsideServiceStoreException("Excepción almacenando Indice de Expediente en BBDD" ,e);
			}
		}
		
		private void saveCarpetaExpediente(ExpedienteInsideIndiceCarpetaIndizada carpetaEntity,Session session) 
			throws InsideServiceStoreException
		{
			try {
				
				carpetaEntity.setCreatedAt(GregorianCalendar.getInstance().getTime());
				
				session.saveOrUpdate(carpetaEntity);
				
				for(ExpedienteInsideIndiceDocumentoIndizado docIndizado : carpetaEntity.getExpedienteInsideIndiceDocumentoIndizados()){
					docIndizado.setCreatedAt(GregorianCalendar.getInstance().getTime());
					session.saveOrUpdate(docIndizado);
				}
				for(ExpedienteInsideIndice expedienteIndizado : carpetaEntity.getExpedienteInsideIndices()){
					saveIndiceExpediente(expedienteIndizado, session);
				}
				for(ExpedienteInsideIndiceCarpetaIndizada carpeta : carpetaEntity.getExpedienteInsideIndiceCarpetaIndizadas()){
					saveCarpetaExpediente(carpeta, session);
				}
			
			} catch (Exception e) {
				throw new InsideServiceStoreException("Excepción almacenando Carpeta Indizada en BBDD" ,e);
			} 
		}

		public SessionFactory getSessionFactory() {
			return sessionFactory;
		}

		@Required
		public void setSessionFactory(SessionFactory sessionFactory) {
			this.sessionFactory = sessionFactory;
		}

		public InsideServiceStoreHibernateConverter<I,E> getConverter() {
			return converter;
		}

		public void setConverter(InsideServiceStoreHibernateConverter<I,E> converter) {
			this.converter = converter;
		}
	
}
