/* Copyright (C) 2012-13 MINHAP, Gobierno de España
   This program is licensed and may be used, modified and redistributed under the terms
   of the European Public License (EUPL), either version 1.1 or (at your
   option) any later version as soon as they are approved by the European Commission.
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
   or implied. See the License for the specific language governing permissions and
   more details.
   You should have received a copy of the EUPL1.1 license
   along with this program; if not, you may find it at
   http://joinup.ec.europa.eu/software/page/eupl/licence-eupl */

package es.mpt.dsic.inside.service.store.impl.hibernate.converter;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.hibernate.classic.Session;

import es.mpt.dsic.inside.model.objetos.ObjetoInsideMetadatoAdicional;
import es.mpt.dsic.inside.model.objetos.ObjetoInsideVersion;
import es.mpt.dsic.inside.model.objetos.documento.ObjetoDocumentoInside;
import es.mpt.dsic.inside.model.objetos.documento.ObjetoDocumentoInsideContenido;
import es.mpt.dsic.inside.model.objetos.documento.metadatos.ObjetoDocumentoInsideMetadatos;
import es.mpt.dsic.inside.model.objetos.documento.metadatos.ObjetoDocumentoInsideMetadatosEstadoElaboracion;
import es.mpt.dsic.inside.model.objetos.documento.metadatos.ObjetoDocumentoInsideMetadatosEstadoElaboracionEnumeracion;
import es.mpt.dsic.inside.model.objetos.documento.metadatos.ObjetoDocumentoInsideMetadatosTipoDocumental;
import es.mpt.dsic.inside.model.objetos.firmas.FirmaInside;
import es.mpt.dsic.inside.service.store.exception.InsideServiceStoreException;
import es.mpt.dsic.inside.service.store.impl.hibernate.entity.DocumentoInside;
import es.mpt.dsic.inside.service.store.impl.hibernate.entity.DocumentoInsideFirmas;
import es.mpt.dsic.inside.service.store.impl.hibernate.entity.DocumentoInsideMetadatosAdicionales;
import es.mpt.dsic.inside.service.store.impl.hibernate.entity.DocumentoInsideOrgano;

public class InsideServiceStoreHibernateConverterDocumento implements InsideServiceStoreHibernateConverter<ObjetoDocumentoInside,DocumentoInside>{

	@Override
	public DocumentoInside toEntity(ObjetoDocumentoInside documento, Session session) throws InsideServiceStoreException 
	{
		DocumentoInside entity = new DocumentoInside();
		entity.setIdentificador(documento.getIdentificador());
		entity.setIdentificadorRepositorio(documento.getIdentificadorRepositorio());
		
		if(documento.getVersion() != null){
			if(documento.getVersion().getFechaVersion() != null){
				entity.setFechaVersion(documento.getVersion().getFechaVersion().getTime());
			}
			entity.setVersion(documento.getVersion().getVersion());
			
		}
		
		if(documento.getMetadatos() != null){
			if(documento.getMetadatos().getEstadoElaboracion() != null){
				if(documento.getMetadatos().getEstadoElaboracion().getValorEstadoElaboracion() != null){
					entity.setEstadoElaboracion(documento.getMetadatos().getEstadoElaboracion().getValorEstadoElaboracion().value());
				}
				entity.setIdentificadorDocumentoOrigen(documento.getMetadatos().getEstadoElaboracion().getIdentificadorDocumentoOrigen());
			}
			
			if(documento.getMetadatos().getFechaCaptura() != null){
				entity.setFechaCaptura(documento.getMetadatos().getFechaCaptura().getTime());
			}
			if(documento.getMetadatos().getMetadatosAdicionales().size() > 0){
				Set<DocumentoInsideMetadatosAdicionales> documentoInsideMetadatosAdicionaleses = new HashSet<DocumentoInsideMetadatosAdicionales>();
				for(ObjetoInsideMetadatoAdicional metadatoAdicional : documento.getMetadatos().getMetadatosAdicionales().values()){
					DocumentoInsideMetadatosAdicionales metadatoAaditionaEntity = new DocumentoInsideMetadatosAdicionales();
					metadatoAaditionaEntity.setDocumentoInside(entity);
					metadatoAaditionaEntity.setNombre(metadatoAdicional.getNombre());
					metadatoAaditionaEntity.setValor(metadatoAdicional.getValor() + "");
					documentoInsideMetadatosAdicionaleses.add(metadatoAaditionaEntity);
				}
				
				entity.setDocumentoInsideMetadatosAdicionaleses(documentoInsideMetadatosAdicionaleses);
			}
			
			Set<DocumentoInsideOrgano> documentoInsideOrganos = new HashSet<DocumentoInsideOrgano>();
			for(String organo : documento.getMetadatos().getOrgano()){
				DocumentoInsideOrgano documentoInsideOrgano = new DocumentoInsideOrgano();
				documentoInsideOrgano.setOrgano(organo);
				documentoInsideOrgano.setDocumentoInside(entity);
				documentoInsideOrganos.add(documentoInsideOrgano);
			}
			entity.setDocumentoInsideOrganos(documentoInsideOrganos);
			if(documento.getMetadatos().getTipoDocumental() != null){
				entity.setTipoDocumental(documento.getMetadatos().getTipoDocumental().value());
			}
			entity.setVersionNti(documento.getMetadatos().getVersionNTI());
			entity.setOrigen(documento.getMetadatos().isOrigenCiudadanoAdministracion());
			
			if(documento.getFirmas() != null){
				Set<DocumentoInsideFirmas> documentoInsideFirmas = new HashSet<DocumentoInsideFirmas>();
				for(FirmaInside firmaInside : documento.getFirmas()){
					DocumentoInsideFirmas firmaEntity = new DocumentoInsideFirmas();
					try {
						firmaEntity.setFirmaInside(InsideServiceStoreHibernateConverterFirma.toEntity(firmaInside));
					} catch (InsideServiceStoreHibernateConverterFirmaException e) {
						throw new  InsideServiceStoreException("Error convirtiendo FirmaInside a Entidad de Hibernate",e);
					}
					firmaEntity.setDocumentoInside(entity);
					documentoInsideFirmas.add(firmaEntity);
				}
				
				entity.setDocumentoInsideFirmases(documentoInsideFirmas);
			}
			
			if(documento.getContenido() != null){				
				entity.setTipoMime(documento.getContenido().getMime());
				entity.setNombreFormato(documento.getContenido().getNombreFormato());
				entity.setIdentificadorEnDocumento(documento.getContenido().getIdentificadorEnDocumento());
				entity.setReferencia(documento.getContenido().getReferencia());
			}
			
		}
		return entity;
	}

//	@SuppressWarnings("unchecked")
	@Override
	public ObjetoDocumentoInside toInside(DocumentoInside entity, Session session) throws InsideServiceStoreException
	{
		ObjetoDocumentoInside documento = new ObjetoDocumentoInside();
		
		documento.setIdentificador(entity.getIdentificador());
		documento.setIdentificadorRepositorio(entity.getIdentificadorRepositorio());
		
		Calendar fechaVersion = null;
		if(entity.getFechaVersion() != null){
			fechaVersion = GregorianCalendar.getInstance();
			fechaVersion.setTime(entity.getFechaVersion());
		}
		documento.setVersion(new ObjetoInsideVersion(entity.getVersion(), (GregorianCalendar) fechaVersion));
		
		documento.setMetadatos(new ObjetoDocumentoInsideMetadatos());
		ObjetoDocumentoInsideMetadatosEstadoElaboracion estadoElaboracion = new ObjetoDocumentoInsideMetadatosEstadoElaboracion();
		if(entity.getEstadoElaboracion() != null){
			estadoElaboracion.setValorEstadoElaboracion(ObjetoDocumentoInsideMetadatosEstadoElaboracionEnumeracion.fromValue(entity.getEstadoElaboracion()));
		}
		estadoElaboracion.setIdentificadorDocumentoOrigen(entity.getIdentificadorDocumentoOrigen());
		documento.getMetadatos().setEstadoElaboracion(estadoElaboracion);
		
		Calendar fechaCaptura = null;
		if(entity.getFechaCaptura() != null){
			fechaCaptura = GregorianCalendar.getInstance();
			fechaCaptura.setTime(entity.getFechaCaptura());
		}
		documento.getMetadatos().setFechaCaptura((GregorianCalendar) fechaCaptura);
		documento.getMetadatos().setIdentificadorObjetoInside(entity.getIdentificador());
		documento.getMetadatos().setOrigenCiudadanoAdministracion(entity.getOrigen() != null && entity.getOrigen());
		
		Map<String, ObjetoInsideMetadatoAdicional> metadatosAdicionales = new HashMap<String, ObjetoInsideMetadatoAdicional>();
		for(DocumentoInsideMetadatosAdicionales metEntry : entity.getDocumentoInsideMetadatosAdicionaleses()){
			ObjetoInsideMetadatoAdicional valorMetadatoAdicional = new ObjetoInsideMetadatoAdicional();
			valorMetadatoAdicional.setNombre(metEntry.getNombre());
			valorMetadatoAdicional.setValor(metEntry.getValor());
			metadatosAdicionales.put(metEntry.getNombre(), valorMetadatoAdicional);
		}
		documento.getMetadatos().setMetadatosAdicionales(metadatosAdicionales);
		
		List<String> organosInside = new ArrayList<String>();
		for(DocumentoInsideOrgano organo : entity.getDocumentoInsideOrganos()){
			organosInside.add(organo.getOrgano());
		}
		documento.getMetadatos().setOrgano(organosInside);
		
		documento.getMetadatos().setVersionNTI(entity.getVersionNti());
		if(entity.getTipoDocumental() != null){
			documento.getMetadatos().setTipoDocumental(ObjetoDocumentoInsideMetadatosTipoDocumental.fromValue(entity.getTipoDocumental()));
		}
		
		List<FirmaInside> firmas = new ArrayList<FirmaInside>();
		for(DocumentoInsideFirmas firmaEntity : entity.getDocumentoInsideFirmases()){
			firmas.add(InsideServiceStoreHibernateConverterFirma.toInside(firmaEntity.getFirmaInside()));
		}
		
		Collections.sort(firmas);
		documento.setFirmas(firmas);
		
		documento.setContenido(new ObjetoDocumentoInsideContenido());
		documento.getContenido().setMime(entity.getTipoMime());
		documento.getContenido().setNombreFormato(entity.getNombreFormato());
		documento.getContenido().setIdentificadorEnDocumento(entity.getIdentificadorEnDocumento());
		documento.getContenido().setReferencia(entity.getReferencia());
		return documento;
		
	}

}
