/* Copyright (C) 2012-13 MINHAP, Gobierno de España
   This program is licensed and may be used, modified and redistributed under the terms
   of the European Public License (EUPL), either version 1.1 or (at your
   option) any later version as soon as they are approved by the European Commission.
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
   or implied. See the License for the specific language governing permissions and
   more details.
   You should have received a copy of the EUPL1.1 license
   along with this program; if not, you may find it at
   http://joinup.ec.europa.eu/software/page/eupl/licence-eupl */

package es.mpt.dsic.inside.service.util;

import java.util.ArrayList;
import java.util.List;

import es.mpt.dsic.inside.model.objetos.ObjetoInsideMetadatos;
import es.mpt.dsic.inside.model.objetos.documento.ObjetoDocumentoInside;
import es.mpt.dsic.inside.model.objetos.firmas.FirmaInside;
import es.mpt.dsic.inside.model.objetos.firmas.FirmaInsideTipoFirmaEnum;
import es.mpt.dsic.inside.model.objetos.firmas.contenido.ContenidoFirmaCertificadoAlmacenableInside;
import es.mpt.dsic.inside.model.objetos.firmas.contenido.ContenidoFirmaCertificadoReferenciaInside;
import es.mpt.dsic.inside.model.objetos.firmas.contenido.ContenidoFirmaInside;

public class InsideObjectsUtils {
	
	
	/**
	 * Devuelve las firmas que referencian a un nodo en concreto
	 * @param identificadorNodo
	 * @return
	 */
	public static List<FirmaInside> getFirmasReferencianNodo (String identificadorNodo, List<FirmaInside> firmas) {
		List<FirmaInside> firmasRef = new ArrayList<FirmaInside> ();
		if (firmas != null) {
			for (FirmaInside firma : firmas) {
				if (firma.getContenidoFirma() instanceof ContenidoFirmaCertificadoReferenciaInside) {
					ContenidoFirmaCertificadoReferenciaInside contRef = (ContenidoFirmaCertificadoReferenciaInside) firma.getContenidoFirma();
					if (contRef.getReferenciaFirma().replace("#","").contentEquals(identificadorNodo)) {
						firmasRef.add(firma);
					}
				}
			}			
		}
		return firmasRef;
	}
	/**
	 * Devuelve los identificadores de los nodos que son firmas, exceptuando si es firma CSV
	 * @param documentoInside
	 * @return
	 */
	public static List<String> getIdNodosSonFirma (ObjetoDocumentoInside documento) {
		List<String> idNodosToSend = new ArrayList<String> ();
		
		if (documento.getFirmas() != null && documento.getFirmas().size() > 0) {
						
			for (FirmaInside firma : documento.getFirmas()) {
				ContenidoFirmaInside contenidoFirma = firma.getContenidoFirma();
				
				if (contenidoFirma instanceof ContenidoFirmaCertificadoReferenciaInside) {
					ContenidoFirmaCertificadoReferenciaInside contRef = (ContenidoFirmaCertificadoReferenciaInside) contenidoFirma;						
					if (!idNodosToSend.contains(contRef.getReferenciaFirma())) {
						idNodosToSend.add(contRef.getReferenciaFirma().replace("#", ""));
					}
					
				} else if (contenidoFirma instanceof ContenidoFirmaCertificadoAlmacenableInside) {
					ContenidoFirmaCertificadoAlmacenableInside contAlm = (ContenidoFirmaCertificadoAlmacenableInside) contenidoFirma;
					if (!idNodosToSend.contains(firma.getIdentificadorEnDocumento())) {
						idNodosToSend.add(firma.getIdentificadorEnDocumento());
					}
				}
			}
		}
		
		return idNodosToSend;
	}
	
	/**
	 * Devuelve la firma cuyo identificador de nodo coincida con el identificador pasado en el parámetro
	 * @param identificador
	 * @param firmas
	 * @return
	 */
	public static FirmaInside getFirmaInsideByIdentificadorEnDocumento (String identificador, List<FirmaInside> firmas) {
		FirmaInside firma = null;
		if (identificador != null) {
			int i = 0;
			
			while (i < firmas.size() && firma == null) {
				if (identificador.contentEquals(firmas.get(i).getIdentificadorEnDocumento())) {
					firma = firmas.get(i);					
				}
				i++;
			}
		}
		return firma;
	}
	
	/**
	 * Devuelve una lista con las firmas que sean de tipo CSV
	 * @param firmas
	 * @return
	 */
	public static List<FirmaInside> getFirmasCSV (List<FirmaInside> firmas) {
		List<FirmaInside> firmasCSV = new ArrayList<FirmaInside> ();
		if (firmas != null) {
			for (FirmaInside firma : firmas) {
				if (firma.getTipoFirma().equals(FirmaInsideTipoFirmaEnum.TF_01)) {
					firmasCSV.add(firma);
				}
			}
		}
		return firmasCSV;
	}
	
	/**
	 * Devuelve una lista con las firmas que NO sean de tipo CSV
	 * @param firmas
	 * @return
	 */
	public static List<FirmaInside> getFirmasNoCSV (List<FirmaInside> firmas) {
		List<FirmaInside> firmasCSV = new ArrayList<FirmaInside> ();
		if (firmas != null) {
			for (FirmaInside firma : firmas) {
				if (!firma.getTipoFirma().equals(FirmaInsideTipoFirmaEnum.TF_01)) {
					firmasCSV.add(firma);
				}
			}
		}
		return firmasCSV;
	}
	
	/**
	 * Obtiene el valor de un metadato adicional.
	 * Devuelve la cadena vacía si no lo encuentra
	 * @param metadatos
	 * @param nombreMetadato
	 * @return
	 */
	public static String getMetadatoAdicionalNotNull (ObjetoInsideMetadatos metadatos, String nombreMetadato) {
		String ret = getMetadatoAdicional (metadatos, nombreMetadato);
		if (ret == null) {
			ret = "";
		}
		
		return ret;
	}
	/**
	 * Obtiene el valor de un metadato Adicional.
	 * Devuelve null si no lo encuentra.
	 */
	public static String getMetadatoAdicional (ObjetoInsideMetadatos metadatos, String nombreMetadato) {
		String ret = null;
		if (metadatos != null && metadatos.getMetadatosAdicionales() != null) {
			if (metadatos.getMetadatosAdicionales().get(nombreMetadato) != null) {
				ret = metadatos.getMetadatosAdicionales().get(nombreMetadato).getValor().toString();
			} 
		}
		return ret;
	}
}
