/* Copyright (C) 2012-13 MINHAP, Gobierno de España
   This program is licensed and may be used, modified and redistributed under the terms
   of the European Public License (EUPL), either version 1.1 or (at your
   option) any later version as soon as they are approved by the European Commission.
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
   or implied. See the License for the specific language governing permissions and
   more details.
   You should have received a copy of the EUPL1.1 license
   along with this program; if not, you may find it at
   http://joinup.ec.europa.eu/software/page/eupl/licence-eupl */

package es.mpt.dsic.inside.service.util;


import java.io.File;
import java.io.IOException;
import java.util.List;

import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;
import javax.xml.validation.ValidatorHandler;

import org.xml.sax.ContentHandler;

import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;



public class XMLUtils {
	
	final static String W3C_XML_SCHEMA = "http://www.w3.org/2001/XMLSchema";


	/**
	 * Obtiene un Source por cada schema que haya dentro del directorio "dir"
	 * @param dir
	 * @return
	 */
	public static Source[] getSchemasSources (String dir) {
		
		File f = new File (dir);
		
		List<File> filesIn = FileUtils.getFilesInFolder (f, ".xsd");
		
		Source[] schemasSources = new Source[filesIn.size()];
	    int i = 0;
	    for (File schema : filesIn) {
	      schemasSources[i] = new StreamSource(schema);
	      i++;
	    }
		
		return schemasSources;
	}
	
	/**
	 * Crea un parseador para validar contra unos schemas pasados como parámetro
	 * @param schemasSources schemas contra los que se desea validar
	 * @return El parseador
	 * @throws SAXException Cuando no puedan parsearse los schemas
	 * @throws IOException Cuando no se proporcionen los schemas
	 */
	public static XMLReader createParserForValidation (Source[] schemasSources) throws SAXException, IOException {
			
			SchemaFactory schemaFactory = SchemaFactory.newInstance(W3C_XML_SCHEMA);
	    	
	    	if (schemasSources.length == 0) {
	    		throw new IOException ("No se han proporcionado los schemas");
	    	}
	    	
	    	
	    	Schema schemaGrammar = schemaFactory.newSchema(schemasSources);
	    	
	        Validator schemaValidator = schemaGrammar.newValidator();
	        ValidatorHandler vHandler = schemaGrammar.newValidatorHandler();
	        
	        DefaultHandler validationHandler = new DefaultHandler();
	        schemaValidator.setErrorHandler(validationHandler);
	        
	        ContentHandler cHandler = validationHandler;
	        vHandler.setContentHandler(cHandler);
	        
	        XMLReader parser = XMLReaderFactory.createXMLReader();
	        	        
	        parser.setContentHandler(vHandler);
	        
	        return parser;
		}  
	}
	

