/* Copyright (C) 2012-13 MINHAP, Gobierno de España
   This program is licensed and may be used, modified and redistributed under the terms
   of the European Public License (EUPL), either version 1.1 or (at your
   option) any later version as soon as they are approved by the European Commission.
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
   or implied. See the License for the specific language governing permissions and
   more details.
   You should have received a copy of the EUPL1.1 license
   along with this program; if not, you may find it at
   http://joinup.ec.europa.eu/software/page/eupl/licence-eupl */

package es.mpt.dsic.inside.service.visualizacion.impl;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.UUID;
import java.util.concurrent.atomic.AtomicInteger;

import javax.annotation.PostConstruct;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Required;

import es.mpt.dsic.eeutil.client.model.ApplicationLogin;
import es.mpt.dsic.eeutil.client.model.DocumentoContenido;
import es.mpt.dsic.eeutil.client.model.Item;
import es.mpt.dsic.eeutil.client.model.ListaCadenas;
import es.mpt.dsic.eeutil.client.model.ListaPropiedades;
import es.mpt.dsic.eeutil.client.model.OpcionesLogo;
import es.mpt.dsic.eeutil.client.model.OpcionesVisualizacion;
import es.mpt.dsic.eeutil.client.model.Propiedad;
import es.mpt.dsic.eeutil.client.model.SalidaVisualizacion;
import es.mpt.dsic.eeutil.client.service.EeUtilService;
import es.mpt.dsic.infofirma.model.FirmaElectronica;
import es.mpt.dsic.infofirma.model.InfoFirmaElectronica;
import es.mpt.dsic.infofirma.model.InfoFirmante;
import es.mpt.dsic.infofirma.model.OpcionesInfoFirma;
import es.mpt.dsic.infofirma.service.InfoFirmaService;
import es.mpt.dsic.infofirma.service.exception.InfoFirmaServiceException;
import es.mpt.dsic.inside.model.converter.InsideConverterUtils;
import es.mpt.dsic.inside.model.converter.exception.InsideConverterUtilsException;
import es.mpt.dsic.inside.model.objetos.ObjetoInsideMetadatoAdicional;
import es.mpt.dsic.inside.model.objetos.ObjetoInsideMetadatos;
import es.mpt.dsic.inside.model.objetos.documento.ObjetoDocumentoInside;
import es.mpt.dsic.inside.model.objetos.documento.ObjetoDocumentoInsideContenido;
import es.mpt.dsic.inside.model.objetos.documento.metadatos.ObjetoDocumentoInsideMetadatos;
import es.mpt.dsic.inside.model.objetos.expediente.ObjetoExpedienteInside;

import es.mpt.dsic.inside.model.objetos.expediente.indice.ObjetoExpedienteInsideIndiceContenido;
import es.mpt.dsic.inside.model.objetos.expediente.indice.ObjetoExpedienteInsideIndiceContenidoCarpetaIndizada;
import es.mpt.dsic.inside.model.objetos.expediente.indice.ObjetoExpedienteInsideIndiceContenidoDocumentoIndizado;
import es.mpt.dsic.inside.model.objetos.expediente.indice.ObjetoExpedienteInsideIndiceContenidoElementoIndizado;
import es.mpt.dsic.inside.model.objetos.firmas.FirmaInside;

import es.mpt.dsic.inside.model.objetos.firmas.contenido.ContenidoFirmaCSVInside;
import es.mpt.dsic.inside.model.objetos.firmas.contenido.ContenidoFirmaCertificadoAlmacenableInside;

import es.mpt.dsic.inside.service.foliado.impl.InsideServiceFoliadoImpl;
import es.mpt.dsic.inside.service.store.InsideServiceStore;
import es.mpt.dsic.inside.service.store.exception.InsideServiceStoreException;
import es.mpt.dsic.inside.service.store.exception.InsideStoreObjectNotFoundException;
import es.mpt.dsic.inside.service.util.InsideObjectsUtils;
import es.mpt.dsic.inside.service.util.InsideUtils;
import es.mpt.dsic.inside.service.visualizacion.InsideServiceVisualizacionIndice;
import es.mpt.dsic.inside.service.visualizacion.OpcionesVisualizacionIndice;
import es.mpt.dsic.inside.service.visualizacion.ResultadoVisualizacionDocumento;
import es.mpt.dsic.inside.service.visualizacion.exception.InsideServiceVisualizacionIndiceException;

public class InsideServiceVisualizacionIndiceImpl implements InsideServiceVisualizacionIndice{
	
protected static final Log logger = LogFactory.getLog(InsideServiceVisualizacionIndiceImpl.class);
	
	private Properties properties;
	private EeUtilService port;
	private ApplicationLogin applicationLogin;
	
	private List<String> filasNombreOrganismo;
	private String modeloExpediente;
	private String modeloDocumento;
	
	private boolean activo;
	private static String ACTIVO = "S";
	
	@Autowired
	private InsideServiceStore insideStore;
	@Autowired	
	private InfoFirmaService infoFirmaService;
	
	@PostConstruct
	public void configure () {
		
		String visualizacionActivo = properties.getProperty("visualizacion.activo");
		
		if (!ACTIVO.contentEquals(visualizacionActivo)) {
			logger.info("El WS de VISUALIZACIÓN no está activo");
			activo = false;
		} else {
		
			URL url = null;
			String urlVisualizacion = null;
			try {
				urlVisualizacion = properties.getProperty("visualizacion.url");
				logger.debug(String.format("El WS de VISUALIZACION se encuentra en %s", urlVisualizacion));
				url = new URL (urlVisualizacion);
			} catch (MalformedURLException me) {
				logger.error("No se puede crear la URL del servicio de visualizacion " + urlVisualizacion, me);
			}
			
			es.mpt.dsic.eeutil.client.service.EeUtilServiceImplService ss = new es.mpt.dsic.eeutil.client.service.EeUtilServiceImplService(url);	
			
		    port = ss.getEeUtilServiceImplPort();  
		    
		    applicationLogin = new ApplicationLogin();
		    applicationLogin.setIdaplicacion(properties.getProperty("visualizacion.idaplicacion"));
		    applicationLogin.setPassword(properties.getProperty("visualizacion.password"));
		    
		    logger.debug(String.format("Utilizando para visualizacion idaplicacion/password : %s/%s", properties.getProperty("visualizacion.idaplicacion"), properties.getProperty("visualizacion.password")));
		    
		    filasNombreOrganismo = Arrays.asList(properties.getProperty("visualizacion.nombre.organismo").split(";"));
		    
		    logger.debug(String.format("Utilizando nombre de organismo: %s", properties.getProperty("visualizacion.nombre.organismo")));
		    
		    modeloExpediente = properties.getProperty("visualizacion.modelo.expediente");
		    
		    logger.debug(String.format("Utilizando modelo de expediente: %s", properties.getProperty("visualizacion.modelo.expediente")));
		    
		    modeloDocumento = properties.getProperty("visualizacion.modelo.documento");
		    
		    logger.debug(String.format("Utilizando modelo de documento: %s", properties.getProperty("visualizacion.modelo.documento")));
		    
		    activo = true;
		}	    
	    
	}
	
	@Override
	/**
	 * Devuelve un expediente Inside con la visualización del índice con las opciones indicadas en el parámetro.
	 */
	public ObjetoExpedienteInside expedienteConVisualizacionIndice (ObjetoExpedienteInside expediente, OpcionesVisualizacionIndice opciones) throws InsideServiceVisualizacionIndiceException{
		
		if (!activo) {
			throw new InsideServiceVisualizacionIndiceException ("El WS de VISUALIZACION no se encuentra activo");
		}
		// Construimos los parámetros de la llamada al servicio.
		Item itemExpediente = expedienteInsideToItemVisualizacion (expediente, opciones);
		
		OpcionesVisualizacion oVisualizacion = getOpcionesVisualizacionWS(opciones, modeloExpediente);
		SalidaVisualizacion salida = null;
		try {
			// Llamada al servicio
			salida = port.visualizar(applicationLogin, itemExpediente, oVisualizacion);
		} catch (Throwable t) {
			throw new InsideServiceVisualizacionIndiceException ("Error al obtener la visualización del índice", t);
		}
		
		// Rellenamos el objeto que contiene la visualización del índice.
		ObjetoDocumentoInsideContenido contenidoVisualizacion = new ObjetoDocumentoInsideContenido ();
		contenidoVisualizacion.setIdentificadorEnDocumento("VISUALIZACION_INDICE");
		contenidoVisualizacion.setValorBinario(salida.getDocumentoContenido().getBytesDocumento());
		contenidoVisualizacion.setMime(salida.getDocumentoContenido().getMimeDocumento());
		String nombreFormato = InsideUtils.getNombreFormatoByMime(salida.getDocumentoContenido().getMimeDocumento());
		if (nombreFormato == null) {
			throw new InsideServiceVisualizacionIndiceException ("No se ha encontrado nombre de formato para el mime del contenido de la visualización del Índice");
		}
		contenidoVisualizacion.setNombreFormato(nombreFormato);
		
		expediente.setVisualizacionIndice(contenidoVisualizacion);
		return expediente;

	}
	/**
	 * Devuelve un expediente Inside con la visualización del índice con las opciones por defecto.	 * 
	 */
	@Override
	public ObjetoExpedienteInside expedienteConVisualizacionIndice (ObjetoExpedienteInside expediente) throws InsideServiceVisualizacionIndiceException{
				
		OpcionesVisualizacionIndice opciones = new OpcionesVisualizacionIndice ();
		opciones.setExterno(false);
		opciones.setEstamparImagen(true);
		opciones.setEstamparNombreOrganismo(true);
		opciones.setLineasNombreOrganismo(filasNombreOrganismo);
		
		return expedienteConVisualizacionIndice (expediente, opciones);
	
	}
	
	@Override
	/**
	 * Devuelve la visualización de un documento inside, con opciones de visualización.
	 */
	public ResultadoVisualizacionDocumento visualizacionDocumento (ObjetoDocumentoInside documento, OpcionesVisualizacionIndice opciones) throws InsideServiceVisualizacionIndiceException {
		
		if (!activo) {
			throw new InsideServiceVisualizacionIndiceException ("El WS de VISUALIZACION no se encuentra activo");
		}
		
		ResultadoVisualizacionDocumento resultado = new ResultadoVisualizacionDocumento ();
		Item itemDocumento = documentoInsideToItemVisualizacion (documento);
		
		OpcionesVisualizacion oVisualizacion = getOpcionesVisualizacionWS(opciones, modeloDocumento);
		SalidaVisualizacion salida = null;
		try {
			// Llamada al servicio
			salida = port.visualizar(applicationLogin, itemDocumento, oVisualizacion);
		} catch (Throwable t) {
			throw new InsideServiceVisualizacionIndiceException ("Error al obtener la visualización del documento", t);
		}
		
		resultado.setVisualizacion(salida.getDocumentoContenido().getBytesDocumento());
		resultado.setMime(salida.getDocumentoContenido().getMimeDocumento());
		
		return resultado;
	}
	
	@Override
	/**
	 * Devuelve la visualización de un documento inside, sin opciones, por lo que se usarán las de por defecto
	 */
	public ResultadoVisualizacionDocumento visualizacionDocumento (ObjetoDocumentoInside documento) throws InsideServiceVisualizacionIndiceException {
		
		if (!activo) {
			throw new InsideServiceVisualizacionIndiceException ("El WS de VISUALIZACION no se encuentra activo");
		}
		
		OpcionesVisualizacionIndice opciones = new OpcionesVisualizacionIndice ();
		opciones.setExterno(false);
		opciones.setEstamparImagen(true);
		opciones.setEstamparNombreOrganismo(true);
		opciones.setLineasNombreOrganismo(filasNombreOrganismo);
		
		return visualizacionDocumento (documento, opciones);
		
	}
	
	@Override
	/**
	 * Devuelve true si el servicio está activo, false en caso contrario	 
	 */
	public boolean isActivo () {
		return this.activo;
	}
	
	/**
	 * Construcción de la estructura de datos jerárquica que le pasamos al WS de obtención de visualización del índice.
	 * @param expediente
	 * @return
	 */
	private Item expedienteInsideToItemVisualizacion (ObjetoExpedienteInside expediente, OpcionesVisualizacionIndice opciones) throws InsideServiceVisualizacionIndiceException {
		Item padre = new Item ();
		padre.setIdentificador(expediente.getIdentificador());
		padre.setNombre(expediente.getIdentificador());
		padre.setPropiedadesItem(getPropiedadesExpediente(expediente));		
		
		fillItemWithIndice (padre, expediente.getIndice().getIndiceContenido().getElementosIndizados(), new AtomicInteger(), opciones);
		
		return padre;
	}
	
	/**
	 * Construye la estructura jerárquica de un item a partir de sus hijos.
	 * @param item
	 * @param elementosIndice Elementos que serán hijos de "item"
	 * @param AtomicInteger Entero que se va incrementando.
	 */
	private void fillItemWithIndice (Item item, List<ObjetoExpedienteInsideIndiceContenidoElementoIndizado> elementosIndice, AtomicInteger orden, OpcionesVisualizacionIndice opciones) throws InsideServiceVisualizacionIndiceException {
		
		// Metemos un AtomicInteger como parámetro porque los Integer no se pueden incrementar, y los int se pasan por valor.
		
		for (ObjetoExpedienteInsideIndiceContenidoElementoIndizado elementoHijo : elementosIndice) {
			Item itemHijo = new Item();
			itemHijo.setPadre(item);
			itemHijo.setIdentificador("ID_" + UUID.randomUUID());
			
			if (elementoHijo instanceof ObjetoExpedienteInsideIndiceContenidoDocumentoIndizado) {
				ObjetoExpedienteInsideIndiceContenidoDocumentoIndizado docIndizado = (ObjetoExpedienteInsideIndiceContenidoDocumentoIndizado) elementoHijo;
				
				itemHijo.setNombre(getNombreDocumentoIndizado(docIndizado, opciones.isExterno()));
				itemHijo.setPropiedadesItem(getPropiedadesDocumentoIndizado(docIndizado, orden.addAndGet(1)));
				
			} else if (elementoHijo instanceof ObjetoExpedienteInsideIndiceContenido) {
				ObjetoExpedienteInsideIndiceContenido indiceContenidoIndizado = (ObjetoExpedienteInsideIndiceContenido) elementoHijo;
				itemHijo.setNombre("Expediente Electrónico: " + indiceContenidoIndizado.getIdentificadorExpedienteAsociado());
				fillItemWithIndice (itemHijo, indiceContenidoIndizado.getElementosIndizados(), orden, opciones);
				
			} else if (elementoHijo instanceof ObjetoExpedienteInsideIndiceContenidoCarpetaIndizada) {
				ObjetoExpedienteInsideIndiceContenidoCarpetaIndizada carpetaIndizada = (ObjetoExpedienteInsideIndiceContenidoCarpetaIndizada) elementoHijo;
				itemHijo.setNombre(carpetaIndizada.getIdentificadorCarpeta());
				fillItemWithIndice (itemHijo, carpetaIndizada.getElementosIndizados(), orden, opciones);				
			}
			
			item.getHijos().getItems().getItems().add(itemHijo);		
		}
	}
	
	private void fillItemWithInfoDocumento (Item item, ObjetoDocumentoInside documento) throws InsideServiceVisualizacionIndiceException{
		
		byte[] contenidoDocumento = null;
		String mimeContenidoDocumento = null;
		InfoFirmaElectronica infoFirma = null;
		// Obtenemos las firmas que NO son CSV, esto nos valdrá más adelante
		List<FirmaInside> firmasNoCSV = InsideObjectsUtils.getFirmasNoCSV(documento.getFirmas());
		
		// CASO 1 - El documento se encuentra en el contenido del documento Inside
		if (documento.getContenido().getReferencia() == null) {
			
			List<FirmaInside> firmasRef = InsideObjectsUtils.getFirmasReferencianNodo(documento.getContenido().getIdentificadorEnDocumento(), documento.getFirmas());
			
			// CASO 1.1 El contenido es referenciado desde alguna firma
			if (firmasRef.size() > 0) {
				
				// Llamaremos al servicio de infofirma para obtener información del contenido y los firmantes
				try {
					
					infoFirma = llamadaInfoFirmaService (documento.getContenido().getValorBinario(), true, true, true);
					contenidoDocumento = infoFirma.getContenidoFirmado();
					mimeContenidoDocumento = infoFirma.getMimeContenidoFirmado();
					fillItemWithFirmasInfo(item, infoFirma);
					
				} catch (InfoFirmaServiceException e) {
					throw new InsideServiceVisualizacionIndiceException ("Error al obtener información de la firma a partir del contenido del documento", e);
				}
				
			// CASO 1.2 El contenido NO es referenciado desde ninguna firma
			} else {
				
				// Tenemos el contenido
				contenidoDocumento = documento.getContenido().getValorBinario();
				mimeContenidoDocumento = documento.getContenido().getMime();
								
				// CASO 1.2.1 Hay firmas que NO son CSV
				if (firmasNoCSV.size() > 0) {
					
					// Obtenemos los firmantes llamando a infofirmaservice de cada firma que no sea referencia
					// TODO ¿Esto funciona?
					for (FirmaInside firma : firmasNoCSV) {
						if (firma.getContenidoFirma() instanceof ContenidoFirmaCertificadoAlmacenableInside) {
							ContenidoFirmaCertificadoAlmacenableInside contAlm = (ContenidoFirmaCertificadoAlmacenableInside) firma.getContenidoFirma();
							try {
								infoFirma = llamadaInfoFirmaService (contAlm.getValorBinario(), false, true, true);								
							} catch (InfoFirmaServiceException e) {
								throw new InsideServiceVisualizacionIndiceException ("Error al obtener información de la firma a partir de una firma que NO está referenciada", e);
							}
							fillItemWithFirmasInfo (item, infoFirma);
						}
							
						
					}
					
				// CASO 1.2.2 Sólo hay firmas CSV	
				} else {
					
					// No hacemos nada
				}
			}
						
			
		// CASO 2 - El documento se encuentra implícito en alguna firma
		} else {
			// Obtenemos la firma implícita
			FirmaInside firmaImplicita = InsideObjectsUtils.getFirmaInsideByIdentificadorEnDocumento(documento.getContenido().getReferencia().replace("#", ""), documento.getFirmas());
			ContenidoFirmaCertificadoAlmacenableInside contAlm = (ContenidoFirmaCertificadoAlmacenableInside) firmaImplicita.getContenidoFirma();
			
			// Llamamos a infofirma para obtener el contenido firmado y los firmantes.
			try {
				infoFirma = llamadaInfoFirmaService (contAlm.getValorBinario(), true, true, true);
				contenidoDocumento = infoFirma.getContenidoFirmado();
				mimeContenidoDocumento = infoFirma.getMimeContenidoFirmado();
				
			} catch (InfoFirmaServiceException e) {
				throw new InsideServiceVisualizacionIndiceException ("Error al obtener información de la firma a partir de una firma implícita referenciada desde el documento", e);
			}
			fillItemWithFirmasInfo (item, infoFirma);			
		}
		
		// Rellenamos el contenido del documento.
		DocumentoContenido dc = new DocumentoContenido ();
		dc.setBytesDocumento(contenidoDocumento);
		dc.setMimeDocumento(mimeContenidoDocumento);
		
		item.setDocumentoContenido(dc);
		
		// Por último, recuperamos las firmas CSV para meterlas también el Item de la petición
		List<FirmaInside> firmasCSV = InsideObjectsUtils.getFirmasCSV(documento.getFirmas());
		for (FirmaInside firmaCSV : firmasCSV) {
			Item itemCSV = getItemFirmaCSV (item, firmaCSV);
			item.getHijos().getItems().getItems().add(itemCSV);
		}
		
	}
	
	private InfoFirmaElectronica llamadaInfoFirmaService (byte[] firma, boolean obtenerDatosFirmados, boolean obtenerFirmantes,  boolean obtenerTipoFirma) throws InfoFirmaServiceException {
			
			FirmaElectronica peticionFirma = new FirmaElectronica ();
			peticionFirma.setFirma(firma);
			
			OpcionesInfoFirma peticionOpciones = new OpcionesInfoFirma ();
			peticionOpciones.setObtenerDatosFirmados(obtenerDatosFirmados);
			peticionOpciones.setObtenerFirmantes(obtenerFirmantes);
			peticionOpciones.setObtenerTipoFirma(obtenerTipoFirma);
			
			return infoFirmaService.getInfoFirma(peticionFirma, peticionOpciones);
			
	}
	
	private void fillItemWithFirmasInfo (Item item, InfoFirmaElectronica infoFirma) {
		
		if (infoFirma.getFirmantes() != null) {
			
			for (InfoFirmante infoFirmante : infoFirma.getFirmantes()) {
				Item itemFirma = getItemFirmaNoCSV (item, infoFirma.getTipoFirma().getTipoFirma(), infoFirmante);
				item.getHijos().getItems().getItems().add(itemFirma);
			}
		}
		
	}
	
	/**
	 * Construye la lista de propiedades de un expediente que aparecerán en la visualización del índice.
	 * @param expediente
	 * @return
	 */
	private ListaPropiedades getPropiedadesExpediente (ObjetoExpedienteInside expediente) {
		
		ListaPropiedades lista = new ListaPropiedades();
		ListaPropiedades.Propiedades prop = new ListaPropiedades.Propiedades();
		lista.setPropiedades(prop);
		
		lista.getPropiedades().getPropiedades().add(createPropiedad("Referencia", expediente.getIdentificador()));
		lista.getPropiedades().getPropiedades().add(createPropiedad("Descripción", InsideObjectsUtils.getMetadatoAdicionalNotNull(expediente.getMetadatos(), "Descripcion")));
		lista.getPropiedades().getPropiedades().add(createPropiedad("Órganos", InsideUtils.listaToString(expediente.getMetadatos().getOrgano().toArray(new String[]{}), "; ", false, true)));
		lista.getPropiedades().getPropiedades().add(createPropiedad("Procedimiento", expediente.getMetadatos().getClasificacion()));
		lista.getPropiedades().getPropiedades().add(createPropiedad("Descripción Procedimiento", InsideObjectsUtils.getMetadatoAdicionalNotNull(expediente.getMetadatos(), "DescripcionProcedimiento")));
		lista.getPropiedades().getPropiedades().add(createPropiedad("Fecha Apertura Expediente ENI", InsideUtils.calendarToStringISO8601(expediente.getMetadatos().getFechaAperturaExpediente())));
		lista.getPropiedades().getPropiedades().add(createPropiedad("Interesados", InsideUtils.listaToString(expediente.getMetadatos().getInteresado().toArray(new String[]{}), "; ", false, true)));
		
		return lista;
	}
	
	private ListaPropiedades getPropiedadesDocumento (ObjetoDocumentoInside documento) {
		ListaPropiedades lista = new ListaPropiedades();
		ListaPropiedades.Propiedades prop = new ListaPropiedades.Propiedades();
		lista.setPropiedades(prop);
		
		lista.getPropiedades().getPropiedades().add(createPropiedad("Versión NTI", documento.getMetadatos().getVersionNTI()));
		lista.getPropiedades().getPropiedades().add(createPropiedad("Identificador", documento.getIdentificador()));
		lista.getPropiedades().getPropiedades().add(createPropiedad("Órganos", InsideUtils.listaToString(documento.getMetadatos().getOrgano().toArray(new String[]{}), "; ", false, true)));
		lista.getPropiedades().getPropiedades().add(createPropiedad("Fecha Captura", InsideUtils.calendarToStringISO8601(documento.getMetadatos().getFechaCaptura())));
		lista.getPropiedades().getPropiedades().add(createPropiedad("Origen", getOrigen(documento.getMetadatos().isOrigenCiudadanoAdministracion())));
		lista.getPropiedades().getPropiedades().add(createPropiedad("Estado Elaboración", documento.getMetadatos().getEstadoElaboracion().getValorEstadoElaboracion().value()));
		lista.getPropiedades().getPropiedades().add(createPropiedad("Tipo Documental", documento.getMetadatos().getTipoDocumental().value()));
		
		Map<String, ObjetoInsideMetadatoAdicional> metadatosAdicionales = documento.getMetadatos().getMetadatosAdicionales();
		Iterator<String> nombresMeta = metadatosAdicionales.keySet().iterator();
		
		while (nombresMeta.hasNext()) {
			String nombre = nombresMeta.next();
			String valor = metadatosAdicionales.get(nombre).getValor().toString();
			lista.getPropiedades().getPropiedades().add(createPropiedad(nombre, valor));
			
		}
		
		return lista;
	}
	
	private Item getItemFirmaCSV (Item padre, FirmaInside firmaInside) {
		Item firma = new Item ();
		firma.setIdentificador("ID_" + UUID.randomUUID());
		firma.setNombre("TF01 - CSV");
		firma.setPadre(padre);
		
		ListaPropiedades lista = new ListaPropiedades();
		ListaPropiedades.Propiedades prop = new ListaPropiedades.Propiedades();
		lista.setPropiedades(prop);
		
		ContenidoFirmaCSVInside contenidoFirma = (ContenidoFirmaCSVInside) firmaInside.getContenidoFirma();
		lista.getPropiedades().getPropiedades().add(createPropiedad("valorCSV", contenidoFirma.getValorCSV()));
		lista.getPropiedades().getPropiedades().add(createPropiedad("regulacionCSV", contenidoFirma.getRegulacionGeneracionCSV()));
		
		firma.setPropiedadesItem(lista);
		
		return firma;
	}
	
	private Item getItemFirmaNoCSV (Item padre, String tipoFirma, InfoFirmante infoFirmante) {
		Item firma = new Item ();
		
		firma.setIdentificador("ID_" + UUID.randomUUID());
		firma.setNombre(getTipoFirma(tipoFirma));
		firma.setPadre(padre);
		
		ListaPropiedades lista = new ListaPropiedades();
		ListaPropiedades.Propiedades prop = new ListaPropiedades.Propiedades();
		lista.setPropiedades(prop);
		
		lista.getPropiedades().getPropiedades().add(createPropiedad("nombreFirmante", infoFirmante.getDescripcion()));
		lista.getPropiedades().getPropiedades().add(createPropiedad("fechaFirma", infoFirmante.getFecha()));
		
		firma.setPropiedadesItem(lista);
		
		return firma;
	}
	
	private String getTipoFirma (String tipo) {
		
		String result = "";
		if (tipo.equalsIgnoreCase("XADES DETACHED")) {
			result = "TF02 - XAdES internally detached";
		} else if (tipo.equalsIgnoreCase("XADES ENVELOPED")) {
			result = "TF03 - XAdES enveloped";
		} else if (tipo.equalsIgnoreCase("CADES")) {
			result = "TF05 - CAdES attached";
		} else if (tipo.equalsIgnoreCase("PADES")) {
			result = "TF06 - PAdES";
		}	
		
		return result;
	}
	
	private String getOrigen (boolean origen) {
		String sOrigen = "Ciudadano";
		
		if (origen) {
			sOrigen = "Administración";
		}
		return sOrigen;
	}
	
	
	
	/**
	 * Construye la lista de propiedades de un documento que aparecerán en la visualización del índice.
	 * @param documentoIndizado
	 * @return
	 */
	private ListaPropiedades getPropiedadesDocumentoIndizado (ObjetoExpedienteInsideIndiceContenidoDocumentoIndizado documentoIndizado, int orden) {
		ListaPropiedades lista = new ListaPropiedades();
		ListaPropiedades.Propiedades prop = new ListaPropiedades.Propiedades();
		lista.setPropiedades(prop);
		
		// Queremos que el orden sea absoluto, así que no nos vale el orden del documento, que es relativo a la carpeta donde está.
		lista.getPropiedades().getPropiedades().add(createPropiedad("Orden", new Integer(orden).toString()));
		//lista.getPropiedades().getPropiedades().add(createPropiedad("Orden", new Integer(documentoIndizado.getOrden()).toString()));
		lista.getPropiedades().getPropiedades().add(createPropiedad("Hash", documentoIndizado.getValorHuella()));		
		return lista;
	}
	
	/**
	 * Obtiene el nombre que se mostrará de un documento Indizado.
	 * Si el expediente es externo a InSide (no lo tenemos almacenado), se meterá el identificador del documento.
	 * Si el expediente es interno (lo tenemos almacenado): si el documento tiene el Metadato Adicional "DescripcionDocumento" se enviará ese, en caso contrario
	 * se enviará el identificador del documento indizado.
	 */
	private String getNombreDocumentoIndizado (ObjetoExpedienteInsideIndiceContenidoDocumentoIndizado documentoIndizado, boolean externo) throws InsideServiceVisualizacionIndiceException {
		
		String nombreDocumento = "";
		if (externo) {
			nombreDocumento = "[" + documentoIndizado.getIdentificadorDocumento() + "]";
		} else {
			ObjetoDocumentoInsideMetadatos metadatosDocumento = null;
			try {
				metadatosDocumento = insideStore.getObjectMetadatos(ObjetoDocumentoInside.class, documentoIndizado.getIdentificadorDocumento());
				
			} catch (InsideStoreObjectNotFoundException e) {
				throw new InsideServiceVisualizacionIndiceException ("Error al obtener la descripcion del documento con identificador " + documentoIndizado.getIdentificadorDocumento(), e);
			} catch (InsideServiceStoreException e) {
				throw new InsideServiceVisualizacionIndiceException ("Error al obtener la descripcion del documento con identificador " + documentoIndizado.getIdentificadorDocumento(), e);
			}
			
			nombreDocumento = InsideObjectsUtils.getMetadatoAdicionalNotNull (metadatosDocumento, "DescripcionDocumento");
			nombreDocumento = nombreDocumento + " [" + documentoIndizado.getIdentificadorDocumento() + "]";				
		}
			
		return nombreDocumento;
			
	}
	
	/**
	 * Crea una propiedad con una clave y un valor.
	 * @param clave
	 * @param valor
	 * @return
	 */
	private Propiedad createPropiedad (String clave, String valor) {
		Propiedad p = new Propiedad();
		p.setClave(clave);
		p.setValor(valor);
		return p;
	}
	
	
	/*private ListaPropiedades createListaUnaPropiedad (String clave, String valor) {
		ListaPropiedades lista = new ListaPropiedades();
		ListaPropiedades.Propiedades prop = new ListaPropiedades.Propiedades();
		lista.setPropiedades(prop);
		
		lista.getPropiedades().getPropiedades().add(createPropiedad(clave, valor));
		return lista;
	}*/
	
	
	/**
	 * Crea las opciones de visualización del WS a partir de las opciones de visualizacion de InSide.
	 * @return
	 */
	private OpcionesVisualizacion getOpcionesVisualizacionWS (OpcionesVisualizacionIndice opciones, String modelo) throws InsideServiceVisualizacionIndiceException {
		
		if (opciones == null) {
			throw new InsideServiceVisualizacionIndiceException ("No se pueden obtener las opciones de visualizacion del WS porque el parámetro es nulo");
		}
		OpcionesVisualizacion oVisualizacionWS = new OpcionesVisualizacion();
		OpcionesLogo oLogo = new OpcionesLogo();
		oLogo.setEstamparLogo(opciones.isEstamparImagen());
		oLogo.setEstamparNombreOrganismo(opciones.isEstamparNombreOrganismo());
		
		ListaCadenas listaCadenas = new ListaCadenas();
		ListaCadenas.Cadenas cadenas = new ListaCadenas.Cadenas();
		listaCadenas.setCadenas(cadenas);
		
		// Si se ha marcado la opcion de estampar el nombre del organismo y el nombre no se envía, se estampa el nombre por defecto.
		if (opciones.isEstamparNombreOrganismo()) {
			if (opciones.getLineasNombreOrganismo() != null && opciones.getLineasNombreOrganismo().size() > 0) {
				listaCadenas.getCadenas().getCadenas().addAll(opciones.getLineasNombreOrganismo());
			} else {
				listaCadenas.getCadenas().getCadenas().addAll(filasNombreOrganismo);
			}
		} else {
			// TODO- Hay que cambiar el eeutil para que si el parámetro "estamparNombreOrganismo" es false no estampe nada y no falle.
			listaCadenas.getCadenas().getCadenas().add("");
		}
						
		oLogo.setListaCadenasNombreOrganismo(listaCadenas);		
		oVisualizacionWS.setOpcionesLogo(oLogo);
		oVisualizacionWS.setModelo(modelo);
		
		return oVisualizacionWS;
		
	}
	
	private Item documentoInsideToItemVisualizacion (ObjetoDocumentoInside documento) throws InsideServiceVisualizacionIndiceException {
		Item padre = new Item ();
		padre.setIdentificador(documento.getIdentificador());
		padre.setNombre(documento.getIdentificador());
		padre.setPropiedadesItem(getPropiedadesDocumento(documento));		
		
		fillItemWithInfoDocumento (padre, documento);
		
		return padre;
	}
	
	
	
	
	public Properties getProperties() {
		return properties;
	}

	@Required
	public void setProperties(Properties properties) {
		this.properties = properties;
	}
	
	
}
