/* Copyright (C) 2012-13 MINHAP, Gobierno de España
   This program is licensed and may be used, modified and redistributed under the terms
   of the European Public License (EUPL), either version 1.1 or (at your
   option) any later version as soon as they are approved by the European Commission.
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
   or implied. See the License for the specific language governing permissions and
   more details.
   You should have received a copy of the EUPL1.1 license
   along with this program; if not, you may find it at
   http://joinup.ec.europa.eu/software/page/eupl/licence-eupl */

package es.mpt.dsic.inside.ws.exception;

/**
 * Enum para agrupar los distintos errores o clases de errores
 * que se pueden generar en el WS.
 * @author miguel.ortega
 *
 */
public enum InsideWsErrors {
	
	/**
	 * Rangos de Errores tal como se han puesto en la documentación:
	 * E-001		Error de autenticación. Crecenciales erroneas
	 * E-002		Error de autenticación. Operación no permitida
	 * E-1xx		Error de Sintaxis en la llamada WS
	 * E-2xx		Omisión de alguno de los parámetros de entrada
	 * E-3xx		Error en el rango de alguno de los parámetros de entrada
	 * E-4XX		Errores diversos motivados por algún error en la petición.
	 * E-5XX		Error interno de Inside
	 * E-999        Operación no permitida
	 * E-998		Aplicación marcada como deshabilitada
	 */
	
	CREDENTIALS_ERROR(new InsideWSError(001,"Credenciales Erroneas. compruebe id de aplicacion y password")),
	UNAUTHORIZED_OPERATION_ERROR (new InsideWSError (999, "Operación no autorizada")),
	DISABLED_ERROR (new InsideWSError (998, "La aplicación está marcada como deshabilitada")),
	
	UNMARSHALL_ERROR(new InsideWSError(101,"Error en los datos de entrada/salida. Posiblemente falte algun campo obligatorio o el xml de la petición esté mal formado. Compruebe el WSDL y los campos obligatorios.")),
	XML_PARSING_ERROR(new InsideWSError(102,"Error parseando xml.")),
	
	EXPEDIENTE_ENI_VACIO(new InsideWSError(201,"El parámetro Expediente ENI no puede estar vacío")),
	METADATOS_EXPEDIENTE_ENI_VACIO(new InsideWSError(202,"El parámetro metadatos del Expediente ENI no puede estar vacío")), 
	IDENTIFICADOR_EXPEDIENTE_ENI_VACIO(new InsideWSError(203,"El parámetro identificador de los metadatos del Expediente ENI no puede estar vacío")),
	IDENTIFICADOR_EXPEDIENTE_VACIO(new InsideWSError(204,"El parámetro identificador del Expediente ENI no puede estar vacío")),
	IDENTIFICADOR_CARPETA_VACIO(new InsideWSError(205,"El parámetro identificador de la carpeta no puede estar vacío")),
	
	VERSION_NTI_10_ERRONEA(new InsideWSError(290,"La versión del NTI del mensaje no puede ser distinta a http://administracionelectronica.gob.es/ENI/XSD/v1.0/expediente-e")), 
	
	OBJECT_VALIDATION_ERROR(new InsideWSError(300,"Error en la validación del objeto")),
	ESTADO_VACIO(new InsideWSError(301,"El valor del estado del expediente no es válido")),
	
	REQUEST_ERROR(new InsideWSError(400,"Error en la petición a Inside")),
	OBJECT_NOT_FOUND(new InsideWSError(401,"No se encuentra el objeto requerido")),
	OBJECT_VERSION_NOT_FOUND(new InsideWSError(402,"No se encuentra el objeto en la versión requerida")),
	OBJECT_ALREADY_EXISTS(new InsideWSError(402,"Ya existe un objeto con el identificador proporcionado")),
	OPERATION_NOT_SUPPORTED(new InsideWSError(499,"Operación no soportada")),
	
	INTERNAL_SERVICE_ERROR(new InsideWSError(500,"Error interno de Inside")), 
	
	;

	private final InsideWSError value;
	
	InsideWsErrors(InsideWSError error){
		value = error;
	}

	public InsideWSError getValue() {
		return value;
	}
	
	public static InsideWsErrors valueFromCodigo(int codigo){
		for (InsideWsErrors c: InsideWsErrors.values()) {
            if (c.value.getCodigo() == codigo) {
                return c;
            }
        }
        throw new IllegalArgumentException("No hay error definido para  el código " + codigo);
	}
}
