/* Copyright (C) 2012-13 MINHAP, Gobierno de España
   This program is licensed and may be used, modified and redistributed under the terms
   of the European Public License (EUPL), either version 1.1 or (at your
   option) any later version as soon as they are approved by the European Commission.
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
   or implied. See the License for the specific language governing permissions and
   more details.
   You should have received a copy of the EUPL1.1 license
   along with this program; if not, you may find it at
   http://joinup.ec.europa.eu/software/page/eupl/licence-eupl */

package es.mpt.dsic.inside.ws.service;

import java.util.List;

import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebResult;
import javax.jws.WebService;
import javax.xml.bind.annotation.XmlElement;

import org.apache.cxf.annotations.GZIP;
import org.apache.cxf.annotations.Logging;
import org.apache.cxf.interceptor.OutFaultInterceptors;
import org.apache.cxf.interceptor.OutInterceptors;

import es.mpt.dsic.inside.ws.cxf.interceptor.InsideWsErrorInterceptor;
import es.mpt.dsic.inside.ws.cxf.interceptor.InsideWsIndiceExpedienteSignerInterceptor;
import es.mpt.dsic.inside.ws.exception.InsideWSException;
import es.mpt.dsic.inside.xml.eni.documento.TipoDocumento;
import es.mpt.dsic.inside.xml.eni.documento.contenido.TipoContenido;
import es.mpt.dsic.inside.xml.eni.expediente.TipoExpediente;
import es.mpt.dsic.inside.xml.eni.expediente.indice.TipoIndice;
import es.mpt.dsic.inside.xml.eni.expediente.metadatos.EnumeracionEstados;
import es.mpt.dsic.inside.xml.eni.expediente.metadatos.TipoMetadatos;
import es.mpt.dsic.inside.xml.inside.ws.busqueda.ConsultaWsInside;
import es.mpt.dsic.inside.xml.inside.ws.documento.DocumentoInsideInfo;
import es.mpt.dsic.inside.xml.inside.DocumentoInsideMetadatos;
import es.mpt.dsic.inside.xml.inside.ws.busqueda.DocumentoResultadoBusqueda;
import es.mpt.dsic.inside.xml.inside.ExpedienteInsideInfo;
import es.mpt.dsic.inside.xml.inside.ExpedienteInsideMetadatos;
import es.mpt.dsic.inside.xml.inside.ws.busqueda.ExpedienteResultadoBusqueda;
import es.mpt.dsic.inside.xml.inside.ws.credential.WSCredentialInside;
import es.mpt.dsic.inside.xml.inside.ws.documento.alta.TipoDocumentoAltaInside;
import es.mpt.dsic.inside.xml.inside.ws.documento.alta.TipoDocumentoAsociadoaExpediente;
import es.mpt.dsic.inside.xml.inside.ws.documento.conversion.TipoDocumentoConversionInside;
import es.mpt.dsic.inside.xml.inside.ws.expediente.conversion.TipoExpedienteConversionInside;
import es.mpt.dsic.inside.xml.inside.TipoDocumentoInside;
import es.mpt.dsic.inside.xml.inside.TipoExpedienteInside;
import es.mpt.dsic.inside.xml.inside.TipoMetadatosAdicionales;
import es.mpt.dsic.inside.xml.inside.TipoVersionInside;

@WebService(name="InSideWebService",
			targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService")
@Logging
@GZIP
@OutInterceptors (interceptors = {},classes={InsideWsIndiceExpedienteSignerInterceptor.class})
@OutFaultInterceptors(interceptors = {},classes={InsideWsErrorInterceptor.class})
public interface InSideWebService {
	
	@WebMethod(operationName = "altaExpediente", action = "urn:altaExpediente")
	@WebResult(name = "ExpedienteInsideInfo", partName = "ExpedienteInsideInfo",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/expediente-e/expedienteInfo")
	public ExpedienteInsideInfo altaExpediente(
			@WebParam(name = "credential",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/types/credential") @XmlElement(required = true, name = "credential") WSCredentialInside info,
			@WebParam(name = "metadatosEni") @XmlElement(required = true, name = "metadatosEni") TipoMetadatos metadatosEni,
			@WebParam(name = "metadatosAdicionales") @XmlElement(required = false, name = "metadatosAdicionales") TipoMetadatosAdicionales metadatosAdicionales)
			throws InsideWSException;
	
	@WebMethod(operationName = "altaExpedienteEni", action = "urn:altaExpedienteEni")
	@WebResult(name = "ExpedienteInsideInfo", partName = "ExpedienteInsideInfo",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/expediente-e/expedienteInfo")
	public ExpedienteInsideInfo altaExpedienteEni(
			@WebParam(name = "credential",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/types/credential") @XmlElement(required = true, name = "credential") WSCredentialInside info,
			@WebParam(name = "expediente") @XmlElement(required = true, name = "expediente") TipoExpediente expediente,
			@WebParam(name = "metadatosAdicionales") @XmlElement(required = false, name = "metadatosAdicionales") TipoMetadatosAdicionales metadatosAdicionales)
			throws InsideWSException;
	
	@WebMethod(operationName = "cambiaMetadatosExpediente", action = "urn:cambiaMetadatosExpediente")
	@WebResult(name = "ExpedienteInsideInfo", partName = "ExpedienteInsideInfo",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/expediente-e/expedienteInfo")
	public ExpedienteInsideInfo cambiaMetadatosExpediente(
			@WebParam(name = "credential",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/types/credential") @XmlElement(required = true, name = "credential") WSCredentialInside info,
			@WebParam(name = "identificador") @XmlElement(required = true, name = "identificador") String identificador,
			@WebParam(name = "metadatosEni") @XmlElement(required = true, name = "metadatosEni") TipoMetadatos metadatosEni,
			@WebParam(name = "metadatosAdicionales") @XmlElement(required = false, name = "metadatosAdicionales") TipoMetadatosAdicionales metadatosAdicionales)
			throws InsideWSException;
	
	@WebMethod(operationName = "modificaExpedienteEni", action = "urn:modificaExpedienteEni")
	@WebResult(name = "ExpedienteInsideInfo", partName = "ExpedienteInsideInfo",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/expediente-e/expedienteInfo")
	public ExpedienteInsideInfo modificaExpedienteEni(
			@WebParam(name = "credential",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/types/credential") @XmlElement(required = true, name = "credential") WSCredentialInside info,
			@WebParam(name = "expediente") @XmlElement(required = true, name = "expediente") TipoExpediente expediente,
			@WebParam(name = "metadatosAdicionales") @XmlElement(required = false, name = "metadatosAdicionales") TipoMetadatosAdicionales metadatosAdicionales)
			throws InsideWSException;
	
	@WebMethod(operationName = "altaCarpetaEnExpediente", action = "urn:altaCarpetaEnExpediente")
	@WebResult(name = "ExpedienteInsideInfo", partName = "ExpedienteInsideInfo",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/expediente-e/expedienteInfo")
	public ExpedienteInsideInfo altaCarpetaEnExpediente(
			@WebParam(name = "credential",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/types/credential") @XmlElement(required = true, name = "credential") WSCredentialInside info,
			@WebParam(name = "identificadorExpediente") @XmlElement(required = true, name = "identificadorExpediente") String identificadorExpediente,
			@WebParam(name = "identificadorCarpeta") @XmlElement(required = true, name = "identificadorCarpeta") String identificadorCarpeta,
			@WebParam(name = "ruta") @XmlElement(required = false, name = "ruta") String ruta)
			throws InsideWSException;
	
	@WebMethod(operationName = "borrarCarpetaEnExpediente", action = "urn:borrarCarpetaEnExpediente")
	@WebResult(name = "ExpedienteInsideInfo", partName = "ExpedienteInsideInfo",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/expediente-e/expedienteInfo")
	public ExpedienteInsideInfo borrarCarpetaEnExpediente(
			@WebParam(name = "credential",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/types/credential") @XmlElement(required = true, name = "credential") WSCredentialInside info,
			@WebParam(name = "identificadorExpediente") @XmlElement(required = true, name = "identificadorExpediente") String identificadorExpediente,
			@WebParam(name = "identificadorCarpeta") @XmlElement(required = true, name = "identificadorCarpeta") String identificadorCarpeta,
			@WebParam(name = "ruta") @XmlElement(required = false, name = "ruta") String ruta)
			throws InsideWSException;
	
	
	@WebMethod(operationName = "cambiaEstadoExpediente", action = "urn:cambiaEstadoExpediente")
	@WebResult(name = "ExpedienteInsideInfo", partName = "ExpedienteInsideInfo",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/expediente-e/expedienteInfo")
	public ExpedienteInsideInfo cambiaEstadoExpediente(
			@WebParam(name = "credential",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/types/credential") @XmlElement(required = true, name = "credential") WSCredentialInside info,
			@WebParam(name = "identificador") @XmlElement(required = true, name = "identificador") String identificador,
			@WebParam(name = "estado") @XmlElement(required = false, name = "estado") EnumeracionEstados estado)
			throws InsideWSException;
	
	@WebMethod(operationName = "importarExpediente", action = "urn:importarExpediente")
	@WebResult(name = "ExpedienteInsideInfo", partName = "ExpedienteInsideInfo",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/expediente-e/expedienteInfo")
	public ExpedienteInsideInfo importarExpediente(
			@WebParam(name = "credential",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/types/credential") @XmlElement(required = true, name = "credential") WSCredentialInside info,
			@WebParam(name = "identificador") @XmlElement(required = true, name = "identificador") String identificador,
			@WebParam(name = "identificadorImportado") @XmlElement(required = true, name = "identificadorImportado") String identificadorImportado,
			@WebParam(name = "ruta") @XmlElement(required = true, name = "ruta") String ruta)
			throws InsideWSException;
	
	@WebMethod(operationName = "vincularExpediente", action = "urn:vincularExpediente")
	@WebResult(name = "ExpedienteInsideInfo", partName = "ExpedienteInsideInfo",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/expediente-e/expedienteInfo")
	public ExpedienteInsideInfo vincularExpediente(
			@WebParam(name = "credential",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/types/credential") @XmlElement(required = true, name = "credential") WSCredentialInside info,
			@WebParam(name = "identificador") @XmlElement(required = true, name = "identificador") String identificador,
			@WebParam(name = "identificadorVinculado") @XmlElement(required = true, name = "identificadorVinculado") String identificadorVinculado,
			@WebParam(name = "ruta") @XmlElement(required = true, name = "ruta") String ruta)
			throws InsideWSException;
	
	@WebMethod(operationName = "crearVistaAbiertaExpediente", action = "urn:crearVistaAbiertaExpediente")
	@WebResult(name = "ExpedienteInsideInfo", partName = "ExpedienteInsideInfo",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/expediente-e/expedienteInfo")
	public ExpedienteInsideInfo crearVistaAbiertaExpediente(
			@WebParam(name = "credential",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/types/credential") @XmlElement(required = true, name = "credential") WSCredentialInside info,
			@WebParam(name = "identificador") @XmlElement(required = true, name = "identificador") String identificador)
			throws InsideWSException;
	
	@WebMethod(operationName = "crearVistaCerradaExpediente", action = "urn:crearVistaCerradaExpediente")
	@WebResult(name = "ExpedienteInsideInfo", partName = "ExpedienteInsideInfo",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/expediente-e/expedienteInfo")
	public ExpedienteInsideInfo crearVistaCerradaExpediente(
			@WebParam(name = "credential",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/types/credential") @XmlElement(required = true, name = "credential") WSCredentialInside info,
			@WebParam(name = "identificador") @XmlElement(required = true, name = "identificador") String identificador)
			throws InsideWSException;
	
	
	@WebMethod(operationName = "vincularDocumentosEnExpediente", action = "urn:vincularDocumentosEnExpediente")
	@WebResult(name = "ExpedienteInsideInfo", partName = "ExpedienteInsideInfo",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/expediente-e/expedienteInfo")
	public ExpedienteInsideInfo vincularDocumentosEnExpediente(
			@WebParam(name = "credential",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/types/credential") @XmlElement(required = true, name = "credential") WSCredentialInside info,
			@WebParam(name = "identificador") @XmlElement(required = true, name = "identificador") String identificador,
			@WebParam(name = "identificadorDocumento") @XmlElement(required = true, name = "identificadorDocumento") List<String> identificadoresDocumentos,
			@WebParam(name = "ruta") @XmlElement(required = true, name = "ruta") String ruta)
			throws InsideWSException;
	
	@WebMethod(operationName = "getExpediente", action = "urn:getExpediente")
	@WebResult(name = "expediente", partName = "expediente",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/expediente-e")
	public TipoExpedienteInside getExpediente(
			@WebParam(name = "credential",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/types/credential") @XmlElement(required = true, name = "credential") WSCredentialInside info,
			@WebParam(name = "identificador") @XmlElement(required = true, name = "identificador") String identificador,
			@WebParam(name = "version") @XmlElement(name = "version") Integer version)
			throws InsideWSException;
	
	@WebMethod(operationName = "getExpedienteEni", action = "urn:getExpedienteEni")
	@WebResult(name = "expediente", partName = "expediente",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/expediente-e")
	public TipoExpediente getExpedienteEni(
			@WebParam(name = "credential",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/types/credential") @XmlElement(required = true, name = "credential") WSCredentialInside info,
			@WebParam(name = "identificador") @XmlElement(required = true, name = "identificador") String identificador)
			throws InsideWSException;
	
	@WebMethod(operationName = "getFicheroFoliado", action = "urn:getFicheroFoliado")
	@WebResult(name = "ficheroFoliado", partName = "ficheroFoliado")
	public TipoContenido getFicheroFoliado(
			@WebParam(name = "credential",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/types/credential") @XmlElement(required = true, name = "credential") WSCredentialInside info,
			@WebParam(name = "identificador") @XmlElement(required = true, name = "identificador") String identificador)
			throws InsideWSException;
	
	@WebMethod(operationName = "getIndiceExpediente", action = "urn:getIndiceExpediente")
	@WebResult(name = "indice", partName = "indice",targetNamespace="http://administracionelectronica.gob.es/ENI/XSD/v1.0/expediente-e/indice-e")
	public TipoIndice getIndiceExpediente(
			@WebParam(name = "credential",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/types/credential") @XmlElement(required = true, name = "credential") WSCredentialInside info,
			@WebParam(name = "identificador") @XmlElement(required = true, name = "identificador") String identificador)
			throws InsideWSException;
	
	@WebMethod(operationName = "getExpedienteMetadatos", action = "urn:getExpedienteMetadatos")
	@WebResult(name = "metadatos", partName = "metadatos",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/expediente-e/metadatos")
	public ExpedienteInsideMetadatos getExpedienteMetadatos(
			@WebParam(name = "credential",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/types/credential") @XmlElement(required = true, name = "credential") WSCredentialInside info,
			@WebParam(name = "identificador") @XmlElement(required = true, name = "identificador") String identificador,
			@WebParam(name = "version") @XmlElement(name = "version") Integer version)
			throws InsideWSException;
	
	@WebMethod(operationName = "getVersionesExpediente", action = "urn:getVersionesExpediente")
	@WebResult(name = "versiones", partName = "versiones",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/version")
	public List<TipoVersionInside> getVersionesExpediente(
			@WebParam(name = "credential",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/types/credential") @XmlElement(required = true, name = "credential") WSCredentialInside info,
			@WebParam(name = "identificador") @XmlElement(required = true, name = "identificador") String identificador)
			throws InsideWSException;
	
	@WebMethod(operationName = "altaDocumentoOriginal", action = "urn:altaDocumentoOriginal")
	@WebResult(name = "DocumentoInsideInfo", partName = "DocumentoInsideInfo",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/documento-e/documentoInfo")
	public DocumentoInsideInfo altaDocumentoOriginal(
			@WebParam(name = "credential",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/types/credential") @XmlElement(required = true, name = "credential") WSCredentialInside info,
			@WebParam(name = "documento") @XmlElement(required = true, name = "documento") TipoDocumentoAltaInside documento)
			throws InsideWSException;
	
	@WebMethod(operationName = "altaDocumentoDigitalizado", action = "urn:altaDocumentoDigitalizado")
	@WebResult(name = "DocumentoInsideInfo", partName = "DocumentoInsideInfo",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/documento-e/documentoInfo")
	public DocumentoInsideInfo altaDocumentoDigitalizado(
			@WebParam(name = "credential",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/types/credential") @XmlElement(required = true, name = "credential") WSCredentialInside info,
			@WebParam(name = "documento") @XmlElement(required = true, name = "documento") TipoDocumentoAltaInside documento)
			throws InsideWSException;
	
	@WebMethod(operationName = "altaDocumentoCopiaParcial", action = "urn:altaDocumentoCopiaParcial")
	@WebResult(name = "DocumentoInsideInfo", partName = "DocumentoInsideInfo",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/documento-e/documentoInfo")
	public DocumentoInsideInfo altaDocumentoCopiaParcial(
			@WebParam(name = "credential",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/types/credential") @XmlElement(required = true, name = "credential") WSCredentialInside info,
			@WebParam(name = "documento") @XmlElement(required = true, name = "documento") TipoDocumentoAltaInside documento,
			@WebParam(name = "identificadorDocumentoOrigen") @XmlElement(required = true, name = "identificadorDocumentoOrigen") String identificadorDocumentoOrigen )
			throws InsideWSException;
	
	@WebMethod(operationName = "altaDocumentoCambioFormato", action = "urn:altaDocumentoCambioFormato")
	@WebResult(name = "DocumentoInsideInfo", partName = "DocumentoInsideInfo",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/documento-e/documentoInfo")
	public DocumentoInsideInfo altaDocumentoCambioFormato(
			@WebParam(name = "credential",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/types/credential") @XmlElement(required = true, name = "credential") WSCredentialInside info,
			@WebParam(name = "documento") @XmlElement(required = true, name = "documento") TipoDocumentoAltaInside documento,
			@WebParam(name = "identificadorDocumentoOrigen") @XmlElement(required = true, name = "identificadorDocumentoOrigen") String identificadorDocumentoOrigen )
			throws InsideWSException;
	
	@WebMethod(operationName = "altaDocumentoEni", action = "urn:altaDocumentoEni")
	@WebResult(name = "DocumentoInsideInfo", partName = "DocumentoInsideInfo",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/documento-e/documentoInfo")
	public DocumentoInsideInfo altaDocumentoEni(
			@WebParam(name = "credential",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/types/credential") @XmlElement(required = true, name = "credential") WSCredentialInside info,
			@WebParam(name = "documento") @XmlElement(required = true, name = "documento") TipoDocumento documento,
			@WebParam(name = "asociadoaExpediente") @XmlElement(required = false, name = "asociadoaExpediente") TipoDocumentoAsociadoaExpediente asociadoaExpediente,
			@WebParam(name = "metadatosAdicionales") @XmlElement(required = false, name = "metadatosAdicionales") TipoMetadatosAdicionales metadatosAdicionales,
			@WebParam(name = "firmaServidor") @XmlElement(required = false, name = "firmaServidor") boolean firmaServidor)
			throws InsideWSException;
	
	@WebMethod(operationName = "modificaDocumento", action = "urn:modificaDocumento")
	@WebResult(name = "DocumentoInsideInfo", partName = "DocumentoInsideInfo",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/documento-e/documentoInfo")
	public DocumentoInsideInfo modificaDocumento(
			@WebParam(name = "credential",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/types/credential") @XmlElement(required = true, name = "credential") WSCredentialInside info,
			@WebParam(name = "documento") @XmlElement(required = true, name = "documento") TipoDocumento documento,
			@WebParam(name = "metadatosAdicionales") @XmlElement(required = true, name = "metadatosAdicionales") TipoMetadatosAdicionales metadatosAdicionales) 
			throws InsideWSException;
	
	@WebMethod(operationName = "getDocumento", action = "urn:getDocumento")
	@WebResult(name = "documento", partName = "documento",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/documento-e")
	public TipoDocumentoInside getDocumento(
			@WebParam(name = "credential",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/types/credential") @XmlElement(required = true, name = "credential") WSCredentialInside info,
			@WebParam(name = "identificador") @XmlElement(required = true, name = "identificador") String identificador,
			@WebParam(name = "version") @XmlElement( name = "version") Integer version)
			throws InsideWSException;
	
	@WebMethod(operationName = "getDocumentoEni", action = "urn:getDocumentoEni")
	@WebResult(name = "documento", partName = "documento",targetNamespace="http://administracionelectronica.gob.es/ENI/XSD/v1.0/documento-e")
	public TipoDocumento getDocumentoEni(
			@WebParam(name = "credential",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/types/credential") @XmlElement(required = true, name = "credential") WSCredentialInside info,
			@WebParam(name = "identificador") @XmlElement(required = true, name = "identificador") String identificador)
			throws InsideWSException;
	
	@WebMethod(operationName = "getDocumentoContenido", action = "urn:getDocumentoContenido")
	@WebResult(name = "documento", partName = "documento",targetNamespace="http://administracionelectronica.gob.es/ENI/XSD/v1.0/documento-e/contenido")
	public TipoContenido getDocumentoContenido(
			@WebParam(name = "credential",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/types/credential") @XmlElement(required = true, name = "credential") WSCredentialInside info,
			@WebParam(name = "identificador") @XmlElement(required = true, name = "identificador") String identificador,
			@WebParam(name = "version") @XmlElement(name = "version") Integer version)
			throws InsideWSException;
	
	@WebMethod(operationName = "getMetadatosDocumento", action = "urn:getMetadatosDocumento")
	@WebResult(name = "metadatos", partName = "metadatos",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/documento-e/metadatos")
	public DocumentoInsideMetadatos getMetadatosDocumento(
			@WebParam(name = "credential",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/types/credential") @XmlElement(required = true, name = "credential") WSCredentialInside info,
			@WebParam(name = "identificador") @XmlElement(required = true, name = "identificador") String identificador,
			@WebParam(name = "version") @XmlElement(name = "version") Integer version)
			throws InsideWSException;

	@WebMethod(operationName = "getVersionesDocumento", action = "urn:getVersionesDocumento")
	@WebResult(name = "versiones", partName = "versiones",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/version")
	public List<TipoVersionInside> getVersionesDocumento(
			@WebParam(name = "credential",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/types/credential") @XmlElement(required = true, name = "credential") WSCredentialInside info,
			@WebParam(name = "identificador") @XmlElement(required = true, name = "identificador") String identificador)
			throws InsideWSException;
	
	@WebMethod(operationName = "buscarExpedientes", action = "urn:buscarExpedientes")
	@WebResult(name = "resultados", partName = "resultados",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/busqueda/resultados")
	public ExpedienteResultadoBusqueda buscarExpedientes(
			@WebParam(name = "credential",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/types/credential") @XmlElement(required = true, name = "credential") WSCredentialInside info,
			@WebParam(name = "consulta") @XmlElement(required = true, name = "consulta") ConsultaWsInside consulta,
			@WebParam(name = "limite") @XmlElement(required = false, name = "limite") int limite,
			@WebParam(name = "pagina") @XmlElement(required = false, name = "pagina") int pagina)
			throws InsideWSException;
	
	@WebMethod(operationName = "buscarDocumentos", action = "urn:buscarDocumentos")
	@WebResult(name = "resultados", partName = "resultados",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/busqueda/resultados")
	public DocumentoResultadoBusqueda buscarDocumentos(
			@WebParam(name = "credential",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/types/credential") @XmlElement(required = true, name = "credential") WSCredentialInside info,
			@WebParam(name = "consulta") @XmlElement(required = true, name = "consulta") ConsultaWsInside consulta,
			@WebParam(name = "limite") @XmlElement(required = false, name = "limite") int limite,
			@WebParam(name = "pagina") @XmlElement(required = false, name = "pagina") int pagina)
			throws InsideWSException;
	
	@WebMethod(operationName = "convertirDocumentoAEni", action = "urn:convertirDocumentoAEni")
	@WebResult(name = "documento", partName = "documento",targetNamespace="http://administracionelectronica.gob.es/ENI/XSD/v1.0/documento-e")
	public TipoDocumento convertirDocumentoAEni(
			@WebParam(name = "credential",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/types/credential") @XmlElement(required = true, name = "credential") WSCredentialInside info,
			@WebParam(name = "documento") @XmlElement(required = true, name = "documento") TipoDocumentoConversionInside documento)
			throws InsideWSException;
	
	@WebMethod(operationName = "convertirExpedienteAEni", action = "urn:convertirExpedienteAEni")
	@WebResult(name = "expediente", partName = "expediente",targetNamespace="http://administracionelectronica.gob.es/ENI/XSD/v1.0/expediente-e")
	public TipoExpediente convertirExpedienteAEni (
			@WebParam(name = "credential",targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/types/credential") @XmlElement(required = true, name = "credential") WSCredentialInside info,
			@WebParam(name = "expediente") @XmlElement(required = true, name = "expediente") TipoExpedienteConversionInside expediente) throws InsideWSException;
	


}
