/* Copyright (C) 2012-13 MINHAP, Gobierno de España
   This program is licensed and may be used, modified and redistributed under the terms
   of the European Public License (EUPL), either version 1.1 or (at your
   option) any later version as soon as they are approved by the European Commission.
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
   or implied. See the License for the specific language governing permissions and
   more details.
   You should have received a copy of the EUPL1.1 license
   along with this program; if not, you may find it at
   http://joinup.ec.europa.eu/software/page/eupl/licence-eupl */

package es.mpt.dsic.inside.ws.service.impl;


import java.io.FileOutputStream;
import java.util.List;

import javax.jws.WebService;


import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;


import es.mpt.dsic.infofirma.service.InfoFirmaService;
import es.mpt.dsic.inside.model.converter.InsideConverterDocumento;
import es.mpt.dsic.inside.model.converter.InsideConverterExpediente;
import es.mpt.dsic.inside.model.converter.InsideConverterMetadatos;
import es.mpt.dsic.inside.model.converter.InsideConverterValidacion;
import es.mpt.dsic.inside.model.objetos.documento.ObjetoDocumentoInside;
import es.mpt.dsic.inside.model.objetos.enivalidation.ResultadoValidacionDocumento;
import es.mpt.dsic.inside.model.objetos.enivalidation.ResultadoValidacionExpediente;
import es.mpt.dsic.inside.model.objetos.expediente.ObjetoExpedienteInside;
import es.mpt.dsic.inside.service.visualizacion.ResultadoVisualizacionDocumento;
import es.mpt.dsic.inside.ws.exception.InsideExceptionConverter;
import es.mpt.dsic.inside.ws.exception.InsideWSException;
import es.mpt.dsic.inside.ws.service.GInSideWebService;
import es.mpt.dsic.inside.ws.validation.InsideValidationDataService;
import es.mpt.dsic.inside.xml.eni.documento.TipoDocumento;
import es.mpt.dsic.inside.xml.eni.expediente.TipoExpediente;
import es.mpt.dsic.inside.xml.inside.TipoDocumentoInsideConMAdicionales;
import es.mpt.dsic.inside.xml.inside.TipoExpedienteInsideConMAdicionales;
import es.mpt.dsic.inside.xml.inside.TipoMetadatosAdicionales;
import es.mpt.dsic.inside.xml.inside.ws.credential.WSCredentialInside;
import es.mpt.dsic.inside.xml.inside.ws.documento.conversion.TipoDocumentoConversionInside;
import es.mpt.dsic.inside.xml.inside.ws.expediente.conversion.TipoExpedienteConversionInside;
import es.mpt.dsic.inside.xml.inside.ws.validacion.documento.TipoDocumentoValidacionInside;
import es.mpt.dsic.inside.xml.inside.ws.validacion.documento.resultados.TipoResultadoValidacionDocumentoInside;
import es.mpt.dsic.inside.xml.inside.ws.validacion.expediente.TipoExpedienteValidacionInside;
import es.mpt.dsic.inside.xml.inside.ws.validacion.expediente.resultados.TipoResultadoValidacionExpedienteInside;
import es.mpt.dsic.inside.xml.inside.ws.visualizacion.documento.TipoDocumentoVisualizacionInside;
import es.mpt.dsic.inside.xml.inside.ws.visualizacion.documento.TipoResultadoVisualizacionDocumentoInside;

@WebService(endpointInterface="es.mpt.dsic.inside.ws.service.GInSideWebService",
		targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService",
		portName="GInsideWSPort",
		serviceName="GInsideWS")

public class GInSideWebServiceImpl implements GInSideWebService{
	
	protected static final Log logger = LogFactory.getLog(GInSideWebServiceImpl.class);
	
	protected static final String VERSION_NTI = "http://administracionelectronica.gob.es/ENI/XSD/v1.0/expediente-e"; 

	@Autowired
	private es.mpt.dsic.inside.service.InSideService service;

	@Autowired
	private InfoFirmaService infofirmaService;
	
	@Autowired
	private InsideValidationDataService validationService;
	


	@Override
	public TipoDocumento convertirDocumentoAEni(WSCredentialInside info,TipoDocumentoConversionInside documentoConversion) 
		throws InsideWSException 
	{	
		try {
			logger.debug("Validando documentoConversión a nivel de WS");
			 
			documentoConversion = validationService.validaTipoDocumentoConversionInside(documentoConversion, true);
			
			logger.debug("Convirtiendo el documentoConversion a ENI");
			ObjetoDocumentoInside documentoInside = InsideConverterDocumento.documentoConversionToInside(documentoConversion, infofirmaService);
			
			logger.debug("Validando documento Inside a nivel de Inside");
			documentoInside = service.validarDocumento(documentoInside);
			
			TipoDocumento documentoEni = InsideConverterDocumento.documentoInsideToEni(documentoInside);
			
			logger.info("Convertido Documento de entrada a Eni " + documentoInside.getIdentificador());
			return documentoEni;
			
		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
		
	}
	
	@Override
	public TipoDocumentoInsideConMAdicionales convertirDocumentoAEniConMAdicionales(WSCredentialInside info, TipoDocumentoConversionInside documentoConversion, TipoMetadatosAdicionales metadatosAdicionales)
		throws InsideWSException {
		
		try {
			logger.debug("Validando documentoConversión a nivel de WS");
			 
			documentoConversion = validationService.validaTipoDocumentoConversionInside(documentoConversion, true);
						
			logger.debug("Convirtiendo el documentoConversion a ENI");
			ObjetoDocumentoInside documentoInside = InsideConverterDocumento.documentoConversionToInside(documentoConversion, infofirmaService);
			documentoInside.getMetadatos().setMetadatosAdicionales(InsideConverterMetadatos.metadatosAdicionalesXmlToInside(metadatosAdicionales));
			
			logger.debug("Validando documento Inside a nivel de Inside");
			documentoInside = service.validarDocumento(documentoInside);
			
			TipoDocumentoInsideConMAdicionales documentoConMAdicionales = InsideConverterDocumento.documentoInsideToConMAdicionales(documentoInside);
			
			logger.info("Convertido Documento de entrada a TipoDocumentoInsideConMAdicionales " + documentoInside.getIdentificador());
			return documentoConMAdicionales;
			
		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
		
	}
		

	@Override
	public TipoExpediente convertirExpedienteAEni (WSCredentialInside info, TipoExpedienteConversionInside expedienteConversion)	throws InsideWSException 
	{
		try  {
			logger.debug("Validando expedienteConversion a nivel de WS");
			expedienteConversion = validationService.validaTipoExpedienteConversionInside(expedienteConversion, true);
			
			logger.debug("Convirtiendo expedienteConversion a Inside");
			ObjetoExpedienteInside expedienteInside = InsideConverterExpediente.expedienteConversionToInside (expedienteConversion);
			
			logger.debug("Validando expediente Inside");
			expedienteInside = service.validarExpediente(expedienteInside);
			
			logger.debug("Obteniendo Visualización del Índice");
			expedienteInside = service.obtenerVisualizacionIndice(expedienteInside, 
																  expedienteConversion.getOpcionesVisualizacion().isEstamparImagen(),
																  expedienteConversion.getOpcionesVisualizacion().isEstamparNombreOrganismo(),
										                          expedienteConversion.getOpcionesVisualizacion().getFilasNombreOrganismo() != null ? expedienteConversion.getOpcionesVisualizacion().getFilasNombreOrganismo().getFila() : null,
										                          true);
						
			
			logger.debug("Convirtiendo expediente de Inside a ENI");
			
			return InsideConverterExpediente.expedienteInsideToEni(expedienteInside);
			
		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}
	
	@Override
	public TipoExpedienteInsideConMAdicionales convertirExpedienteAEniConMAdicionales (WSCredentialInside info, TipoExpedienteConversionInside expedienteConversion, TipoMetadatosAdicionales metadatosAdicionales)
	throws InsideWSException {
		try  {
			logger.debug("Validando expedienteConversion a nivel de WS");
			expedienteConversion = validationService.validaTipoExpedienteConversionInside(expedienteConversion, true);
			
			logger.debug("Convirtiendo expedienteConversion a Inside");
			ObjetoExpedienteInside expedienteInside = InsideConverterExpediente.expedienteConversionToInside (expedienteConversion);
			expedienteInside.getMetadatos().setMetadatosAdicionales(InsideConverterMetadatos.metadatosAdicionalesXmlToInside(metadatosAdicionales));
			
			logger.debug("Validando expediente Inside");
			expedienteInside = service.validarExpediente(expedienteInside);
			
			logger.debug("Obteniendo Visualización del Índice");
			expedienteInside = service.obtenerVisualizacionIndice(expedienteInside, 
																  expedienteConversion.getOpcionesVisualizacion().isEstamparImagen(),
																  expedienteConversion.getOpcionesVisualizacion().isEstamparNombreOrganismo(),
										                          expedienteConversion.getOpcionesVisualizacion().getFilasNombreOrganismo() != null ? expedienteConversion.getOpcionesVisualizacion().getFilasNombreOrganismo().getFila() : null,
										                          true);
						
			
			logger.debug("Convertido Expediente Inside a TipoExpedienteInsideConMAdicionales");
			
			return InsideConverterExpediente.expedienteInsideToConMAdicionales(expedienteInside);
			
		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}
	
	@Override
	public TipoResultadoValidacionDocumentoInside validarDocumentoEniFile ( WSCredentialInside info, TipoDocumentoValidacionInside documentoValidacion) throws InsideWSException {
		try {
			logger.debug ("Inicio operación validarDocumentoEniFile");
			
			logger.debug ("Validando documentoValidacion a nivel de WS");
			documentoValidacion = validationService.validaTipoDocumentoValidacionInside(documentoValidacion, true);
			
			List<ResultadoValidacionDocumento> resultados = service.validaDocumentoEniFile(documentoValidacion.getContenido(), 
					InsideConverterValidacion.tipoOpcionesValidacionDocumentoInsideToListOpcionValidacionDocumento(documentoValidacion.getOpcionesValidacionDocumento()));
			
			logger.debug ("Fin operación validarDocumentoEniFile");
			
			return InsideConverterValidacion.listResultadoValidacionDocumentoToTipoResultadoValidacionDocumentoInside(resultados);
		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}
	
	@Override
	public TipoResultadoValidacionExpedienteInside validarExpedienteEniFile ( WSCredentialInside info, TipoExpedienteValidacionInside expedienteValidacion) throws InsideWSException {
		try {
			logger.debug ("Inicio operación validarExpedienteEniFile");
			
			logger.debug ("Validando expedienteValidacion a nivel de WS");
			expedienteValidacion = validationService.validaTipoExpedienteValidacionInside(expedienteValidacion, true);
			
			List<ResultadoValidacionExpediente> resultados = service.validaExpedienteEniFile(expedienteValidacion.getContenido(), 
					InsideConverterValidacion.tipoOpcionesValidacionExpedienteInsideToListOpcionValidacionExpediente(expedienteValidacion.getOpcionesValidacionExpediente()));
			
			logger.debug ("Fin operación validarExpedienteEniFile");
			
			//return InsideConverterValidacion.resultadoValidacionToTipoResultadoValidacionInside (resultado);
			return InsideConverterValidacion.listResultadoValidacionExpedienteToTipoResultadoValidacionExpedienteInside(resultados);
		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}
	
	@Override 
	public TipoResultadoVisualizacionDocumentoInside visualizarDocumentoEni (WSCredentialInside info, TipoDocumentoVisualizacionInside documentoVisualizacion) throws InsideWSException {
		
		TipoResultadoVisualizacionDocumentoInside resultado = new TipoResultadoVisualizacionDocumentoInside ();
		
		try {
			logger.debug ("Inicio operación visualizarDocumentoEni");
			
			logger.debug ("Validando documentoVisualizacion a nivel de WS");
			documentoVisualizacion = validationService.validaTipoDocumentoVisualizacionInside(documentoVisualizacion, true);
			
			logger.debug ("Convirtiendo a documento Inside");
			ObjetoDocumentoInside documentoInside = InsideConverterDocumento.documentoVisualizacionToInside(documentoVisualizacion);
			
			ResultadoVisualizacionDocumento vis = service.visualizaDocumentoInside(documentoInside, 
											 	  documentoVisualizacion.getOpcionesVisualizacionDocumento().isEstamparImagen(), 
											 	  documentoVisualizacion.getOpcionesVisualizacionDocumento().isEstamparNombreOrganismo(),
											 	  documentoVisualizacion.getOpcionesVisualizacionDocumento().getFilasNombreOrganismo() != null ? documentoVisualizacion.getOpcionesVisualizacionDocumento().getFilasNombreOrganismo().getFila() : null);
			
			resultado.setContenido(vis.getVisualizacion());
			resultado.setMime(vis.getMime());
			
			logger.debug ("Fin operación visualizarDocumentoEni");
			
			return resultado;
		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}
}
