/* Copyright (C) 2012-13 MINHAP, Gobierno de España
   This program is licensed and may be used, modified and redistributed under the terms
   of the European Public License (EUPL), either version 1.1 or (at your
   option) any later version as soon as they are approved by the European Commission.
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
   or implied. See the License for the specific language governing permissions and
   more details.
   You should have received a copy of the EUPL1.1 license
   along with this program; if not, you may find it at
   http://joinup.ec.europa.eu/software/page/eupl/licence-eupl */

package es.mpt.dsic.inside.ws.service.impl;

import java.util.List;

import javax.jws.WebService;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.annotation.Secured;
import org.springframework.util.StringUtils;

import es.mpt.dsic.infofirma.service.InfoFirmaService;
import es.mpt.dsic.inside.model.busqueda.resultado.ResultadoBusquedaInside;
import es.mpt.dsic.inside.model.converter.InsideConverterBusqueda;
import es.mpt.dsic.inside.model.converter.InsideConverterBusquedaResultado;
import es.mpt.dsic.inside.model.converter.InsideConverterDocumento;
import es.mpt.dsic.inside.model.converter.InsideConverterExpediente;
import es.mpt.dsic.inside.model.converter.InsideConverterExpedienteInfo;
import es.mpt.dsic.inside.model.converter.InsideConverterFoliado;
import es.mpt.dsic.inside.model.converter.InsideConverterIndice;
import es.mpt.dsic.inside.model.converter.InsideConverterMetadatos;
import es.mpt.dsic.inside.model.converter.InsideConverterVersion;
import es.mpt.dsic.inside.model.converter.exception.InsideConverterDocumentoEniToInsideException;
import es.mpt.dsic.inside.model.objetos.documento.ObjetoDocumentoInside;
import es.mpt.dsic.inside.model.objetos.documento.ObjetoDocumentoInsideContenido;
import es.mpt.dsic.inside.model.objetos.documento.metadatos.ObjetoDocumentoInsideMetadatos;
import es.mpt.dsic.inside.model.objetos.documento.metadatos.ObjetoDocumentoInsideMetadatosEstadoElaboracion;
import es.mpt.dsic.inside.model.objetos.documento.metadatos.ObjetoDocumentoInsideMetadatosEstadoElaboracionEnumeracion;
import es.mpt.dsic.inside.model.objetos.expediente.ObjetoExpedienteInside;
import es.mpt.dsic.inside.model.objetos.expediente.metadatos.ObjetoExpedienteInsideMetadatos;
import es.mpt.dsic.inside.model.objetos.expediente.metadatos.ObjetoExpedienteInsideMetadatosEnumeracionEstados;
import es.mpt.dsic.inside.model.objetos.foliado.FoliadoInside;
import es.mpt.dsic.inside.service.exception.InSideServiceException;
import es.mpt.dsic.inside.ws.exception.InsideExceptionConverter;
import es.mpt.dsic.inside.ws.exception.InsideWSException;
import es.mpt.dsic.inside.ws.exception.InsideWsErrors;
import es.mpt.dsic.inside.ws.service.InSideWebService;
import es.mpt.dsic.inside.ws.validation.InsideValidationDataService;
import es.mpt.dsic.inside.xml.eni.documento.TipoDocumento;
import es.mpt.dsic.inside.xml.eni.documento.contenido.TipoContenido;
import es.mpt.dsic.inside.xml.eni.expediente.TipoExpediente;
import es.mpt.dsic.inside.xml.eni.expediente.indice.TipoIndice;
import es.mpt.dsic.inside.xml.eni.expediente.metadatos.EnumeracionEstados;
import es.mpt.dsic.inside.xml.eni.expediente.metadatos.TipoMetadatos;
import es.mpt.dsic.inside.xml.inside.ws.busqueda.ConsultaWsInside;
import es.mpt.dsic.inside.xml.inside.ws.documento.DocumentoInsideInfo;
import es.mpt.dsic.inside.xml.inside.DocumentoInsideMetadatos;
import es.mpt.dsic.inside.xml.inside.ws.busqueda.DocumentoResultadoBusqueda;
import es.mpt.dsic.inside.xml.inside.ExpedienteInsideInfo;
import es.mpt.dsic.inside.xml.inside.ExpedienteInsideMetadatos;
import es.mpt.dsic.inside.xml.inside.ws.busqueda.ExpedienteResultadoBusqueda;
import es.mpt.dsic.inside.xml.inside.ws.credential.WSCredentialInside;
import es.mpt.dsic.inside.xml.inside.ws.documento.alta.TipoDocumentoAltaInside;
import es.mpt.dsic.inside.xml.inside.ws.documento.alta.TipoDocumentoAsociadoaExpediente;
import es.mpt.dsic.inside.xml.inside.ws.documento.conversion.TipoDocumentoConversionInside;
import es.mpt.dsic.inside.xml.inside.ws.expediente.conversion.TipoExpedienteConversionInside;
import es.mpt.dsic.inside.xml.inside.TipoDocumentoInside;
import es.mpt.dsic.inside.xml.inside.TipoExpedienteInside;
import es.mpt.dsic.inside.xml.inside.TipoMetadatosAdicionales;
import es.mpt.dsic.inside.xml.inside.TipoVersionInside;

@WebService(endpointInterface = "es.mpt.dsic.inside.ws.service.InSideWebService", targetNamespace = "https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService", portName = "InsideWSPort", serviceName = "InsideWS")
// @WebService(targetNamespace="https://ssweb.seap.minhap.es/Inside/XSD/v1.0/WebService/impl")
public class InSideWebServiceImpl implements InSideWebService {
	protected static final Log logger = LogFactory
			.getLog(InSideWebServiceImpl.class);

	protected static final String VERSION_NTI = "http://administracionelectronica.gob.es/ENI/XSD/v1.0/expediente-e";

	@Autowired
	private es.mpt.dsic.inside.service.InSideService service;

	@Autowired
	private InfoFirmaService infofirmaService;

	@Autowired
	private InsideValidationDataService validationService;

	/*
	 * private static es.mpt.dsic.inside.security.model.InSideUserDetails
	 * getUserDetails() {
	 * 
	 * es.mpt.dsic.inside.security.model.InSideUserDetails
	 * user=(es.mpt.dsic.inside
	 * .security.model.InSideUserDetails)SecurityContextHolder
	 * .getContext().getAuthentication().getPrincipal(); return user; }
	 */

	@Override
	@Secured("ROLE_ALTA_EXPEDIENTE")
	public ExpedienteInsideInfo altaExpediente(WSCredentialInside info,
			TipoMetadatos metadatosEni,
			TipoMetadatosAdicionales metadatosAdicionales)
			throws InsideWSException {
		try {
			logger.debug ("Inicio operacion altaExpediente de aplicacion " + info.getIdaplicacion());
			logger.debug("metadatosEni: " + InsideConverterMetadatos.tipoMetadatosToString(metadatosEni));
			logger.debug("metadatosAdicionales: " + InsideConverterMetadatos.tipoMetadatosAdicionalesToString(metadatosAdicionales));
			
			checkVersionNTI(metadatosEni);

			logger.debug("Convirtiendo los metadatos del Eni y los adicionales a Expediente Inside");
			ObjetoExpedienteInside expediente = InsideConverterExpediente
					.fromMetadatos(metadatosEni, metadatosAdicionales);
			
			logger.debug("ObjetoExpedienteInside obtenido de la conversion: " + InsideConverterExpediente.expedienteInsideToString(expediente));

			expediente = service.altaExpediente(expediente);
			
			logger.debug("ObjetoExpedienteInside obtenido del alta: " + InsideConverterExpediente.expedienteInsideToString(expediente));

			logger.debug("Convirtiendo el expediente Inside a InfoInside");
			ExpedienteInsideInfo expedienteInfo = InsideConverterExpedienteInfo
					.fromExpediente(expediente);
			
			logger.debug("expedienteInsideInfo obtenido de la conversion: " + InsideConverterExpedienteInfo.expedienteInsideInfoToString(expedienteInfo));
			logger.debug("Fin operacion altaExpediente de aplicacion " + info.getIdaplicacion());

			return expedienteInfo;

		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}

	@Override
	@Secured("ROLE_ALTA_EXPEDIENTE")
	public ExpedienteInsideInfo altaExpedienteEni(WSCredentialInside info,
			TipoExpediente expedienteEni,
			TipoMetadatosAdicionales metadatosAdicionales)
			throws InsideWSException {
		try {
			checkVersionNTI(expedienteEni.getMetadatosExp());

			logger.debug("Convirtiendo el expedienteEni a Inside");
			ObjetoExpedienteInside expediente = InsideConverterExpediente
					.expedienteEniToInside(expedienteEni, metadatosAdicionales);

			expediente = service.altaExpediente(expediente);

			logger.debug("Convirtiendo el expediente Inside a InfoInside");
			ExpedienteInsideInfo expedienteInfo = InsideConverterExpedienteInfo
					.fromExpediente(expediente);

			return expedienteInfo;

		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}

	@Override
	@Secured("ROLE_MODIFICA_EXPEDIENTE")
	public ExpedienteInsideInfo cambiaMetadatosExpediente(
			WSCredentialInside info, String identificador,
			TipoMetadatos metadatosEni,
			TipoMetadatosAdicionales metadatosAdicionales)
			throws InsideWSException {
		try {
			checkVersionNTI(metadatosEni);

			logger.debug("Convirtiendo los metadatosEni a Inside");
			ObjetoExpedienteInsideMetadatos metadatos = InsideConverterMetadatos.Expediente
					.metadatosXmlToMetadatosInside(metadatosEni,
							metadatosAdicionales);
			ObjetoExpedienteInside expediente = service
					.cambiaMetadatosExpediente(identificador, metadatos);

			logger.debug("Convirtiendo el expediente Inside a InfoInside");
			ExpedienteInsideInfo expedienteInfo = InsideConverterExpedienteInfo
					.fromExpediente(expediente);

			return expedienteInfo;

		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}

	@Override
	@Secured("ROLE_MODIFICA_EXPEDIENTE")
	public ExpedienteInsideInfo modificaExpedienteEni(WSCredentialInside info,
			TipoExpediente expedienteEni,
			TipoMetadatosAdicionales metadatosAdicionales)
			throws InsideWSException {
		try {
			checkVersionNTI(expedienteEni.getMetadatosExp());
			validaIdentificador(expedienteEni);

			logger.debug("Convirtiendo el expedienteEni a Inside");
			ObjetoExpedienteInside expediente = InsideConverterExpediente
					.expedienteEniToInside(expedienteEni, metadatosAdicionales);

			expediente = service.modificaExpedienteInside(expediente);

			logger.debug("Convirtiendo el expediente Inside a InfoInside");
			ExpedienteInsideInfo expedienteInfo = InsideConverterExpedienteInfo
					.fromExpediente(expediente);

			return expedienteInfo;

		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}

	@Override
	public ExpedienteInsideInfo altaCarpetaEnExpediente(
			WSCredentialInside info, String identificadorExpediente,
			String identificadorCarpeta, String ruta) throws InsideWSException {
		try {
			if (identificadorExpediente == null) {
				throw new InsideWSException(
						InsideWsErrors.IDENTIFICADOR_EXPEDIENTE_VACIO);
			}
			if (identificadorCarpeta == null) {
				throw new InsideWSException(
						InsideWsErrors.IDENTIFICADOR_CARPETA_VACIO);
			}

			if (ruta == null) {
				ruta = "";
			}

			logger.debug("llamando a alta de carpeta para expediente "
					+ identificadorExpediente
					+ " con identificador de carpeta " + identificadorCarpeta
					+ " en ruta " + ruta);
			ObjetoExpedienteInside expediente = service.altaCarpeta(
					identificadorExpediente, identificadorCarpeta, ruta);

			logger.debug("Convirtiendo el expediente Inside a InfoInside");
			ExpedienteInsideInfo expedienteInfo = InsideConverterExpedienteInfo
					.fromExpediente(expediente);

			return expedienteInfo;

		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}

	@Override
	public ExpedienteInsideInfo borrarCarpetaEnExpediente(
			WSCredentialInside info, String identificadorExpediente,
			String identificadorCarpeta, String ruta) throws InsideWSException {
		try {
			if (identificadorExpediente == null) {
				throw new InsideWSException(
						InsideWsErrors.IDENTIFICADOR_EXPEDIENTE_VACIO);
			}
			if (identificadorCarpeta == null) {
				throw new InsideWSException(
						InsideWsErrors.IDENTIFICADOR_CARPETA_VACIO);
			}

			if (ruta == null) {
				ruta = "";
			}

			logger.debug("llamando a borrar carpeta para expediente "
					+ identificadorExpediente
					+ " con identificador de carpeta " + identificadorCarpeta
					+ " en ruta " + ruta);
			ObjetoExpedienteInside expediente = service.borrarCarpeta(
					identificadorExpediente, identificadorCarpeta, ruta);

			logger.debug("Convirtiendo el expediente Inside a InfoInside");
			ExpedienteInsideInfo expedienteInfo = InsideConverterExpedienteInfo
					.fromExpediente(expediente);

			return expedienteInfo;

		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}

	@Override
	@Secured("ROLE_MODIFICA_EXPEDIENTE")
	public ExpedienteInsideInfo cambiaEstadoExpediente(WSCredentialInside info,
			String identificador, EnumeracionEstados estado)
			throws InsideWSException {
		try {
			if (estado == null) {
				throw new InsideWSException(InsideWsErrors.ESTADO_VACIO);
			}

			ObjetoExpedienteInsideMetadatosEnumeracionEstados estadoInside = ObjetoExpedienteInsideMetadatosEnumeracionEstados
					.fromValue(estado.value());

			ObjetoExpedienteInside expediente = service.setEstadoExpediente(
					identificador, estadoInside);

			ExpedienteInsideInfo expedienteInfo = InsideConverterExpedienteInfo
					.fromExpediente(expediente);

			logger.info("Cambiado estado del Expediente con identificador "
					+ identificador + " a " + estado.value());
			return expedienteInfo;

		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}

	@Override
	@Secured("ROLE_MODIFICA_EXPEDIENTE")
	public ExpedienteInsideInfo importarExpediente(WSCredentialInside info,
			String identificador, String identificadorImportado, String ruta)
			throws InsideWSException {
		try {

			ObjetoExpedienteInside expediente = service.importarExpediente(
					identificador, identificadorImportado, ruta);

			ExpedienteInsideInfo expedienteInfo = InsideConverterExpedienteInfo
					.fromExpediente(expediente);

			logger.info("importado Expediente con identificador "
					+ identificadorImportado + " a expediente" + identificador);
			return expedienteInfo;

		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}

	@Override
	@Secured("ROLE_MODIFICA_EXPEDIENTE")
	public ExpedienteInsideInfo vincularExpediente(WSCredentialInside info,
			String identificador, String identificadorVinculado, String ruta)
			throws InsideWSException {
		try {

			ObjetoExpedienteInside expediente = service.vincularExpediente(
					identificador, identificadorVinculado, ruta);

			ExpedienteInsideInfo expedienteInfo = InsideConverterExpedienteInfo
					.fromExpediente(expediente);

			logger.info("Vinculado Expediente con identificador "
					+ identificadorVinculado + " a expediente" + identificador);
			return expedienteInfo;

		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}

	@Override
	@Secured("ROLE_MODIFICA_EXPEDIENTE")
	public ExpedienteInsideInfo crearVistaAbiertaExpediente(
			WSCredentialInside info, String identificador)
			throws InsideWSException {
		try {

			ObjetoExpedienteInside expediente = service
					.crearVistaAbiertaExpediente(identificador);

			ExpedienteInsideInfo expedienteInfo = InsideConverterExpedienteInfo
					.fromExpediente(expediente);

			logger.info("Se ha creado una vista abierta del Expediente "
					+ identificador
					+ ". El identificador del nuevo expediente es: "
					+ expedienteInfo.getIdentificador());
			return expedienteInfo;

		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}

	@Override
	@Secured("ROLE_MODIFICA_EXPEDIENTE")
	public ExpedienteInsideInfo crearVistaCerradaExpediente(
			WSCredentialInside info, String identificador)
			throws InsideWSException {
		try {

			ObjetoExpedienteInside expediente = service
					.crearVistaCerradaExpediente(identificador);

			ExpedienteInsideInfo expedienteInfo = InsideConverterExpedienteInfo
					.fromExpediente(expediente);

			logger.info("Se ha creado una vista cerrada del Expediente "
					+ identificador
					+ ". El identificador del nuevo expediente es: "
					+ expedienteInfo.getIdentificador());
			return expedienteInfo;

		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}

	@Override
	@Secured("ROLE_MODIFICA_EXPEDIENTE")
	public ExpedienteInsideInfo vincularDocumentosEnExpediente(
			WSCredentialInside info, String identificador,
			List<String> identificadoresDocumentos, String ruta)
			throws InsideWSException {
		try {

			ObjetoExpedienteInside expediente = service
					.vincularDocumentosEnExpediente(identificador,
							identificadoresDocumentos, ruta);

			ExpedienteInsideInfo expedienteInfo = InsideConverterExpedienteInfo
					.fromExpediente(expediente);

			logger.info("importados Documentos " + identificadoresDocumentos
					+ " a expediente" + identificador + " en ruta " + ruta);
			return expedienteInfo;

		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}

	@Override
	@Secured("ROLE_LEER_EXPEDIENTE")
	public TipoExpedienteInside getExpediente(WSCredentialInside info,
			String identificador, Integer version) throws InsideWSException {
		try {

			ObjetoExpedienteInside expedienteInside = service.getExpediente(
					identificador, version);

			TipoExpedienteInside expedienteInsideXml = InsideConverterExpediente
					.expedienteInsideToXml(expedienteInside);

			return expedienteInsideXml;

		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}

	@Override
	@Secured("ROLE_LEER_EXPEDIENTE")
	public TipoExpediente getExpedienteEni(WSCredentialInside info,
			String identificador) throws InsideWSException {
		try {

			ObjetoExpedienteInside expediente = service
					.getExpediente(identificador);

			TipoExpediente expedienteEni = InsideConverterExpediente
					.expedienteInsideToEni(expediente);
			logger.info("Devuelto Expediente ENI con identificador "
					+ identificador);
			return expedienteEni;

		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}

	@Override
	@Secured("ROLE_LEER_EXPEDIENTE")
	public TipoIndice getIndiceExpediente(WSCredentialInside info,
			String identificador) throws InsideWSException {
		try {

			ObjetoExpedienteInside expedienteInside = service
					.getExpediente(identificador);

			logger.info("Devuelto índice de Expediente ENI con identificador "
					+ identificador);

			return InsideConverterIndice.indiceInsideToEni(
					expedienteInside.getIndice(), identificador);

		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}

	@Override
	@Secured("ROLE_LEER_EXPEDIENTE")
	public ExpedienteInsideMetadatos getExpedienteMetadatos(
			WSCredentialInside info, String identificador, Integer version)
			throws InsideWSException {
		try {

			ObjetoExpedienteInsideMetadatos metadatos = service
					.getMetadatosExpediente(identificador, version);

			logger.info("Devuelto Metadatos del Expediente con identificador "
					+ identificador);
			return InsideConverterMetadatos.Expediente
					.metadatosInsideToXml(metadatos);

		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}

	@Override
	@Secured("ROLE_LEER_EXPEDIENTE")
	public List<TipoVersionInside> getVersionesExpediente(
			WSCredentialInside info, String identificador)
			throws InsideWSException {
		try {

			return InsideConverterVersion.versionesInsideToXml(service
					.getVersionesExpediente(identificador));

		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}

	@Override
	@Secured("ROLE_ALTA_DOCUMENTO")
	public DocumentoInsideInfo altaDocumentoOriginal(WSCredentialInside info,
			TipoDocumentoAltaInside documentoAlta) throws InsideWSException {
		try {

			ObjetoDocumentoInside documentoInside = this
					.altaDocumento(
							documentoAlta,
							ObjetoDocumentoInsideMetadatosEstadoElaboracionEnumeracion.EE_01,
							null);

			DocumentoInsideInfo documentoInfo = InsideConverterDocumento
					.documentoInsideToDocumentoInfo(documentoInside);

			logger.info("Dado de alta Documento original con identificador "
					+ documentoInside.getIdentificador());
			return documentoInfo;

		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}

	}

	@Override
	@Secured("ROLE_ALTA_DOCUMENTO")
	public DocumentoInsideInfo altaDocumentoDigitalizado(
			WSCredentialInside info, TipoDocumentoAltaInside documentoAlta)
			throws InsideWSException {
		try {

			ObjetoDocumentoInside documentoInside = this
					.altaDocumento(
							documentoAlta,
							ObjetoDocumentoInsideMetadatosEstadoElaboracionEnumeracion.EE_03,
							null);

			DocumentoInsideInfo documentoInfo = InsideConverterDocumento
					.documentoInsideToDocumentoInfo(documentoInside);

			logger.info("Dado de alta Documento digitalizado con identificador "
					+ documentoInside.getIdentificador());
			return documentoInfo;

		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}

	@Override
	@Secured("ROLE_ALTA_DOCUMENTO")
	public DocumentoInsideInfo altaDocumentoCopiaParcial(
			WSCredentialInside info, TipoDocumentoAltaInside documentoAlta,
			String identificadorDocumentoOrigen) throws InsideWSException {
		try {

			ObjetoDocumentoInside documentoInside = this
					.altaDocumento(
							documentoAlta,
							ObjetoDocumentoInsideMetadatosEstadoElaboracionEnumeracion.EE_04,
							identificadorDocumentoOrigen);

			DocumentoInsideInfo documentoInfo = InsideConverterDocumento
					.documentoInsideToDocumentoInfo(documentoInside);

			logger.info("Dado de alta Documento con identificador "
					+ documentoInside.getIdentificador()
					+ " proviniente de copia parcial de documento origen con identificador "
					+ identificadorDocumentoOrigen);
			return documentoInfo;

		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}

	@Override
	@Secured("ROLE_ALTA_DOCUMENTO")
	public DocumentoInsideInfo altaDocumentoCambioFormato(
			WSCredentialInside info, TipoDocumentoAltaInside documentoAlta,
			String identificadorDocumentoOrigen) throws InsideWSException {
		try {

			ObjetoDocumentoInside documentoInside = this
					.altaDocumento(
							documentoAlta,
							ObjetoDocumentoInsideMetadatosEstadoElaboracionEnumeracion.EE_02,
							identificadorDocumentoOrigen);

			DocumentoInsideInfo documentoInfo = InsideConverterDocumento
					.documentoInsideToDocumentoInfo(documentoInside);

			logger.info("Dado de alta Documento con identificador "
					+ documentoInside.getIdentificador()
					+ " proviniente de cambio de formato de documento origen con identificador "
					+ identificadorDocumentoOrigen);
			return documentoInfo;

		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}

	@Override
	@Secured("ROLE_ALTA_DOCUMENTO")
	public DocumentoInsideInfo altaDocumentoEni(WSCredentialInside info,
			TipoDocumento documentoEni,
			TipoDocumentoAsociadoaExpediente asociadoAExpediente,
			TipoMetadatosAdicionales metadatosAdicionales, boolean firmaServidor)
			throws InsideWSException {
		try {
			checkVersionNTI(documentoEni.getMetadatos());

			logger.debug("Convirtiendo el documentoEni a Inside");

			ObjetoDocumentoInside documento = InsideConverterDocumento
					.documentoEniAndMetadatosToDocumentoInside(documentoEni,
							metadatosAdicionales);

			if (firmaServidor) {
				logger.debug("Se firmará en servidor el documento");
				documento = service.firmaServidorDocumento(documento);
			}
			if (asociadoAExpediente != null
					&& StringUtils.hasText(asociadoAExpediente
							.getIdentificadorExpediente())) {
				documento = service.altaDocumento(documento,
						asociadoAExpediente.getIdentificadorExpediente(),
						asociadoAExpediente.getIdentificadorCarpeta());
			} else {
				documento = service.altaDocumento(documento);
			}

			logger.debug("Convirtiendo el documentoInside a InfoInside");
			DocumentoInsideInfo documentoInfo = InsideConverterDocumento
					.documentoInsideToDocumentoInfo(documento);

			logger.info("Importado Documento desde ENI con identificador "
					+ documento.getIdentificador());

			return documentoInfo;

		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}

	@Override
	@Secured("ROLE_MODIFICA_DOCUMENTO")
	public DocumentoInsideInfo modificaDocumento(WSCredentialInside info,
			TipoDocumento documentoEni,
			TipoMetadatosAdicionales metadatosAdicionales)
			throws InsideWSException {
		try {
			checkVersionNTI(documentoEni.getMetadatos());

			logger.debug("Convirtiendo el documentoEni a Inside");
			ObjetoDocumentoInside documento = InsideConverterDocumento
					.documentoEniAndMetadatosToDocumentoInside(documentoEni,
							metadatosAdicionales);

			documento = service.modificaDocumento(documento);

			logger.debug("Convirtiendo el documentoInside a InfoInside");
			DocumentoInsideInfo documentoInfo = InsideConverterDocumento
					.documentoInsideToDocumentoInfo(documento);

			logger.info("Modificado Documento ENI con identificador "
					+ documento.getIdentificador());
			return documentoInfo;
		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}

	@Override
	@Secured("ROLE_LEER_DOCUMENTO")
	public TipoDocumentoInside getDocumento(WSCredentialInside info,
			String identificador, Integer version) throws InsideWSException {
		try {

			ObjetoDocumentoInside documento = service.getDocumento(
					identificador, version);

			TipoDocumentoInside documentoXml = InsideConverterDocumento
					.documentoInsideToXml(documento);

			logger.info("Devuelto Documento con identificador "
					+ documento.getIdentificador());

			return documentoXml;

		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}

	@Override
	@Secured("ROLE_LEER_DOCUMENTO")
	public TipoDocumento getDocumentoEni(WSCredentialInside info,
			String identificador) throws InsideWSException {
		try {

			ObjetoDocumentoInside documento = service
					.getDocumento(identificador);

			TipoDocumento documentoEni = InsideConverterDocumento
					.documentoInsideToEni(documento);

			logger.info("Devuelto Documento ENI con identificador "
					+ documento.getIdentificador());

			return documentoEni;

		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}

	@Override
	@Secured("ROLE_LEER_DOCUMENTO")
	public TipoContenido getDocumentoContenido(WSCredentialInside info,
			String identificador, Integer version) throws InsideWSException {
		try {

			ObjetoDocumentoInsideContenido contenidoInside = service
					.getDocumentoContenido(identificador, version);

			TipoContenido contenido = InsideConverterDocumento.Contenido
					.contenidoInsideToEni(contenidoInside);

			logger.info("Devuelto Contenido de Documento ENI con identificador "
					+ identificador);

			return contenido;

		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}

	@Override
	@Secured("ROLE_LEER_DOCUMENTO")
	public DocumentoInsideMetadatos getMetadatosDocumento(
			WSCredentialInside info, String identificador, Integer version)
			throws InsideWSException {
		try {

			ObjetoDocumentoInsideMetadatos metadatosInside = service
					.getMetadatosDocumento(identificador, version);

			DocumentoInsideMetadatos metadatos = InsideConverterMetadatos.Documento
					.metadatosInsideToXml(metadatosInside);

			logger.info("Devueltos Metadatos de Documento ENI con identificador "
					+ identificador);

			return metadatos;

		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}

	@Override
	@Secured("ROLE_LEER_DOCUMENTO")
	public List<TipoVersionInside> getVersionesDocumento(
			WSCredentialInside info, String identificador)
			throws InsideWSException {
		try {

			return InsideConverterVersion.versionesInsideToXml(service
					.getVersionesDocumento(identificador));

		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}

	@Override
	@Secured("ROLE_LEER_EXPEDIENTE")
	public TipoContenido getFicheroFoliado(WSCredentialInside info,
			String identificador) throws InsideWSException {
		try {

			FoliadoInside foliadoInside = service
					.getFoliadoExpediente(identificador);

			return InsideConverterFoliado
					.contenidoFoliadoInsideToEni(foliadoInside);

			/*
			 * ObjetoDocumentoInside documento =
			 * service.getFoliadoExpediente(identificador);
			 * 
			 * TipoContenido contenido =
			 * InsideConverterDocumento.Contenido.contenidoInsideToEni
			 * (documento.getContenido());
			 * 
			 * return contenido;
			 */

		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}

	@Override
	@Secured("ROLE_LEER_EXPEDIENTE")
	public ExpedienteResultadoBusqueda buscarExpedientes(
			WSCredentialInside info, ConsultaWsInside consulta, int limite,
			int pagina) throws InsideWSException {
		try {
			ResultadoBusquedaInside<ObjetoExpedienteInsideMetadatos> res = service
					.buscarExpedientes(InsideConverterBusqueda
							.consultaXmlToInside(consulta), limite, pagina);
			return InsideConverterBusquedaResultado
					.respuestaBusquedaExpedienteInsideToXml(res);
		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}

	}

	@Override
	@Secured("ROLE_LEER_DOCUMENTO")
	public DocumentoResultadoBusqueda buscarDocumentos(WSCredentialInside info,
			ConsultaWsInside consulta, int limite, int pagina)
			throws InsideWSException {
		try {
			ResultadoBusquedaInside<ObjetoDocumentoInsideMetadatos> res = service
					.buscarDocumentos(InsideConverterBusqueda
							.consultaXmlToInside(consulta), limite, pagina);
			return InsideConverterBusquedaResultado
					.respuestaBusquedaDocumentoInsideToXml(res);
		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}

	@Override
	public TipoDocumento convertirDocumentoAEni(WSCredentialInside info,
			TipoDocumentoConversionInside documentoConversion)
			throws InsideWSException {
		try {
			logger.debug("Validando documentoConversión a nivel de WS");
			 
			documentoConversion = validationService.validaTipoDocumentoConversionInside(documentoConversion, true);
			
			logger.debug("Convirtiendo el documentoConversion a ENI");
			ObjetoDocumentoInside documentoInside = InsideConverterDocumento.documentoConversionToInside(documentoConversion, infofirmaService);
			
			logger.debug("Validando documento Inside a nivel de Inside");
			documentoInside = service.validarDocumento(documentoInside);
			
			TipoDocumento documentoEni = InsideConverterDocumento.documentoInsideToEni(documentoInside);
			
			logger.info("Convertido Documento de entrada a Eni " + documentoInside.getIdentificador());
			return documentoEni;
		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}

	}

	@Override
	public TipoExpediente convertirExpedienteAEni(WSCredentialInside info,
			TipoExpedienteConversionInside expedienteConversion)
			throws InsideWSException {
		try {
			logger.debug("Validando expedienteConversion a nivel de WS");
			expedienteConversion = validationService.validaTipoExpedienteConversionInside(expedienteConversion, true);
			
			logger.debug("Convirtiendo expedienteConversion a Inside");
			ObjetoExpedienteInside expedienteInside = InsideConverterExpediente.expedienteConversionToInside (expedienteConversion);
			
			logger.debug("Validando expediente Inside");
			expedienteInside = service.validarExpediente(expedienteInside);
			
			logger.debug("Obteniendo Visualización del Índice");
			expedienteInside = service.obtenerVisualizacionIndice(expedienteInside, 
																  expedienteConversion.getOpcionesVisualizacion().isEstamparImagen(),
																  expedienteConversion.getOpcionesVisualizacion().isEstamparNombreOrganismo(),
										                          expedienteConversion.getOpcionesVisualizacion().getFilasNombreOrganismo() != null ? expedienteConversion.getOpcionesVisualizacion().getFilasNombreOrganismo().getFila() : null,
										                          true);
						
			
			logger.debug("Convirtiendo expediente de Inside a ENI");
			
			return InsideConverterExpediente.expedienteInsideToEni(expedienteInside);
			

		} catch (Exception e) {
			throw InsideExceptionConverter.convertToException(e);
		}
	}

	private ObjetoDocumentoInside altaDocumento(
			TipoDocumentoAltaInside documentoAlta,
			ObjetoDocumentoInsideMetadatosEstadoElaboracionEnumeracion estadoElaboracion,
			String identificadorDocumentoOrigen)
			throws InsideConverterDocumentoEniToInsideException,
			InSideServiceException {
		logger.debug("Convirtiendo el documentoAlta a Inside");
		ObjetoDocumentoInside documentoInside = InsideConverterDocumento
				.documentoAltaXmlToInside(documentoAlta, infofirmaService);

		ObjetoDocumentoInsideMetadatosEstadoElaboracion estadoElaboracionCambioFormato = new ObjetoDocumentoInsideMetadatosEstadoElaboracion();
		if (identificadorDocumentoOrigen != null) {
			estadoElaboracionCambioFormato
					.setIdentificadorDocumentoOrigen(identificadorDocumentoOrigen);
		}
		estadoElaboracionCambioFormato
				.setValorEstadoElaboracion(estadoElaboracion);
		documentoInside.getMetadatos().setEstadoElaboracion(
				estadoElaboracionCambioFormato);
		documentoInside.getMetadatos().setVersionNTI(VERSION_NTI);

		if (documentoAlta.isFirmadoEnServidor()) {
			logger.debug("Se firmará en servidor el documento");
			documentoInside = service.firmaServidorDocumento(documentoInside);
		}

		if (documentoAlta.getAsociadoaExpediente() != null
				&& StringUtils.hasText(documentoAlta.getAsociadoaExpediente()
						.getIdentificadorExpediente())) {
			logger.debug("Llamando a altaDocumento asociado a Expediente "
					+ documentoAlta.getAsociadoaExpediente()
							.getIdentificadorExpediente());
			String carpeta = documentoAlta.getAsociadoaExpediente()
					.getIdentificadorCarpeta() != null ? documentoAlta
					.getAsociadoaExpediente().getIdentificadorCarpeta() : "";
			documentoInside = service.altaDocumento(documentoInside,
					documentoAlta.getAsociadoaExpediente()
							.getIdentificadorExpediente(), carpeta);
		} else {
			logger.debug("Llamando a altaDocumento");
			documentoInside = service.altaDocumento(documentoInside);
		}

		return documentoInside;

	}

	/**
	 * 
	 * @param expedienteEni
	 * @throws InsideWSException
	 */
	private void validaIdentificador(TipoExpediente expedienteEni)
			throws InsideWSException {

		if (expedienteEni == null) {
			throw new InsideWSException(InsideWsErrors.EXPEDIENTE_ENI_VACIO);
		}
		if (expedienteEni.getMetadatosExp() == null) {
			throw new InsideWSException(
					InsideWsErrors.METADATOS_EXPEDIENTE_ENI_VACIO);
		}

		if (expedienteEni.getMetadatosExp().getIdentificador() == null) {
			throw new InsideWSException(
					InsideWsErrors.IDENTIFICADOR_EXPEDIENTE_ENI_VACIO);
		}

	}

	private void checkVersionNTI(TipoMetadatos metadatosEni)
			throws InsideWSException {
		metadatosEni
				.setVersionNTI(checkVersionNTI(metadatosEni.getVersionNTI()));
	}

	private void checkVersionNTI(
			es.mpt.dsic.inside.xml.eni.documento.metadatos.TipoMetadatos metadatosEni)
			throws InsideWSException {
		metadatosEni
				.setVersionNTI(checkVersionNTI(metadatosEni.getVersionNTI()));
	}

	/**
	 * Nos aseguramos que el valor del metadato versionNTI siempre sea la 1.0
	 * (de momento no soportamos otra cosa)
	 * 
	 * @param versionNTI
	 * @return
	 * @throws InsideWSException
	 */
	private String checkVersionNTI(String versionNTI) throws InsideWSException {
		if (versionNTI != null && !versionNTI.equalsIgnoreCase(VERSION_NTI)) {
			throw new InsideWSException(InsideWsErrors.VERSION_NTI_10_ERRONEA);
		}
		return VERSION_NTI;
	}

	/*
	 * private InsideWSException convertToException(Exception e) {
	 * logger.info("Error de tipo " + e.getClass().getSimpleName() +
	 * " procesando petición en Inside : " + e.getMessage(),e);
	 * //result.printStackTrace();
	 * 
	 * InsideWSException result = null; if(e instanceof InsideWSException){
	 * result = (InsideWSException) e; }else if(e instanceof
	 * InSideServiceException){ result = new
	 * InsideWSException((InSideServiceException)e); }else if (e instanceof
	 * InsideValidationDataException) { result = new
	 * InsideWSException((InsideValidationDataException)e); }else{ result = new
	 * InsideWSException(InsideWsErrors.INTERNAL_SERVICE_ERROR,e);
	 * logger.error(e); }
	 * 
	 * return result; }
	 */

}
